/* Denise.c - by Peter Drapich docent@union.pl */

#include "denise.h"
#include "DeniseGUI_protos.h"

int HandleQuit(struct DeniseHandle*, int, int,char *);
int ShowHelp(struct DeniseHandle*, int, int,char *);
int ShowAbout(struct DeniseHandle*, int, int,char *);
int ShowFirst(struct DeniseHandle*, int, int,char *);
int ShowStatus(struct DeniseHandle*, int, int,char *);
int SetDeniseCmd(struct DeniseHandle*, int, int,char *);
int SetDenise(struct DeniseHandle*, int, int,char *);
int GetDenise(struct DeniseHandle*, int, int,char *);
int ToggleDenise(struct DeniseHandle*, int, int,char *);
int LoadDenise(struct DeniseHandle*, int, int,char *);
int SaveDenise(struct DeniseHandle*, int, int,char *);
int WordEditor(struct DeniseHandle*, int, int,char *);

int DeniseCmd;
struct DeniseHandle* handle=NULL;

struct UNN_StateSwitch
{
	struct UNN_StateTransition* Transition;			// pointer to first element in a transition table
	int TransTableSize;								// number of entries in this transition table
};

struct UNN_StateTransition
{
	char Cmd[20];									// cmd which matches this state change
	int	(*StateAction)(struct DeniseHandle*,int,int,char *);	// action to execute for thi state change
	int  UserParam;									// additional parm
	int  DestState;									// state to switch to after execution
};


// te stany musza byc jeden za drugim, bo sa jednoczesnie offsetami w tabeli
#define DENISE_STATE_START	0
#define DENISE_STATE_SET	1
#define DENISE_STATE_GET	2
#define DENISE_STATE_ONOFF	3
#define DENISE_STATE_LOAD	4
#define DENISE_STATE_SAVE	5
#define DENISE_STATE_WED	6

//uzywac ostroznie! nie stanowia offsetow w tabeli!
#define DENISE_STATE_END	10
#define DENISE_STATE_HELP	20


struct UNN_StateTransition starttable[] = 
{
	{"set",		NULL,		0,	DENISE_STATE_SET},
	{"get",		NULL,		0,	DENISE_STATE_GET},
	{"help",	NULL,		0,	DENISE_STATE_HELP},
	{"quit",	HandleQuit,	0,	DENISE_STATE_END},
	{"about",	ShowAbout,	0,	DENISE_STATE_END},
	{"status",	ShowStatus,	0,	DENISE_STATE_END},
	{"load",	NULL,		0,	DENISE_STATE_LOAD},
	{"save",	NULL,		0,	DENISE_STATE_SAVE},
	{"add",		NULL,		0,	DENISE_STATE_WED}
};

struct UNN_StateTransition wedtable[] =
{
	{"word",	WordEditor,	DENISE_CMD_ADDWORD,	DENISE_STATE_END},
	{"inflexion",	WordEditor,	DENISE_CMD_ADDINFL,	DENISE_STATE_END}
};

struct UNN_StateTransition settable[] = 
{
	{"minwords",			SetDeniseCmd,	DENISE_CMD_MINWORDS,				DENISE_STATE_END},
	{"mingroups",			SetDeniseCmd,	DENISE_CMD_MINGROUPS,			DENISE_STATE_END},
	{"maxgroups",			SetDeniseCmd,	DENISE_CMD_MAXGROUPS,			DENISE_STATE_END},
	{"maxfleks",			SetDeniseCmd,	DENISE_CMD_MAXFLEKS,				DENISE_STATE_END},
	{"maxwords",			SetDeniseCmd,	DENISE_CMD_MAXWORDS,				DENISE_STATE_END},
	{"maxclauses",			SetDeniseCmd,	DENISE_CMD_MAXCLAUSES,			DENISE_STATE_END},
	{"firstclause",		SetDeniseCmd,	DENISE_CMD_FIRSTCLAUSE, 		DENISE_STATE_ONOFF},
	{"subjectfirst",		SetDeniseCmd,	DENISE_CMD_SUBJECTFIRST, 		DENISE_STATE_ONOFF},
	{"minprefixlen",		SetDeniseCmd,	DENISE_CMD_MINPREFIXLEN,		DENISE_STATE_END},
	{"prefixcheck",		SetDeniseCmd,	DENISE_CMD_PREFIXCHECK,			DENISE_STATE_ONOFF},
	{"prefixcheckmode",	SetDeniseCmd,	DENISE_CMD_PREFIXCHECKMODE,	DENISE_STATE_END},
	{"rememberfirst",		SetDeniseCmd,	DENISE_CMD_REMEMBERFIRST,		DENISE_STATE_ONOFF},
	{"phrasequality",		SetDeniseCmd,	DENISE_CMD_PHRASEQUALITY,		DENISE_STATE_END},
	{"structurequality",	SetDeniseCmd,	DENISE_CMD_STRUCTUREQUALITY,	DENISE_STATE_END},
	{"functionquality",	SetDeniseCmd,	DENISE_CMD_FUNCTIONQUALITY,	DENISE_STATE_END},
	{"viewwords",			SetDeniseCmd,	DENISE_CMD_VIEWWORDS,			DENISE_STATE_ONOFF},
	{"viewclause",			SetDeniseCmd,	DENISE_CMD_VIEWCLAUSE,			DENISE_STATE_ONOFF},
	{"viewsyntax",			SetDeniseCmd,	DENISE_CMD_VIEWSYNTAX,			DENISE_STATE_ONOFF},
	{"viewsyntaxtime",	SetDeniseCmd,	DENISE_CMD_VIEWSYNTAXTIME,		DENISE_STATE_ONOFF},
	{"viewpreliminary",	SetDeniseCmd,	DENISE_CMD_VIEWPRELIMINARY,	DENISE_STATE_ONOFF},
	{"viewtheme",			SetDeniseCmd,	DENISE_CMD_VIEWTHEME,			DENISE_STATE_ONOFF},
	{"viewinterlocutory",SetDeniseCmd,	DENISE_CMD_VIEWINTERLOCUTORY,	DENISE_STATE_ONOFF},
	{"viewlogic",			SetDeniseCmd,	DENISE_CMD_VIEWLOGIC,			DENISE_STATE_ONOFF},
	{"viewfact",			SetDeniseCmd,	DENISE_CMD_VIEWFACT,				DENISE_STATE_ONOFF},
	{"viewunknownwords",	SetDeniseCmd,	DENISE_CMD_VIEWUNKNOWNWORDS,	DENISE_STATE_ONOFF},
	{"viewconjunctions",	SetDeniseCmd,	DENISE_CMD_VIEWCONJUNCTIONSANALYSIS,	DENISE_STATE_ONOFF},
	{"logwords",			SetDeniseCmd,	DENISE_CMD_LOGWORDS,				DENISE_STATE_ONOFF},
	{"logclauses",			SetDeniseCmd,	DENISE_CMD_LOGCLAUSES, 			DENISE_STATE_ONOFF},
	{"logdialog",			SetDeniseCmd,	DENISE_CMD_LOGDIALOG, 			DENISE_STATE_ONOFF},
	{"logreports",			SetDeniseCmd,	DENISE_CMD_LOGREPORTS, 			DENISE_STATE_ONOFF},
	{"supervisor",			SetDeniseCmd,	DENISE_CMD_SUPERVISOR,			DENISE_STATE_ONOFF},
	{"language",			SetDeniseCmd,	DENISE_CMD_LANGUAGE,				DENISE_STATE_END}
};

struct UNN_StateTransition gettable[] =
{
	{"minwords",			GetDenise,	DENISE_CMD_MINWORDS,				DENISE_STATE_END},
	{"mingroups",			GetDenise,	DENISE_CMD_MINGROUPS,			DENISE_STATE_END},
	{"maxgroups",			GetDenise,	DENISE_CMD_MAXGROUPS,			DENISE_STATE_END},
	{"maxfleks",			GetDenise,	DENISE_CMD_MAXFLEKS,				DENISE_STATE_END},
	{"maxwords",			GetDenise,	DENISE_CMD_MAXWORDS,				DENISE_STATE_END},
	{"maxclauses",			GetDenise,	DENISE_CMD_MAXCLAUSES,			DENISE_STATE_END},
	{"firstclause",		GetDenise,	DENISE_CMD_FIRSTCLAUSE,			DENISE_STATE_END},
	{"subjectfirst",		GetDenise,	DENISE_CMD_SUBJECTFIRST,		DENISE_STATE_END},
	{"minprefixlen",		GetDenise,	DENISE_CMD_MINPREFIXLEN,		DENISE_STATE_END},
	{"prefixcheck",		GetDenise,	DENISE_CMD_PREFIXCHECK,			DENISE_STATE_END},
	{"prefixcheckmode",	GetDenise,	DENISE_CMD_PREFIXCHECKMODE,	DENISE_STATE_END},
	{"rememberfirst",		GetDenise,	DENISE_CMD_REMEMBERFIRST,		DENISE_STATE_END},
	{"phrasequality",		GetDenise,	DENISE_CMD_MINPREFIXLEN,		DENISE_STATE_END},
	{"structurequality",	GetDenise,	DENISE_CMD_STRUCTUREQUALITY,	DENISE_STATE_END},
	{"functionquality",	GetDenise,	DENISE_CMD_FUNCTIONQUALITY,	DENISE_STATE_END},
	{"viewwords",			GetDenise,	DENISE_CMD_VIEWWORDS,			DENISE_STATE_END},
	{"viewclause",			GetDenise,	DENISE_CMD_VIEWCLAUSE,			DENISE_STATE_END},
	{"viewsyntax",			GetDenise,	DENISE_CMD_VIEWSYNTAX,			DENISE_STATE_END},
	{"viewsyntaxtime",	GetDenise,	DENISE_CMD_VIEWSYNTAXTIME,		DENISE_STATE_END},
	{"viewpreliminary",	GetDenise,	DENISE_CMD_VIEWPRELIMINARY,	DENISE_STATE_END},
	{"viewtheme",			GetDenise,	DENISE_CMD_VIEWTHEME,			DENISE_STATE_END},
	{"viewinterlocutory",GetDenise,	DENISE_CMD_VIEWINTERLOCUTORY,	DENISE_STATE_END},
	{"viewlogic",			GetDenise,	DENISE_CMD_VIEWLOGIC,			DENISE_STATE_END},
	{"viewfact",			GetDenise,	DENISE_CMD_VIEWFACT,				DENISE_STATE_END},
	{"viewunknownwords",	GetDenise,	DENISE_CMD_VIEWUNKNOWNWORDS,	DENISE_STATE_END},
	{"viewconjunctions",	GetDenise,	DENISE_CMD_VIEWCONJUNCTIONSANALYSIS,	DENISE_STATE_END},
	{"logwords",			GetDenise,	DENISE_CMD_LOGWORDS,				DENISE_STATE_END},
	{"logclauses",			GetDenise,	DENISE_CMD_LOGCLAUSES, 			DENISE_STATE_END},
	{"logdialog",			GetDenise,	DENISE_CMD_LOGDIALOG, 			DENISE_STATE_END},
	{"logreports",			GetDenise,	DENISE_CMD_LOGREPORTS, 			DENISE_STATE_END},
	{"supervisor",			GetDenise,	DENISE_CMD_SUPERVISOR,			DENISE_STATE_END},
	{"language",			GetDenise,	DENISE_CMD_LANGUAGE,				DENISE_STATE_END}
};

struct UNN_StateTransition onofftable[] =
{
	{"on",				ToggleDenise,	DENISE_CMD_ON,						DENISE_STATE_END},
	{"off",				ToggleDenise,	DENISE_CMD_OFF,					DENISE_STATE_END}
};

struct UNN_StateTransition loadtable[] =
{
	{"config",			LoadDenise,	DENISE_CMD_LOADCONFIG,			DENISE_STATE_END}
};

struct UNN_StateTransition savetable[] =
{
	{"config",			SaveDenise,	DENISE_CMD_SAVECONFIG,			DENISE_STATE_END}
};

struct UNN_StateSwitch switchtable[] =
{
	{starttable, sizeof(starttable)/sizeof(struct UNN_StateTransition)},
	{settable, sizeof(settable)/sizeof(struct UNN_StateTransition)},
	{gettable, sizeof(gettable)/sizeof(struct UNN_StateTransition)},
	{onofftable, sizeof(onofftable)/sizeof(struct UNN_StateTransition)},
	{loadtable, sizeof(loadtable)/sizeof(struct UNN_StateTransition)},
	{savetable, sizeof(savetable)/sizeof(struct UNN_StateTransition)},
	{wedtable, sizeof(wedtable)/sizeof(struct UNN_StateTransition)},
};

int WordEditor(struct DeniseHandle* handle, int cmd, int state, char *param) {
	if(cmd==DENISE_CMD_ADDWORD)
		wed_add_word(handle);
	if(cmd==DENISE_CMD_ADDINFL)
		wed_add_inflexion(handle);
	return OK;
}

int HandleQuit(struct DeniseHandle* handle, int cmd, int state,char *param)
{
	return -1;
}

int ShowAbout(struct DeniseHandle* handle, int cmd, int state,char *param)
{
	com_about(handle);
	return OK;
}

int ShowStatus(struct DeniseHandle* handle, int cmd, int state,char *param)
{
	com_status(handle);
	return OK;
}

int ShowFirst(struct DeniseHandle* handle, int cmd, int state,char *param)
{
	return OK;
}

int SetDenise(struct DeniseHandle* handle, int cmd, int state,char *param)
{
	if (handle)
	{
		DeniseSet(handle,cmd,NULL);
	}
	return OK;
}

int GetDenise(struct DeniseHandle* handle, int cmd, int state,char *param)
{
	if (handle)
	{
		DeniseGet(handle,cmd,NULL);
	}
	return OK;
}

int ToggleDenise(struct DeniseHandle* handle, int cmd, int state,char *param)
{
	if (handle)
	{
		DeniseSet(handle,DeniseCmd,(void*)cmd);
	}
	return OK;
}

int LoadDenise(struct DeniseHandle* handle, int cmd, int state,char *param)
{
	if (handle)
	{
		DeniseLoad(handle,cmd,(void*)param);
	}
	return OK;
}

int SaveDenise(struct DeniseHandle* handle, int cmd, int state,char *param)
{
	if (handle)
	{
		DeniseSave(handle,cmd,(void*)param);
	}
	return OK;
}

int SetDeniseCmd(struct DeniseHandle* handle,int cmd, int state,char *param)
{
	DeniseCmd=cmd;
	if (handle)
	{
		DeniseSet(handle,DeniseCmd,(void*)param);
	}
//DeniseReport(handle,"cmd: %d, state: %d, param: %s\n",cmd,state,param);
	return OK;
}

int ShowHelp(struct DeniseHandle* handle, int cmd, int state,char *param)
{
	int cnt,transsize;
	struct UNN_StateTransition* transition,*transtable;

	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_AVAILABLE_COMMANDS,"Available commands:"));
	transtable=switchtable[state].Transition;
	transsize=switchtable[state].TransTableSize;
	for (cnt=0;cnt<transsize;cnt++)
	{
		transition=&transtable[cnt];
		DeniseReport(handle,"%s",transition->Cmd);
	}
	return OK;
}

int HandleDeniseCmd(struct DeniseHandle* handle, char* cmd)
{
	char* token,*lasttoken;
	char endtoken[MAX_LINE_LENGHT];
	int cnt,retcode;
	int state,transsize,help;
	struct UNN_StateTransition* transition,*transtable;

//	DeniseReport(handle,"cmd: %s",cmd);
	help=0;
	state=DENISE_STATE_START;
	token=strrchr(cmd,' ');
	if (token==NULL) endtoken[0]='\0';
	else strcpy(endtoken,token+1);
	token=NULL;

	lasttoken=token=strtok(cmd," ");
	while (token)
	{
		transition=NULL;
		transtable=switchtable[state].Transition;
		transsize=switchtable[state].TransTableSize;

		for (cnt=0;cnt<transsize;cnt++)
		{
			transition=&transtable[cnt];
			if (!strcmp(token,transition->Cmd))
			{

				// this handles help command, which displays available option for keywords
				if (!help)
				{
					if (transition->StateAction) retcode=(transition->StateAction)(handle,transition->UserParam,state,endtoken);
					if (retcode==-1) return retcode;	// quit hack
				}

				state=transition->DestState;
				//hadles help state
				if (state==DENISE_STATE_HELP)
				{
					state=DENISE_STATE_START;
					help=1;
				}
				break;
			}
		}
		if (state==DENISE_STATE_END) return DONE;		// end of processing reached
		lasttoken=token;
		token=strtok(NULL," ");
	}
	if (help)
	{
		ShowHelp(handle,0,state,NULL);
		return DONE;
	}
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_SYNTAX_ERROR_AT,"syntax error at %s"),lasttoken);
	return FAIL;
}


#ifdef WIN32
HINSTANCE DeniseInstance;

APIENTRY WinMain( HINSTANCE hInst, HINSTANCE hPrev, LPSTR lpCmd,int nShow )
{
	struct DeniseAppHandle* apphandle;
	DeniseInstance=hInst;
#else
void main(void)
{
	struct DeniseAppHandle* apphandle;
#endif


	if (apphandle=InitApp("Denise-MYTH Artificial Intelligence - GUI-Developer version"))
	{
		HandleDeniseEvents(apphandle);
		CloseApp(apphandle,0);
	}
}
