#ifndef UNN_SYSTEM
#define UNN_SYSTEM

#include "denise.h"
//#include "UNN_System.h"
#include "errors.h"

char* stristr(char *string,char *strCharSet)
{
	char *first,*curr,*currchs;
	for (first=string,curr=string,currchs=strCharSet;*curr&&*currchs;curr++)
		if(!strnicmp(curr,currchs,1)) currchs++;
		else currchs=strCharSet,first=currchs+1;
	return *currchs?NULL:first;
}

/* Lists and nodes */

void UNN_NewList(struct List *list)
{
	list->lh_Head=(struct Node *)&list->lh_Tail;
	list->lh_Tail=NULL;
	list->lh_TailPred=(struct Node *)list;
}

void UNN_AddHead(struct List *list,struct Node *node)
{
	node->ln_Succ=list->lh_Head;
	node->ln_Pred=list->lh_Head->ln_Pred;
	list->lh_Head->ln_Pred=node;
	list->lh_Head=node;
}

void UNN_AddTail(struct List *list,struct Node *node)
{
	node->ln_Succ=(struct Node *)&list->lh_Tail;
	node->ln_Pred=list->lh_TailPred;
	list->lh_TailPred->ln_Succ=node;
	list->lh_TailPred=node;
}

struct Node* UNN_RemHead(struct List *list)
{
	struct Node *node;

	node=list->lh_Head;
	list->lh_Head->ln_Succ->ln_Pred=list->lh_Head->ln_Pred;
	list->lh_Head=list->lh_Head->ln_Succ;
	return node;
}

struct Node* UNN_RemTail(struct List *list)
{
	struct Node *node;

	if(UNN_IsListEmpty(list)) return FAIL;
	node=list->lh_TailPred;
	node->ln_Pred->ln_Succ=(struct Node *)&list->lh_Tail;
	list->lh_TailPred=node->ln_Pred;
	return node;
}

void UNN_Remove(struct Node *node)
{
	node->ln_Succ->ln_Pred=node->ln_Pred;
	node->ln_Pred->ln_Succ=node->ln_Succ;
}


void UNN_SwapNodes(struct Node*src,struct Node* dest)
{

	struct Node* srcpred=src->ln_Pred;
	struct Node* srcsucc=src->ln_Succ;
	struct Node* destpred=dest->ln_Pred;
	struct Node* destsucc=dest->ln_Succ;

	src->ln_Succ=destsucc==src?dest:destsucc;
	src->ln_Pred=destpred==src?dest:destpred;
	dest->ln_Succ=srcsucc==dest?src:srcsucc;
	dest->ln_Pred=srcpred==dest?src:srcpred;

	src->ln_Succ->ln_Pred=src;
	src->ln_Pred->ln_Succ=src;
	dest->ln_Succ->ln_Pred=dest;
	dest->ln_Pred->ln_Succ=dest;
}

struct List* UNN_MergeLists(struct List* list,struct List* list2)
{
	if(UNN_IsListEmpty(list2)) return list;
// ustawic nastepce ostatniego nodu 1 listy na 1 node 2 listy
	list->lh_TailPred->ln_Succ=list2->lh_Head;
// ustawic poprzednika 1 nodu 2 listy na ostatni node 1 listy
	list2->lh_Head->ln_Pred=list->lh_TailPred;
// ustawic nastepce ostatniego nodu 2 listy na 1 liste
	list2->lh_TailPred->ln_Succ=(struct Node *)&list->lh_Tail;
// ustawic pointer do ostatniego nodu w 1 liscie
	list->lh_TailPred=list2->lh_TailPred;

	UNN_NewList(list2);
	return list;
}

struct List* UNN_CopyList(struct List* list,struct List* list2)
{
	if(UNN_IsListEmpty(list2)) return list;
	UNN_Memcpy(list,list2, sizeof(struct List));
	list->lh_Head->ln_Pred=list->lh_Head;
	list->lh_TailPred->ln_Succ=list->lh_Tail;
	UNN_NewList(list2);
	return list;
}

void UNN_FreeList(struct List *list)
{
	struct Node *worknode;
	struct Node *nextnode;
	if (list)
	{
		worknode=(struct Node *) (list->lh_Head);
		while ((nextnode=(struct Node*)(worknode->ln_Succ)))
		{
			UNN_FreeMem(worknode);
         worknode=NULL;
			worknode=nextnode;
		}
//	UNN_FreeMem(list);
	}
}

void UNN_FreeLVList(struct List *list)
{
	struct Node *worknode;
	struct Node *nextnode;
	if (list)	
	{
		worknode=(struct Node *) (list->lh_Head);
		while (nextnode=(struct Node*)(worknode->ln_Succ))
		{
			if (worknode->ln_Name) {UNN_FreeMem(worknode->ln_Name);worknode->ln_Name=NULL;}
			UNN_FreeMem(worknode);
			worknode=nextnode;
		}
		UNN_FreeMem(list);
      list=NULL;
	}
}

struct Node* UNN_AllocNode(ULONG size)
{
	return (UNN_AllocMem(size,MEMF_PUBLIC|MEMF_CLEAR));
}

struct List* UNN_AllocListHeader(ULONG size)
{
	return (UNN_AllocMem(size,MEMF_PUBLIC|MEMF_CLEAR));
}

void UNN_FreeNode(struct Node* node)
{
	if (node) {UNN_FreeMem(node);node=NULL;}
}

void UNN_FreeListHeader(struct List* list)
{
	if (list) {UNN_FreeMem(list);list=NULL;}
}

void UNN_FreeMinList(struct MinList *list)
{
	struct MinNode *worknode;
	struct MinNode *nextnode;
	if (list)
	{
		worknode=(struct MinNode*) (list->mlh_Head);
		while ((nextnode=(struct MinNode*)(worknode->mln_Succ)))
		{
			UNN_FreeMem(worknode);		/* zwalnia struct listy*/
			worknode=nextnode;
		}
//	UNN_FreeMem(list);
	}
}

int UNN_CountNodes(struct List *list)
{
	struct Node *worknode;
	struct Node *nextnode;
	int numnodes=0;
	if (list)
	{
		worknode=(struct Node *) (list->lh_Head);
		while ((nextnode=(struct Node*)(worknode->ln_Succ)))
		{
			numnodes++;
			worknode=nextnode;
		}
		return numnodes;
	}
	return -1;
}

/*
char* UNN_Error(long errorcode)
{
	if (errorcode>UNN_MAX_ERRORS) errorcode=UNN_MAX_ERRORS;
	if (errorcode) return(UNN_ErrorTable[errorcode]);
	else return(UNN_ErrorTable[UNN_globalErr]);
}

void UNN_Quit(char* errortext,long errorcode)
{
	UNN_errText[0]='\0';
	if (errortext) strcpy(UNN_errText,errortext);
	if (errorcode!=0) strcat(UNN_errText,UNN_Error(errorcode));
	UNN_Print(UNN_errText);
	UNN_CloseAPI();
	exit(errorcode);
}
*/

ULONG UNN_DosError(FILE* filehandle)
{
	long DosError;
	ULONG UNN_globalErr;
	if (filehandle)
	{
		if (feof(filehandle)) return UNN_globalErr=ERR_FILE_EOF;
		DosError=ferror(filehandle);
	} else DosError=errno;
	switch (DosError)
	{
		case EOF:
			UNN_globalErr=ERR_FILE_EOF;
			break;
		case ENOENT:
			UNN_globalErr=ERR_FILE_NOTFOUND;
			break;
		case EACCES:
			UNN_globalErr=ERR_FILE_ACCESSDENIED;
			break;
		case EIO:
			UNN_globalErr=ERR_FILE_IOERROR;
			break;
		case ENOSPC:
			UNN_globalErr=ERR_FILE_DISKFULL;
			break;
		case 0:
			UNN_globalErr=ERR_NOERR;
			break;
		default:
			UNN_globalErr=ERR_FILE_DEFAULT;
			break;
	}
	return UNN_globalErr;
}

#endif
