// ***************************************************************************************
//                                          LANGUAGE
// ***************************************************************************************

#include "denise.h"
#include "denise_protos.h"

// deklaracje prywatne
static BOOL load_wed(struct DeniseHandle* handle);
static BOOL load_voc(struct DeniseHandle* handle);
static BOOL load_wgn(struct DeniseHandle* handle);
static BOOL load_gr1(struct DeniseHandle* handle);
static BOOL load_gr2(struct DeniseHandle* handle);
static BOOL load_gr3(struct DeniseHandle* handle);
static BOOL load_gr4(struct DeniseHandle* handle);
static BOOL load_gr5(struct DeniseHandle* handle);
static BOOL load_gr6(struct DeniseHandle* handle);
static BOOL load_gr7(struct DeniseHandle* handle);
static BOOL load_lo1(struct DeniseHandle* handle);
static BOOL load_lo2(struct DeniseHandle* handle);
static BOOL load_lo3(struct DeniseHandle* handle);
static BOOL load_lo4(struct DeniseHandle* handle);
static BOOL load_lo5(struct DeniseHandle* handle);
static BOOL load_lo6(struct DeniseHandle* handle);
static BOOL load_lo7(struct DeniseHandle* handle);
static BOOL load_lo8(struct DeniseHandle* handle);
static BOOL load_lo9(struct DeniseHandle* handle);
static BOOL load_fil(struct DeniseHandle* handle);
static void free_odm(struct Odmiany *odm);
static void sort_voc(struct DeniseHandle* handle);
static void free_odm(struct Odmiany *odm);
static void free_fleks(struct DeniseHandle* handle,struct Fleksja *fleks);
static int dane_strncmp(char *dane1,char *dane2,int len);

// ***************************************************************************************
// EXTERN                    BOOL dat_language_load(struct DeniseHandle*)
// ***************************************************************************************
BOOL dat_language_load(struct DeniseHandle* handle)
{
   // funkcja wczytuje wszystkie bazy faktow podane w pliku

   FILE *stream=NULL;
   char temp[100];
   int nr1;

	if (handle->language.name==NULL)
	{
		DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LANGUAGE_NOT_SELECTED,"nie wybrano jezyka"));
		return FALSE;
	}

   handle->language.wordeditor=NULL;
   handle->language.vocabulary=NULL;
   handle->language.voctemp=NULL;
   handle->language.wordgenerator=NULL;
   handle->language.gr1=NULL;
   handle->language.lo1=NULL;
   for (nr1=0;nr1<100;nr1++)
   {
      handle->language.gr3[nr1]=NULL;
      handle->language.gr4[nr1]=NULL;
      handle->language.gr5[nr1]=NULL;
      handle->language.gr7[nr1]=NULL;
   }
   for (nr1=0;nr1<5;nr1++) handle->language.gr6[nr1]=NULL;
   for (nr1=0;nr1<4;nr1++) handle->language.lo2[nr1]=NULL;
   handle->language.lo3[0]=NULL; handle->language.lo3[1]=NULL;
   for (nr1=0;nr1<100;nr1++) handle->language.lo3_voc_filter[nr1]=FALSE;
   handle->language.lo3[2]=NULL; handle->language.lo3[3]=NULL;
   for (nr1=0;nr1<4;nr1++) handle->language.lo4[nr1]=NULL;
   handle->language.lo5=NULL;
   handle->language.lo6=NULL;
   handle->language.lo7=NULL;
   handle->language.lo8=NULL;
   handle->language.lo9=NULL;
   handle->language.fil=NULL;

   sprintf(temp,"%s%s%s.catalog",handle->Prefix,handle->LanguagesDir,handle->language.name);
	if (OpenCatalog(handle,temp)==FALSE) DeniseReport(handle,UNN_GetLocaleString(handle,MSG_FILE_NOT_FOUND,"nie odnaleziono pliku: %s"),temp);

  	if (load_wed(handle)==TRUE && load_voc(handle)==TRUE && load_wgn(handle)==TRUE &&
		load_gr1(handle)==TRUE && load_gr2(handle)==TRUE && load_gr3(handle)==TRUE && 
		load_gr4(handle)==TRUE && load_gr5(handle)==TRUE && load_gr6(handle)==TRUE && 
		load_gr7(handle)==TRUE && load_lo1(handle)==TRUE && load_lo2(handle)==TRUE && 
		load_lo3(handle)==TRUE && load_lo4(handle)==TRUE && load_lo5(handle)==TRUE && 
		load_lo6(handle)==TRUE && load_lo7(handle)==TRUE && load_lo8(handle)==TRUE && 
		load_lo9(handle)==TRUE && load_fil(handle)==TRUE) {
			dat_count_odmiany(handle);
			return TRUE;
	}

	return FAIL;
}

static BOOL load_wed(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:
   
   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   struct WordEditor *wed;

   sprintf(input,"%s%s%s.wed",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   handle->language.wordeditor=NULL;

   for (;;)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         wed=malloc(sizeof(struct WordEditor));
         wed->data=malloc(strlen(complete)+1);
         memcpy(wed->data,complete,strlen(complete)+1);
         if (handle->language.wordeditor==NULL)
         {
            wed->next=NULL;
            handle->language.wordeditor=wed;
         }
         else
         {
            wed->next=handle->language.wordeditor->next;
            handle->language.wordeditor->next=wed;
         }
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.wed",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),input);
   return FAIL;
}

static BOOL load_voc(struct DeniseHandle* handle)
{
   FILE *stream;
   char ile_list,nr1;
   char podst[50],odmiany[1000],dane[1000];
   struct Fleksja *fleks,*last_fleks=NULL;
   struct Odmiany *odm,*last_odm=NULL;
   char sort_please=0;
   char last_char='a';

   sprintf(dane,"%s%s%s.voc",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,dane,"r"))==NULL) goto loading_error;
   handle->language.vocabulary=NULL;
   handle->language.voctemp=NULL;

   if (!stream)
   {
		DeniseReport(handle,UNN_GetLocaleString(handle,MSG_ERROR_OPENING_FILE,"nie moge otworzyc pliku: '%s'"),dane);
      return FAIL;
   }

   for (;;)
   {
      if (glo_fgets(podst,50,stream)==NULL) goto loading_error;
      if (podst[0]=='~') break;
      podst[strlen(podst)-1]='\0'; // kasujemy ENTER

      // sprawdzamy, czy dane sa dobrze posortowane (kontrola tylko pierwszego znaku!)
      if (sort_please==0 && last_char!=podst[0])
      {
         if (glo_what_first(handle,&last_char,&podst[0],1)==2) sort_please=1;
         else last_char=podst[0];
      }

      if (glo_fgets(dane,10,stream)==NULL) goto loading_error;
      if ((ile_list=(char) atoi(dane))<=0) goto loading_error;

      fleks=malloc(sizeof(struct Fleksja));
      fleks->next=NULL;
      fleks->podst=malloc(strlen(podst)+1);
      memcpy(fleks->podst,podst,strlen(podst)+1);
      fleks->ile_list=ile_list;
      fleks->odm_head=NULL;

      // wczytujemy kolejne listy odmian
      for (nr1=1;nr1<=ile_list;nr1++)
      {
         if (glo_fgets(dane,1000,stream)==NULL) goto loading_error;
         dane[strlen(dane)-1]='\0'; // kasujemy ENTER
 
         strcpy(odmiany,strchr(dane,' ')+1);
         dane[strlen(dane)-strlen(odmiany)-1]='\0';

         // zapamietanie listy odmian
         odm=malloc(sizeof(struct Odmiany));
         odm->next=NULL;
         odm->opis=malloc(strlen(dane)+1);
         memcpy(odm->opis,dane,strlen(dane)+1);
         odm->odmiany=malloc(strlen(odmiany)+1);
         memcpy(odm->odmiany,odmiany,strlen(odmiany)+1);

         if (fleks->odm_head==NULL) fleks->odm_head=odm;
         else last_odm->next=odm;

         last_odm=odm;
      }

      // dodanie wyrazu do bazy
      if (handle->language.vocabulary==NULL) handle->language.vocabulary=fleks;
      else last_fleks->next=fleks;

      last_fleks=fleks;
   }
   fclose(stream);

   if (sort_please==1) sort_voc(handle);

   return TRUE;

   loading_error:
   if (stream) fclose(stream);
   sprintf(dane,"%s%s%s.voc",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),dane);
   return FAIL;
}

static BOOL save_voc(struct DeniseHandle* handle) 
{
   FILE *stream;
   struct Fleksja *fleks,*fleksnext;
   struct Odmiany *odm,*odmnext;
   char dane[1000];

   sprintf(dane,"%s%s%s.voc",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,dane,"w"))==NULL) goto loading_error;

   if (!stream) {
		DeniseReport(handle,UNN_GetLocaleString(handle,MSG_ERROR_OPENING_FILE,"nie moge otworzyc pliku: '%s'"),dane);
      return FAIL;
   }

   for (fleks=handle->language.vocabulary;fleks!=NULL;fleks=fleks->next)
   {
      fprintf(stream,"%s\n%d\n",fleks->podst,fleks->ile_list);
      for (odm=fleks->odm_head;odm!=NULL;odm=odm->next)
         fprintf(stream,"%s %s\n",odm->opis,odm->odmiany);
   }
   fprintf(stream,"%c\n",'~');
   fclose(stream);

   return TRUE;

loading_error:
   if (stream) fclose(stream);
   sprintf(dane,"%s%s%s.voc",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),dane);
   return FAIL;

}

static BOOL load_wgn(struct DeniseHandle* handle)
{
   FILE *stream0,*stream;
   char line[1000],line1[1000];
   struct WordGenerator *wgen=NULL;
   struct Odmiany *odm;
   struct WordsList *words;

   handle->language.wordgenerator=NULL;

   sprintf(line,"%s%s%s.wgn",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream0=dat_file_open(handle,line,"r"))==NULL) return TRUE;
   if (glo_file_lenght(stream0)==0) {fclose(stream0);return TRUE;}

   for (;;)
   {
      if (glo_fgets(line,100,stream0)==NULL) break;

      if (line[0]=='#') continue;
      line[strlen(line)-1]='\0'; // kasujemy ENTER
      sprintf(line1,"%s%s%s",handle->Prefix,handle->LanguagesDir,line);

      if ((stream=dat_file_open(handle,line1,"r"))==NULL) break;
      if (glo_file_lenght(stream)==0) {fclose(stream);break;}

      for (;;)
      {
         if (glo_fgets(line,500,stream)==NULL) break;
         if (line[0]=='#') continue;
         line[strlen(line)-1]='\0'; // kasujemy ENTER

         // jesli to poczatek nowej sekcji odmian, to tworzymy ja
         if (line[0]=='-')
         {
            wgen=malloc(sizeof(struct WordGenerator));
            wgen->next=NULL;
            wgen->odmiany=NULL;
            wgen->words=NULL;
				wgen->chars=NULL;
            if (handle->language.wordgenerator==NULL) handle->language.wordgenerator=wgen;
            else
            {
               wgen->next=handle->language.wordgenerator;
               handle->language.wordgenerator=wgen;
            }
				wgen->chars=malloc(strlen(line)+1);
            strcpy(wgen->chars,line+1);
//printf("!%s!\n",wgen->chars);
         }
/*         // jesli to ciag parametrow odmiany to zapamietujemy
         else if (line[0]=='=')
         {
            odm=malloc(sizeof(struct Odmiany));
            odm->next=NULL;
            glo_get_part(line+1,1,line1,' ');
            odm->opis=malloc(strlen(line1)+1);
            memcpy(odm->opis,line1,strlen(line1)+1);
            glo_get_part(line+1,2,line1,' ');
            odm->odmiany=malloc(strlen(line1)+1);
            memcpy(odm->odmiany,line1,strlen(line1)+1);

            if (wgen->odmiany==NULL) wgen->odmiany=odm;
            else
            {
               odm->next=wgen->odmiany;
               wgen->odmiany=odm;
            }
         }
         // jesli to wyrazy podstawowe, to dopisujemy do listy
         else
         {
            words=malloc(sizeof(struct WordsList));
            words->next=NULL;
            words->word=malloc(strlen(line)+1);
            memcpy(words->word,line,strlen(line)+1);

            if (wgen->words==NULL) wgen->words=words;
            else
            {
               words->next=wgen->words;
               wgen->words=words;
            }
         }
*/
         else if (wgen) {
		 if (line[0]=='=')
		 {
		    odm=malloc(sizeof(struct Odmiany));
		    odm->next=NULL;
		    glo_get_part(line+1,1,line1,' ');
		    odm->opis=malloc(strlen(line1)+1);
		    memcpy(odm->opis,line1,strlen(line1)+1);
		    glo_get_part(line+1,2,line1,' ');
		    odm->odmiany=malloc(strlen(line1)+1);
		    memcpy(odm->odmiany,line1,strlen(line1)+1);

		    if (wgen->odmiany==NULL) wgen->odmiany=odm;
		    else
		    {
		       odm->next=wgen->odmiany;
		       wgen->odmiany=odm;
		    }
		 }
		 // jesli to wyrazy podstawowe, to dopisujemy do listy
		 else
		 {
		    words=malloc(sizeof(struct WordsList));
		    words->next=NULL;
		    words->word=malloc(strlen(line)+1);
		    memcpy(words->word,line,strlen(line)+1);

		    if (wgen->words==NULL) wgen->words=words;
		    else
		    {
		       words->next=wgen->words;
		       wgen->words=words;
		    }
		 }
	 } else
		 printf("Bladw w pliku .wgn\n"); 


      }
      fclose(stream);
   }
   fclose(stream0);

   return TRUE;
}

static BOOL load_gr1(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:
   
   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   struct GrammarData *gra_word=NULL;

   sprintf(input,"%s%s%s.gr1",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   handle->language.gr1=NULL;

   for (;;)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         gra_word=malloc(sizeof(struct GrammarData));
         gra_word->data=malloc(strlen(complete)+1);
         memcpy(gra_word->data,complete,strlen(complete)+1);
         if (handle->language.gr1==NULL)
         {
            gra_word->next=NULL;
            handle->language.gr1=gra_word;
         }
         else
         {
            gra_word->next=handle->language.gr1->next;
            handle->language.gr1->next=gra_word;
         }
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.gr1",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),input);
   return FAIL;
}

static BOOL load_gr2(struct DeniseHandle* handle)
{
   FILE *stream;
   char input[100],temp[10];

   sprintf(input,"%s%s%s.gr2",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) return FAIL;
   handle->language.gr2[0]='\0';

   for (;;)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarza pomijamy)
      if (glo_fgets(input,100,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      // KOMPILACJA
      sprintf(temp,"%c",atoi(input)+'A'-1);
      strcat(handle->language.gr2,temp);
      // KOMPILACJA KONIEC
   }
   fclose(stream);

   return TRUE;
}

static BOOL load_gr3(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:
   
   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   char temp1[100],temp2[10];
   int poz,typ,nr1,ile;
   struct GrammarExtData *gra_word=NULL;

   sprintf(input,"%s%s%s.gr3",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   for (poz=0;poz<100;poz++) handle->language.gr3[poz]=NULL;

   for (;;)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         typ=atoi(complete);
         strcpy(input,complete+strcspn(complete,":")+2);
         strcpy(complete,input);
         // linia z opisem zwiazku
         if (typ==1)
         {
            gra_word=malloc(sizeof(struct GrammarExtData));
            gra_word->data=malloc(strlen(complete)+1);
            memcpy(gra_word->data,complete,strlen(complete)+1);
            gra_word->parts_number=glo_count_parts(handle,complete,'+');
            gra_word->zakaz_miedzy=NULL;
            gra_word->fleksja_wynikowa=NULL;
            poz=atoi(complete)-1;
            if (handle->language.gr3[poz]==NULL)
            {
               gra_word->next=NULL;
               handle->language.gr3[poz]=gra_word;
            }
            else
            {
               gra_word->next=handle->language.gr3[poz]->next;
               handle->language.gr3[poz]->next=gra_word;
            }
         }
         else if (typ==2)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            gra_word->zakaz_miedzy=malloc(strlen(input)+1);
            memcpy(gra_word->zakaz_miedzy,input,strlen(input)+1);
         }
         else if (typ==3)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            // KOMPILACJA
            // tlumaczenie z formatu: 1-opis,2-opis,?...
            // tworzymy format prosty: AB?..C
            complete[0]='\0';
            for (nr1=0,ile=glo_count_parts(handle,input,',');nr1<ile;nr1++)
            {
               glo_get_part(input,nr1+1,temp1,',');

					if (temp1[0]=='?') sprintf(temp2,"?");
               else sprintf(temp2,"%c",atoi(temp1)+'A'-1);
               strcat(complete,temp2);
            }
            // KOMPILACJA KONIEC

            gra_word->fleksja_wynikowa=malloc(strlen(complete)+1);
            memcpy(gra_word->fleksja_wynikowa,complete,strlen(complete)+1);
         }
         else
         {
            goto LOADING_ERROR;
         }
       }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.gr3",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),input);
   return FAIL;
}

static BOOL load_gr4(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:
   
   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   char temp1[100],temp2[10];
   int poz,typ,nr1,ile;
   struct GrammarExtData *gra_word=NULL;

   sprintf(input,"%s%s%s.gr4",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   for (poz=0;poz<100;poz++) handle->language.gr4[poz]=NULL;

   for (;;)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         typ=atoi(complete);
         strcpy(input,complete+strcspn(complete,":")+2);
         strcpy(complete,input);
         // linia z opisem zwiazku
         if (typ==1)
         {
            gra_word=malloc(sizeof(struct GrammarExtData));
            gra_word->data=malloc(strlen(complete)+1);
            memcpy(gra_word->data,complete,strlen(complete)+1);
            gra_word->parts_number=glo_count_parts(handle,complete,'+');
            gra_word->zakaz_miedzy=NULL;
            gra_word->fleksja_wynikowa=NULL;
            poz=atoi(complete)-1;
            if (handle->language.gr4[poz]==NULL)
            {
               gra_word->next=NULL;
               handle->language.gr4[poz]=gra_word;
            }
            else
            {
               gra_word->next=handle->language.gr4[poz]->next;
               handle->language.gr4[poz]->next=gra_word;
            }
         }
         else if (typ==2)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            gra_word->zakaz_miedzy=malloc(strlen(input)+1);
            memcpy(gra_word->zakaz_miedzy,input,strlen(input)+1);
         }
         else if (typ==3)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            // KOMPILACJA
            // tlumaczenie z formatu: 1-opis,2-opis,?...
            // tworzymy format prosty: AB?..C
            complete[0]='\0';
            for (nr1=0,ile=glo_count_parts(handle,input,',');nr1<ile;nr1++)
            {
               glo_get_part(input,nr1+1,temp1,',');

					if (temp1[0]=='?') sprintf(temp2,"?");
               else sprintf(temp2,"%c",atoi(temp1)+'A'-1);
               strcat(complete,temp2);
            }
            // KOMPILACJA KONIEC

            gra_word->fleksja_wynikowa=malloc(strlen(complete)+1);
            memcpy(gra_word->fleksja_wynikowa,complete,strlen(complete)+1);
         }
         else
         {
            goto LOADING_ERROR;
         }
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.gr4",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),input);
   return FAIL;
}

static BOOL load_gr5(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:
   
   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   char temp1[100],temp2[10];
   int poz,typ,nr1,ile;
   struct GrammarExtData *gra_word=NULL;

   sprintf(input,"%s%s%s.gr5",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;

   for (;;)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         typ=atoi(complete);
         strcpy(input,complete+strcspn(complete,":")+2);
         strcpy(complete,input);
         // linia z opisem zwiazku
         if (typ==1)
         {
            gra_word=malloc(sizeof(struct GrammarExtData));
            gra_word->data=malloc(strlen(complete)+1);
            memcpy(gra_word->data,complete,strlen(complete)+1);
            gra_word->parts_number=glo_count_parts(handle,complete,'+');
            gra_word->zakaz_miedzy=NULL;
            gra_word->fleksja_wynikowa=NULL;
            poz=atoi(complete)-1;
            if (handle->language.gr5[poz]==NULL)
            {
               gra_word->next=NULL;
               handle->language.gr5[poz]=gra_word;
            }
            else
            {
               gra_word->next=handle->language.gr5[poz]->next;
               handle->language.gr5[poz]->next=gra_word;
            }
         }
         else if (typ==2)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            if (gra_word==NULL) goto LOADING_ERROR;
            gra_word->zakaz_miedzy=malloc(strlen(input)+1);
            memcpy(gra_word->zakaz_miedzy,input,strlen(input)+1);
         }
         else if (typ==3)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            // KOMPILACJA
            // tlumaczenie z formatu: 1-opis,2-opis,?...
            // tworzymy format prosty: AB?..C
            complete[0]='\0';
            for (nr1=0,ile=glo_count_parts(handle,input,',');nr1<ile;nr1++)
            {
               glo_get_part(input,nr1+1,temp1,',');

					if (temp1[0]=='?') sprintf(temp2,"?");
               else sprintf(temp2,"%c",atoi(temp1)+'A'-1);
               strcat(complete,temp2);
            }
            // KOMPILACJA KONIEC

            gra_word->fleksja_wynikowa=malloc(strlen(complete)+1);
            memcpy(gra_word->fleksja_wynikowa,complete,strlen(complete)+1);
         }
         else
         {
            goto LOADING_ERROR;
         }
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.gr5",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),input);
   return FAIL;
}

static BOOL load_gr6(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:

   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   char func_name[100];
   char temp1[100],temp2[10];
   int func_nr=0,nr1=0,ile=0;
   struct GrammarData *gra_word;

   func_name[0]='\0';

   sprintf(input,"%s%s%s.gr6",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   handle->language.gr6[0]=NULL;handle->language.gr6[1]=NULL;handle->language.gr6[2]=NULL;handle->language.gr6[3]=NULL;handle->language.gr6[4]=NULL;

   for (;;)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         sprintf(input,"%.*s",(int) strcspn(complete,"!="),complete);
         if (func_name[0]=='\0') strcpy(func_name,input);
         else if (strcmp(func_name,input)!=0)
         {
            strcpy(func_name,input);
            func_nr++;
            if (func_nr==5) goto LOADING_ERROR;
         }

         // KOMPILACJA
         // tworzymy format prosty: ABC...;ABC...;ABC...
         sprintf(input,"%.*s",(int) strlen(complete)-strlen(func_name)-2,complete+strlen(func_name)+2);
         complete[0]='\0';
         for (nr1=0,ile=glo_count_parts(handle,input,';');nr1<ile;nr1++)
         {
            glo_get_part(input,nr1+1,temp1,';');
            sprintf(temp2,"%c",atoi(temp1)+'A'-1);
            strcat(complete,temp2);
         }
         // KOMPILACJA KONIEC

         gra_word=malloc(sizeof(struct GrammarData));
         gra_word->data=malloc(strlen(complete)+1);
         memcpy(gra_word->data,complete,strlen(complete)+1);
         if (handle->language.gr6[func_nr]==NULL)
         {
            gra_word->next=NULL;
            handle->language.gr6[func_nr]=gra_word;
         }
         else
         {
            gra_word->next=handle->language.gr6[func_nr]->next;
            handle->language.gr6[func_nr]->next=gra_word;
         }
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.gr6",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),input);
   return FAIL;
}

static BOOL load_gr7(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:
   
   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   char temp1[300],temp2[300],*ptr;
   int poz,typ,nr1,ile,line=0;
   struct GrammarFuncData *gra_word=NULL;

   sprintf(input,"%s%s%s.gr7",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   for (poz=0;poz<100;poz++) handle->language.gr7[poz]=NULL;

   typ=0;
   for (;;line++)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         typ=atoi(complete);
         strcpy(input,complete+strcspn(complete,":")+2);

         // linia z opisem zwiazku
         if (typ==1)
         {
            gra_word=malloc(sizeof(struct GrammarFuncData));
				gra_word->next=NULL;
            gra_word->data=malloc(strlen(input)+1);
            gra_word->func=NULL;
            gra_word->memb_nadrz=NULL;
            gra_word->memb_podrz=NULL;
            gra_word->zakaz_nadrz=NULL;
            gra_word->zakaz_podrz=NULL;
            gra_word->kolejnosc=0;
            gra_word->zakaz_miedzy_fun=NULL;
            gra_word->zakaz_miedzy_fle=NULL;
            memcpy(gra_word->data,input,strlen(input)+1);
            poz=atoi(input)-1;

            if (handle->language.gr7[poz]==NULL)
            {
               gra_word->next=NULL;
               handle->language.gr7[poz]=gra_word;
            }
            else
            {
               gra_word->next=handle->language.gr7[poz]->next;
               handle->language.gr7[poz]->next=gra_word;
            }
         }
         // linia z opisem funkcji w zdaniu
         else if (typ==2)
         {
            if (gra_word==NULL) goto LOADING_ERROR;

            // KOMPILACJA
            // tlumaczenie z formatu: 1+2;...;3+4;5+6
            // tworzymy format prosty: 12;..;34;56
            complete[0]='\0';
            ile=glo_count_parts(handle,input,';');

            for (nr1=0;nr1<ile;nr1++)
            {
               glo_get_part(input,nr1+1,temp1,';');
               if (nr1==ile-1)
               {
						ptr=strchr(temp1,'+');
                  temp2[0]=temp1[0];
                  temp2[1]=ptr[1];
                  temp2[2]='\0';
               }
               else
               {
						ptr=strchr(temp1,'+');
                  temp2[0]=temp1[0];
                  temp2[1]=ptr[1];
                  temp2[2]=';';
                  temp2[3]='\0';
               }
					strcat(complete,temp2);
            }
            // KOMPILACJA KONIEC
            gra_word->func=malloc(strlen(complete)+1);
            memcpy(gra_word->func,complete,strlen(complete)+1);
         }
         // linie z wymaganymi i zabronionymi czesciami mowy
         else if (typ==3 || typ==4 || typ==5 || typ==6)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            // linia z czesciami mowy wymaganymi w grupie nadrzednej
            if (typ==3)
            {
               gra_word->memb_nadrz=malloc(strlen(input)+1);
               memcpy(gra_word->memb_nadrz,input,strlen(input)+1);
            }
            // linia z czesciami mowy wymaganymi w grupie podrzednej
            else if (typ==4)
            {
               gra_word->memb_podrz=malloc(strlen(input)+1);
               memcpy(gra_word->memb_podrz,input,strlen(input)+1);
            }
            // linia z czesciami mowy zabronionymi w grupie nadrzednej
            else if (typ==5)
            {
               gra_word->zakaz_nadrz=malloc(strlen(input)+1);
               memcpy(gra_word->zakaz_nadrz,input,strlen(input)+1);
            }
            // linia z czesciami mowy zabronionymi w grupie podrzednej
            else if (typ==6)
            {
               gra_word->zakaz_podrz=malloc(strlen(input)+1);
               memcpy(gra_word->zakaz_podrz,input,strlen(input)+1);
            }
         }
         else if (typ==7)
         {
            gra_word->kolejnosc=atoi(input);
         }
         // zabronione czesci zdania miedzy grupami
         else if (typ==8)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            // KOMPILACJA
            // tlumaczenie z formatu: 1-funkcja;2-funkcja;...;3-funkcja
            // tworzymy format prosty: 12..3
            complete[0]='\0';
            for (nr1=0,ile=glo_count_parts(handle,input,';');nr1<ile;nr1++)
            {
               glo_get_part(input,nr1+1,temp1,';');

               sprintf(temp2,"%d",atoi(temp1));
               strcat(complete,temp2);
            }
            // KOMPILACJA KONIEC

            gra_word->zakaz_miedzy_fun=malloc(strlen(complete)+1);
            memcpy(gra_word->zakaz_miedzy_fun,complete,strlen(complete)+1);
         }
         // zabronione czesci zdania miedzy grupami
         else if (typ==9)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            gra_word->zakaz_miedzy_fle=malloc(strlen(input)+1);
            memcpy(gra_word->zakaz_miedzy_fle,input,strlen(input)+1);
         }
/*
         else
         {
            goto LOADING_ERROR;
         }
*/
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.gr7",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_INCORRECT_LINE,"blad w pliku: %s, w linii %d (%s)"),input,line,"(?)");
   return FAIL;
}

static BOOL load_lo1(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:

   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   int nr1,ile;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   char temp1[100],temp2[10];
   struct GrammarData *gra_word;

   sprintf(input,"%s%s%s.lo1",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   handle->language.lo1=NULL;

   for (;;)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         // KOMPILACJA
         // kompilacja formatu: 1-?,2-?,...,3-?,4-?
         // tworzymy format prosty: AB...CD
         strcpy(input,complete);
         complete[0]='\0';
         for (nr1=0,ile=glo_count_parts(handle,input,',');nr1<ile;nr1++)
         {
            glo_get_part(input,nr1+1,temp1,',');
            sprintf(temp2,"%c",atoi(temp1)+'A'-1);
            strcat(complete,temp2);
         }
         // KOMPILACJA KONIEC

         gra_word=malloc(sizeof(struct GrammarData));
         gra_word->data=malloc(strlen(complete)+1);
         memcpy(gra_word->data,complete,strlen(complete)+1);
         if (handle->language.lo1==NULL)
         {
            gra_word->next=NULL;
            handle->language.lo1=gra_word;
         }
         else
         {
            gra_word->next=handle->language.lo1->next;
            handle->language.lo1->next=gra_word;
         }
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.lo1",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),input);
   return FAIL;
}

static BOOL load_lo2(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:

   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   int typ;
   struct GrammarData *gra_word;

   sprintf(input,"%s%s%s.lo2",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   handle->language.lo2[0]=NULL;handle->language.lo2[1]=NULL;

   for (;;)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         typ=atoi(complete)-1;
         strcpy(input,complete+strcspn(complete,":")+2);

         gra_word=malloc(sizeof(struct GrammarData));
         gra_word->next=NULL;
         gra_word->data=malloc(strlen(input)+1);
         memcpy(gra_word->data,input,strlen(input)+1);

         if (handle->language.lo2[typ]==NULL)
         {
            gra_word->next=NULL;
            handle->language.lo2[typ]=gra_word;
         }
         else
         {
            gra_word->next=handle->language.lo2[typ]->next;
            handle->language.lo2[typ]->next=gra_word;
         }
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.lo2",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),input);
   return FAIL;
}

static BOOL load_lo3(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:

   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   int typ,nr1,size;
   struct GrammarData *gra_word;

   sprintf(input,"%s%s%s.lo3",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   handle->language.lo3[0]=NULL;handle->language.lo3[1]=NULL;
   handle->language.lo3[2]=NULL;handle->language.lo3[3]=NULL;

   for (;;)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         typ=atoi(complete)-1;
         strcpy(input,complete+strcspn(complete,":")+2);

         // zamieniany znak '=' na znak '+'
		 size=strlen(input);
         for (nr1=0;nr1<size;nr1++)
            if (input[nr1]=='=') input[nr1]='+';

         gra_word=malloc(sizeof(struct GrammarData));
         gra_word->next=NULL;
         gra_word->data=malloc(strlen(input)+1);
         memcpy(gra_word->data,input,strlen(input)+1);

         // zaznaczamy, ze wyraz o tej czesci mowy moze byc kwantyfikatorem
         handle->language.lo3_voc_filter[atoi(input)]=TRUE;

         if (handle->language.lo3[typ]==NULL)
         {
            gra_word->next=NULL;
            handle->language.lo3[typ]=gra_word;
         }
         else
         {
            gra_word->next=handle->language.lo3[typ]->next;
            handle->language.lo3[typ]->next=gra_word;
         }
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.lo3",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),input);
   return FAIL;
}

static BOOL load_lo4(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:

   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   int typ,nr1,size;
   struct GrammarData *gra_word;

   sprintf(input,"%s%s%s.lo4",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   handle->language.lo4[0]=NULL;handle->language.lo4[1]=NULL;
   handle->language.lo4[2]=NULL;handle->language.lo4[3]=NULL;

   for (;;)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         typ=atoi(complete)-1;
         strcpy(input,complete+strcspn(complete,":")+2);

         // zamieniany znak '=' na znak '+'
		 size=strlen(input);
         for (nr1=0;nr1<size;nr1++)
            if (input[nr1]=='=') input[nr1]='+';

         gra_word=malloc(sizeof(struct GrammarData));
         gra_word->next=NULL;
         gra_word->data=malloc(strlen(input)+1);
         memcpy(gra_word->data,input,strlen(input)+1);

         if (handle->language.lo4[typ]==NULL)
         {
            gra_word->next=NULL;
            handle->language.lo4[typ]=gra_word;
         }
         else
         {
            gra_word->next=handle->language.lo4[typ]->next;
            handle->language.lo4[typ]->next=gra_word;
         }
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.lo4",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),input);
   return FAIL;
}

static BOOL load_lo5(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:
   
   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   int typ,line=0;
   struct GrammarLo5Data *gra_word=NULL;

   sprintf(input,"%s%s%s.lo5",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
	handle->language.lo5=NULL;

   typ=0;
   for (;;line++)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         typ=atoi(complete);
         strcpy(input,complete+strcspn(complete,":")+2);
         // linia z opisem zwiazku
         if (typ==1)
         {
            gra_word=malloc(sizeof(struct GrammarLo5Data));
            gra_word->czesc_mowy=malloc(strlen(input)+1);
            gra_word->func=0;
            gra_word->requied=NULL;
            gra_word->banned=NULL;
            memcpy(gra_word->czesc_mowy,input,strlen(input)+1);

            if (handle->language.lo5==NULL)
            {
               gra_word->next=NULL;
               handle->language.lo5=gra_word;
            }
            else
            {
               gra_word->next=handle->language.lo5->next;
               handle->language.lo5->next=gra_word;
            }
         }
         // linia z opisem funkcji w zdaniu
         else if (typ==2)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            gra_word->func=atoi(input);
         }
         // linie z wymaganymi i zabronionymi czesciami mowy
         else if (typ==3 || typ==4)
         {
            if (gra_word==NULL) goto LOADING_ERROR;

            // linia z czesciami mowy wymaganymi w grupie
            if (typ==3)
            {
               gra_word->requied=malloc(strlen(input)+1);
               memcpy(gra_word->requied,input,strlen(input)+1);
            }
            // linia z czesciami mowy zabronionymi w grupie
            else if (typ==4)
            {
               gra_word->banned=malloc(strlen(input)+1);
               memcpy(gra_word->banned,input,strlen(input)+1);
            }
         }
         else
         {
            goto LOADING_ERROR;
         }
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.lo5",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_INCORRECT_LINE,"blad w pliku: %s, w linii %d (%s)"),input,line,"(?)");
   return FAIL;
}

static BOOL load_lo6(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:

   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   int typ,line=0;
   struct GrammarLo6Data *gra_word=NULL;

   sprintf(input,"%s%s%s.lo6",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   handle->language.lo6=NULL;

   typ=0;
   for (;;line++)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         typ=atoi(complete);
         strcpy(input,complete+strcspn(complete,":")+2);
         // linia z opisem zwiazku
         if (typ==1)
         {
            gra_word=malloc(sizeof(struct GrammarLo6Data));
            gra_word->czesc_mowy=malloc(strlen(input)+1);
            gra_word->func=0;
            gra_word->person=0;
            memcpy(gra_word->czesc_mowy,input,strlen(input)+1);

            if (handle->language.lo6==NULL)
            {
               gra_word->next=NULL;
               handle->language.lo6=gra_word;
            }
            else
            {
               gra_word->next=handle->language.lo6->next;
               handle->language.lo6->next=gra_word;
            }
         }
         // linia z opisem funkcji w zdaniu
         else if (typ==2)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            gra_word->func=atoi(input);
         }
         // linie z wynikajaca osoba
         else if (typ==3)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            gra_word->person=atoi(input);
         }
         else
         {
            goto LOADING_ERROR;
         }
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.lo6",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_INCORRECT_LINE,"blad w pliku: %s, w linii %d (%s)"),input,line,"(?)");
   return FAIL;
}

static BOOL load_lo7(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:
   
   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   struct GrammarData *gra_word;

   sprintf(input,"%s%s%s.lo7",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   handle->language.lo7=NULL;

   for (;;)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         gra_word=malloc(sizeof(struct GrammarData));
         gra_word->data=malloc(strlen(complete)+1);
         memcpy(gra_word->data,complete,strlen(complete)+1);
         if (handle->language.lo7==NULL)
         {
            gra_word->next=NULL;
            handle->language.lo7=gra_word;
         }
         else
         {
            gra_word->next=handle->language.lo7->next;
            handle->language.lo7->next=gra_word;
         }
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.lo7",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),input);
   return FAIL;
}

static BOOL load_lo8(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:

   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   int typ,line;
   struct GrammarLo8Data *gra_word=NULL;

   sprintf(input,"%s%s%s.lo8",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   handle->language.lo8=NULL;

   typ=0;
   for (line=0;;line++)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         typ=atoi(complete);
         strcpy(input,complete+strcspn(complete,":")+2);
         // linia z opisem zwiazku
         if (typ==1)
         {
            gra_word=malloc(sizeof(struct GrammarLo8Data));
            gra_word->cmp_data=malloc(strlen(input)+1);
            gra_word->person=0;
            gra_word->type=0;
            memcpy(gra_word->cmp_data,input,strlen(input)+1);
            if (handle->language.lo8==NULL)
            {
               gra_word->next=NULL;
               handle->language.lo8=gra_word;
            }
            else
            {
               gra_word->next=handle->language.lo8->next;
               handle->language.lo8->next=gra_word;
            }
         }
         // linie z wynikajaca osoba
         else if (typ==2)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            gra_word->person=atoi(input);
         }
         // linie z typem
         else if (typ==3)
         {
            if (gra_word==NULL) goto LOADING_ERROR;
            gra_word->type=atoi(input);
         }
         else
         {
            goto LOADING_ERROR;
         }
      }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.lo8",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_INCORRECT_LINE,"blad w pliku: %s, w linii %d (%s)"),input,line,"(?)");
   return FAIL;
}

static BOOL load_lo9(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:

   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   int typ,line;

   sprintf(input,"%s%s%s.lo9",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   handle->language.lo9=NULL;

   typ=0;
	for (line=0;;line++)
	{
		// wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
		if (glo_fgets(input,1000,stream)==NULL) break;
		input[strlen(input)-1]=input[strlen(input)];
		if (input[0]=='#' || strlen(input)==0) continue;

	    if (kontynuacja==0) complete[0]='\0';

		// jesli linia zaczyna sie trzykropkiem
		if (strncmp(input,"...",3)==0)
		{
			// jesli KONTYNUACJA=0 to BLAD!!!
			if (kontynuacja==0) goto LOADING_ERROR;
			// jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
			memmove(input,input+3,strlen(input)-2);
		}
		// jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
		else if (kontynuacja==1) goto LOADING_ERROR;

		// jesli linia konczy sie trzykropkiem
		if (strcspn(input,"...")==strlen(input)-3)
		{
			// obcinamy trzykropek na koncu
			input[strlen(input)-3]='\0';

			// jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
			if (kontynuacja==0)
			{
				complete[0]='\0';
				kontynuacja=1;
			}
		}
		// w przeciwnym razie ustawiamy KONTYNUACJA=0,
		else kontynuacja=0;

		// sprawdzamy, czy nie przepelnimy bufora
		if (strlen(complete)+strlen(input)>sizeof(complete))
		{
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
			goto LOADING_ERROR;
		}

		// dopisujemy linie do COMPLETE
		strcat(complete,input);

		// jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
		if (kontynuacja==0)
		{
			typ=atoi(complete);
			strcpy(input,complete+strcspn(complete,":")+2);

			if (handle->language.lo9==NULL)
			{
				handle->language.lo9=(struct GrammarLo9Data *) malloc(sizeof(struct GrammarLo9Data));
				handle->language.lo9->concl1=NULL;
				handle->language.lo9->concl2=NULL;
				handle->language.lo9->mean1=NULL;
				handle->language.lo9->mean2=NULL;
			}
         
			// 1-POCZTEK CZʌCI WARUNKOWEJ TOSAMOCI: 'wyrazy','wyrazy'...
			if (typ==1)
			{

				handle->language.lo9->mean1=malloc(strlen(input)+1);
				memcpy(handle->language.lo9->mean1,input,strlen(input)+1);
			}
			// 2-POCZTEK CZʌCI WYNIKOWEJ TOSAMOCI: 'wyrazy','wyrazy'...
			else if (typ==2)
			{

				handle->language.lo9->mean2=malloc(strlen(input)+1);
				memcpy(handle->language.lo9->mean2,input,strlen(input)+1);
			}
			// 3-POCZTEK CZʌCI WARUNKOWEJ WNIOSKOWANIA: 'wyrazy','wyrazy'...
			else if (typ==3)
			{

				handle->language.lo9->concl1=malloc(strlen(input)+1);
				memcpy(handle->language.lo9->concl1,input,strlen(input)+1);
			}
			// 4-POCZTEK CZʌCI WYNIKOWEJ WNIOSKOWANIA: 'wyrazy','wyrazy'...
			else if (typ==4)
			{

				handle->language.lo9->concl2=malloc(strlen(input)+1);
				memcpy(handle->language.lo9->concl2,input,strlen(input)+1);
			}
			else
			{
	            goto LOADING_ERROR;
			}
		}
	}
	fclose(stream);

	return TRUE;

	LOADING_ERROR:
	if (stream) fclose(stream);
	sprintf(input,"%s%s%s.lo9",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_INCORRECT_LINE,"blad w pliku: %s, w linii %d (%s)"),input,line,"(?)");
	return FAIL;
}

static BOOL load_fil(struct DeniseHandle* handle)
{
   /*
   Algorytm wczytywania:
   
   START
   wczytaj linie
   jesli pusta lub komentarz to na START
   jesli koniec pliku to KONIEC

   jesli KONTUNUACJA=0 to kasujemy COMPLETE,
   jesli linia zaczyna sie trzykropkiem:
      - jesli KONTYNUACJA=0 to BLAD!!!
      - jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
   jesli linia nie zaczyna sie trzykropkiem, a KONTYNUACJA=1 to BLAD!!!
   jesli linia konczy sie trzykropkiem to:
      - obcinamy trzykropek na koncu i
         - jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
      - w przeciwnym ustawiamy KONTYNUACJA=0,
   dopisujemy linie do COMPLETE
   jesli KONTYNUACJA=0 to zapamietanie "gdzies tam" COMPLETE (pelna informacja)
   skok na START
   KONIEC
   */

   FILE *stream;
   char kontynuacja=0;
   char input[1000];
   char complete[1000];
   int typ;
   struct FilterData *filter=NULL;

   sprintf(input,"%s%s%s.fil",handle->Prefix,handle->LanguagesDir,handle->language.name);
   if ((stream=dat_file_open(handle,input,"r"))==NULL) goto LOADING_ERROR;
   handle->language.fil=NULL;

   for (;;)
   {
      // wczytujemy linie i kasujemy '\n' (ostatni znak) (linie komentarze pomijamy)
      if (glo_fgets(input,1000,stream)==NULL) break;
      input[strlen(input)-1]=input[strlen(input)];
      if (input[0]=='#' || strlen(input)==0) continue;

      if (kontynuacja==0) complete[0]='\0';

      // jesli linia zaczyna sie trzykropkiem
      if (strncmp(input,"...",3)==0)
      {
         // jesli KONTYNUACJA=0 to BLAD!!!
         if (kontynuacja==0) goto LOADING_ERROR;
         // jesli KONTYNUACJA=1 to obcinamy trzykropek na poczatku,
         memmove(input,input+3,strlen(input)-2);
      }
      // jesli linia nie zaczyna sie trzykropkiem a KONTYNUACJA==1 to BLAD!!!
      else if (kontynuacja==1) goto LOADING_ERROR;

      // jesli linia konczy sie trzykropkiem
      if (strcspn(input,"...")==strlen(input)-3)
      {
         // obcinamy trzykropek na koncu
         input[strlen(input)-3]='\0';

         // jesli KONTYNUACJA=0, to kasujemy COMPLETE, i ustawiamy KONTUNUACJA=1,
         if (kontynuacja==0)
         {
            complete[0]='\0';
            kontynuacja=1;
         }
      }
      // w przeciwnym razie ustawiamy KONTYNUACJA=0,
      else kontynuacja=0;

      // sprawdzamy, czy nie przepelnimy bufora
      if (strlen(complete)+strlen(input)>sizeof(complete))
      {
			DeniseReport(handle,UNN_GetLocaleString(handle,MSG_LINE_TOO_LONG,"linia zbyt dluga"));
         goto LOADING_ERROR;
      }

      // dopisujemy linie do COMPLETE
      strcat(complete,input);

      // jesli KONTYNUACJA=0 to w COMPLETE jest juz pelna informacja
      if (kontynuacja==0)
      {
         typ=atoi(complete);
         strcpy(input,complete+strcspn(complete,":")+2);
         strcpy(complete,input);
         // linia z opisem zwiazku
         if (typ==1)
         {
            filter=malloc(sizeof(struct FilterData));
            filter->find=malloc(strlen(complete)+1);
            memcpy(filter->find,complete,strlen(complete)+1);
            filter->replace=NULL;
            filter->operation=0;
            filter->range=1;

            if (handle->language.fil==NULL)
            {
               filter->next=NULL;
               handle->language.fil=filter;
            }
            else
            {
               filter->next=handle->language.fil->next;
               handle->language.fil->next=filter;
            }
         }
         else if (typ==2)
         {
            if (filter==NULL) goto LOADING_ERROR;

            sprintf(complete,"%.*s",(int) strlen(complete)-2,complete+1);

            filter->replace=malloc(strlen(complete)+1);
            memcpy(filter->replace,complete,strlen(complete)+1);
         }
         else if (typ==3)
         {
            filter->operation=atoi(complete);
         }
         else if (typ==4)
         {
            filter->range=atoi(complete);
         }
         else
         {
            goto LOADING_ERROR;
         }
       }
   }
   fclose(stream);

   return TRUE;

   LOADING_ERROR:
   if (stream) fclose(stream);
   sprintf(input,"%s%s%s.fil",handle->Prefix,handle->LanguagesDir,handle->language.name);
	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_DATA_LOADING_ERROR,"blad odczytu danych: '%s'"),input);
   return FAIL;
}

// ***************************************************************************************
// EXTERN                    BOOL dat_language_save(struct DeniseHandle*)
// ***************************************************************************************
BOOL dat_language_save(struct DeniseHandle* handle)
{
   save_voc(handle);	
   return TRUE;
}

// ***************************************************************************************
// EXTERN                    BOOL dat_language_erase(struct DeniseHandle*)
// ***************************************************************************************
BOOL dat_language_erase(struct DeniseHandle* handle)
{
   struct WordEditor *wed,*wednext;
   struct WordGenerator *wgen,*wgen_next;
   struct WordsList *words,*words_next;
   struct GrammarData *gra_word,*gra_word_next;
   struct GrammarFuncData *gra_funcword,*gra_funcword_next;
   struct GrammarData *gra_data,*gra_data_next;
   struct GrammarExtData *gra_extdata,*gra_extdata_next;
   struct GrammarLo5Data *gra_lo5data,*gra_lo5data_next;
   struct GrammarLo6Data *gra_lo6data,*gra_lo6data_next;
   struct GrammarLo8Data *gra_lo8data,*gra_lo8data_next;
   struct FilterData *filter_data,*filter_data_next;
   struct Fleksja *fleks,*fleksnext;
   struct Odmiany *odm,*odmnext;
   int nr1;

	if (handle->language.name!=NULL)
	{
		free(handle->language.name);
		handle->language.name=NULL;
	}

   for (wgen=handle->language.wordgenerator;wgen!=NULL;wgen=wgen_next)
   {
      wgen_next=wgen->next;
      for (odm=wgen->odmiany;odm!=NULL;odm=odmnext)
      {
         odmnext=odm->next;
         if (odm->opis!=NULL) free(odm->opis);
         if (odm->odmiany!=NULL) free(odm->odmiany);
         free(odm);
      }
      for (words=wgen->words;words!=NULL;words=words_next)
      {
         words_next=words->next;
         if (words->word!=NULL) free(words->word);
         free(words);
      }
		if (wgen->chars!=NULL) free(wgen->chars);
      free(wgen);
   }

   for (wed=handle->language.wordeditor;wed!=NULL;wed=wednext)
   {
      wednext=wed->next;
      if (wed->data) free(wed->data);
      free(wed);
   }

   for (gra_word=handle->language.gr1;gra_word!=NULL;gra_word=gra_word_next)
   {
      gra_word_next=gra_word->next;
      if (gra_word->data) free(gra_word->data);
      free(gra_word);
   }

   for (gra_word=handle->language.lo1;gra_word!=NULL;gra_word=gra_word_next)
   {
      gra_word_next=gra_word->next;
      if (gra_word->data) free(gra_word->data);
      free(gra_word);
   }

   for (nr1=0;nr1<100;nr1++)
   {
      for (gra_extdata=handle->language.gr3[nr1];gra_extdata!=NULL;gra_extdata=gra_extdata_next)
      {
         gra_extdata_next=gra_extdata->next;
         if (gra_extdata->data) free(gra_extdata->data);
         if (gra_extdata->zakaz_miedzy) free(gra_extdata->zakaz_miedzy);
         if (gra_extdata->fleksja_wynikowa) free(gra_extdata->fleksja_wynikowa);
         free(gra_extdata);
      }

      for (gra_extdata=handle->language.gr4[nr1];gra_extdata!=NULL;gra_extdata=gra_extdata_next)
      {
         gra_extdata_next=gra_extdata->next;
         if (gra_extdata->data) free(gra_extdata->data);
         if (gra_extdata->zakaz_miedzy) free(gra_extdata->zakaz_miedzy);
         if (gra_extdata->fleksja_wynikowa) free(gra_extdata->fleksja_wynikowa);
         free(gra_extdata);
      }

      for (gra_extdata=handle->language.gr5[nr1];gra_extdata!=NULL;gra_extdata=gra_extdata_next)
      {
         gra_extdata_next=gra_extdata->next;
         if (gra_extdata->data) free(gra_extdata->data);
         if (gra_extdata->zakaz_miedzy) free(gra_extdata->zakaz_miedzy);
         if (gra_extdata->fleksja_wynikowa) free(gra_extdata->fleksja_wynikowa);
         free(gra_extdata);
      }

      for (gra_funcword=handle->language.gr7[nr1];gra_funcword!=NULL;gra_funcword=gra_funcword_next)
      {
         gra_funcword_next=gra_funcword->next;
         if (gra_funcword->data) free(gra_funcword->data);
         if (gra_funcword->func) free(gra_funcword->func);
         if (gra_funcword->memb_nadrz) free(gra_funcword->memb_nadrz);
         if (gra_funcword->memb_podrz) free(gra_funcword->memb_podrz);
         if (gra_funcword->zakaz_nadrz) free(gra_funcword->zakaz_nadrz);
         if (gra_funcword->zakaz_podrz) free(gra_funcword->zakaz_podrz);
         if (gra_funcword->zakaz_miedzy_fun) free(gra_funcword->zakaz_miedzy_fun);
         if (gra_funcword->zakaz_miedzy_fle) free(gra_funcword->zakaz_miedzy_fle);
         free(gra_funcword);
      }
   }

   for (nr1=0;nr1<5;nr1++)
      for (gra_data=handle->language.gr6[nr1];gra_data!=NULL;gra_data=gra_data_next)
      {
         gra_data_next=gra_data->next;
         if (gra_data->data) free(gra_data->data);
         free(gra_data);
      }

   for (nr1=0;nr1<4;nr1++)
      for (gra_data=handle->language.lo2[nr1];gra_data!=NULL;gra_data=gra_data_next)
      {
         gra_data_next=gra_data->next;
         if (gra_data->data) free(gra_data->data);
         free(gra_data);
      }

   for (nr1=0;nr1<4;nr1++)
      for (gra_data=handle->language.lo3[nr1];gra_data!=NULL;gra_data=gra_data_next)
      {
         gra_data_next=gra_data->next;
         if (gra_data->data) free(gra_data->data);
         free(gra_data);
      }

   for (nr1=0;nr1<4;nr1++)
      for (gra_data=handle->language.lo4[nr1];gra_data!=NULL;gra_data=gra_data_next)
      {
         gra_data_next=gra_data->next;
         if (gra_data->data) free(gra_data->data);
         free(gra_data);
      }


   for (gra_lo5data=handle->language.lo5;gra_lo5data!=NULL;gra_lo5data=gra_lo5data_next)
   {
      gra_lo5data_next=gra_lo5data->next;
      if (gra_lo5data->czesc_mowy) free(gra_lo5data->czesc_mowy);
      if (gra_lo5data->requied) free(gra_lo5data->requied);
      if (gra_lo5data->banned) free(gra_lo5data->banned);
      free(gra_lo5data);
   }

   for (gra_lo6data=handle->language.lo6;gra_lo6data!=NULL;gra_lo6data=gra_lo6data_next)
   {
      gra_lo6data_next=gra_lo6data->next;
      if (gra_lo6data->czesc_mowy) free(gra_lo6data->czesc_mowy);
      free(gra_lo6data);
   }

   for (gra_word=handle->language.lo7;gra_word!=NULL;gra_word=gra_word_next)
   {
      gra_word_next=gra_word->next;
      if (gra_word->data) free(gra_word->data);
      free(gra_word);
   }

   for (gra_lo8data=handle->language.lo8;gra_lo8data!=NULL;gra_lo8data=gra_lo8data_next)
   {
      gra_lo8data_next=gra_lo8data->next;
      if (gra_lo8data->cmp_data) free(gra_lo8data->cmp_data);
      free(gra_lo8data);
   }

	if (handle->language.lo9!=NULL)
	{
		if (handle->language.lo9->mean1) free(handle->language.lo9->mean1);
		if (handle->language.lo9->mean2) free(handle->language.lo9->mean2);
		if (handle->language.lo9->concl1) free(handle->language.lo9->concl1);
		if (handle->language.lo9->concl2) free(handle->language.lo9->concl2);
		free(handle->language.lo9);
	}

   for (filter_data=handle->language.fil;filter_data!=NULL;filter_data=filter_data_next)
   {
      filter_data_next=filter_data->next;
      if (filter_data->find) free(filter_data->find);
      if (filter_data->replace) free(filter_data->replace);
      free(filter_data);
   }

   for (fleks=handle->language.vocabulary;fleks!=NULL;fleks=fleksnext)
   {
      for (odm=fleks->odm_head;odm!=NULL;odm=odmnext)
      {
         odmnext=odm->next;
         if (odm->opis) free(odm->opis);
         if (odm->odmiany) free(odm->odmiany);
         free(odm);
      }
      fleksnext=fleks->next;
      if (fleks->podst) free(fleks->podst);
      free(fleks);
   }

	CloseCatalog(handle);

   return TRUE;
}

static void sort_voc(struct DeniseHandle* handle)
{
   /*
   funkcja sortuje alfabetycznie baze fleksji (VOC),
   sortowanie jest wzgledem wyrazow podstawowych,
   */

   int a=clock();
   char sortowano;
   struct Fleksja *fleks1,*fleks2,*fleks_prev,*fleks_next;

	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_SORTING_VOCABULARY,"sortuje slownik..."));

   for (;;)
   {
      sortowano=0;
      fleks_prev=NULL;
      for (fleks1=handle->language.vocabulary;fleks1->next!=NULL;fleks_prev=fleks1,fleks1=fleks1->next)
      {
         fleks2=fleks1->next;

         if (glo_what_first(handle,fleks1->podst,fleks2->podst,0)==2)
         {
            /*
            prev na fleks2
            fleks2 na fleks1
            fleks1 na fleks2->next
            fleks1=fleks2
            */

            sortowano=1;

            if (fleks2->next==NULL) fleks_next=NULL;
            else fleks_next=fleks2->next;

            if (fleks_prev!=NULL) fleks_prev->next=fleks2;
            fleks2->next=fleks1;
            if (fleks_next==NULL) fleks1->next=NULL;
            else fleks1->next=fleks_next;

            if (fleks1==handle->language.vocabulary) handle->language.vocabulary=fleks2;
            fleks1=fleks2;
         }
      }
      if (sortowano==0) break;
   }

	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_VOCABULARY_SORTED,"posortowano (%d)"),clock()-a);
}

// ***************************************************************************** funkcje pozostale

static void free_odm(struct Odmiany *odm)
{
   free(odm->opis);
   free(odm->odmiany);
   free(odm);
}

static void free_fleks(struct DeniseHandle* handle,struct Fleksja *fleks)
{
   struct Odmiany *odm,*odmnext;

   if (fleks==handle->language.vocabulary) handle->language.vocabulary=NULL;
   if (fleks==handle->language.voctemp) handle->language.voctemp=NULL;

   for (odm=fleks->odm_head;odm!=NULL;odm=odmnext)
   {
      odmnext=odm->next;
      free_odm(odm);
   }
   free(fleks->podst);
   free(fleks);
}

void dat_count_odmiany(struct DeniseHandle* handle)
{
   struct WordGenerator *wgen;
   struct WordsList *words;
   struct Fleksja *fleks;
   struct Odmiany *odm;
   char nr1,ile,temp[500];
   unsigned long wynik1=0,wynik2=0,ilewyr,ilezm;

   // VOC
   for (fleks=handle->language.vocabulary;fleks!=NULL;fleks=fleks->next)
   {
      for (odm=fleks->odm_head;odm!=NULL;odm=odm->next)
      {
         for (nr1=1,ile=glo_count_parts(handle,odm->odmiany,',');nr1<=ile;nr1++)
         {
            glo_get_part(odm->odmiany,nr1,temp,',');
            if (strlen(temp)>0) wynik1++;
         }
      }
   }

   // WG
   for (wgen=handle->language.wordgenerator;wgen!=NULL;wgen=wgen->next)
   {
      // liczymy ile jest zmiennych parametrow odmiany
      ilezm=0;
      for (odm=wgen->odmiany;odm!=NULL;odm=odm->next)
      {
         for (nr1=1,ile=glo_count_parts(handle,odm->odmiany,',');nr1<=ile;nr1++)
         {
            glo_get_part(odm->odmiany,nr1,temp,',');
            if (strlen(temp)>0) ilezm++;
         }
      }

      // liczymy ile jest wyrazow podstawowych
      ilewyr=0;
      for (words=wgen->words;words!=NULL;words=words->next)
      {
         ilewyr++;
      }
      wynik2=wynik2+ilewyr*ilezm;
   }   

	DeniseReport(handle,UNN_GetLocaleString(handle,MSG_WORDS_NUMBER,"liczba odmian: %d (VOC) + %d (WG)"),wynik1,wynik2);
}
// ***************************************************************************************
// EXTERN                              void ???
// ***************************************************************************************
void dat_delete_fleks(struct DeniseHandle* handle,struct Fleksja *fleks)
{
   /*
   funkcja kasuje podany wyraz
   */

   struct Fleksja *fleks1;

   if (fleks==handle->language.vocabulary) {
      handle->language.vocabulary=handle->language.vocabulary->next;
      free_fleks(handle,fleks);
   } else if (fleks==handle->language.voctemp) {
      handle->language.voctemp=handle->language.voctemp->next;
      free_fleks(handle,fleks);
	 } else {
      for (fleks1=handle->language.vocabulary;fleks1!=NULL && fleks1->next!=fleks;fleks1=fleks1->next);
      if (fleks1!=NULL)
      {
         if (fleks1->next==fleks)
         {
            fleks1->next=fleks->next;
            free_fleks(handle,fleks);
         }
      }
   }
}
// ***************************************************************************************
// EXTERN                              void ???
// ***************************************************************************************
void dat_delete_odm(struct DeniseHandle* handle,struct Fleksja *fleks,struct Odmiany *odm)
{
   /*
   funkcja kasuje podana liste odmian,
   */

   struct Odmiany *odm1;

   if (fleks->ile_list==1) dat_delete_fleks(handle,fleks);
   else if (odm==fleks->odm_head)
   {
      fleks->odm_head=fleks->odm_head->next;
      fleks->ile_list--;
      free_odm(odm);
   }
   else
   {
      for (odm1=fleks->odm_head;odm1!=NULL && odm1->next!=odm;odm1=odm1->next);
      if (odm1!=NULL)
      {
         if (odm1->next==odm)
         {
            odm1->next=odm->next;
            fleks->ile_list--;
            free_odm(odm);
         }
      }
   }
}


static int dane_strncmp(char *dane1,char *dane2,int len)
{
	int nr1;

	for (nr1=0;nr1<len;nr1++)
	{
		if (dane1[nr1]=='?' || dane2[nr1]=='?') continue;
		else if (dane1[nr1]==dane2[nr1]) continue;
		else return -1;
	}
	return 0;
}

// ***************************************************************************************
// EXTERN                              void ???
// ***************************************************************************************
struct Fleksja **dat_find_word(struct DeniseHandle *handle,struct Fleksja *start,char *podst,char *wyraz,char *dane,int tryb)
{
   /*
   funkcja szuka w bazie fleksji poczawszy od pozycji "start" wyrazu o podanej
   charakterystyce poprzez wyraz podstawowy, wyraz odmieniony i dane.
   w przypadku nie podania parametru (zapis: ""), zwracany jest pierwszy element
   bedacy zgodnym z podanymi,
   funkcja zwraca wskaznik na odnaleziona strukture lub NULL,

   tryb:
   - 0 - jesli wyraz sklada sie z wielu czesci, to wymagana jest zgodnosc ich wszystkich
   - 1 - jesli wyraz w bazie sklada sie z wielu czesci, to wymagane jest, aby
         chociaz jedna z nich byla taka sama jak wyraz szukany
   */

   struct WordGenerator *wgen;
   struct WordsList *words;
   struct Fleksja *fleks;
   struct Odmiany *odm;
   struct Fleksja **allfleks;
   int fleknum;
   char temp[1000],temp_wyraz[100],ile,temp1[100],temp2[100],temp3[100],temat[100];
	int nr1;

   // akceptowane dwa rodzaje parametrow: NULL lub ""
   if (podst) if (strlen(podst)==0) podst=NULL;
   if (wyraz) if (strlen(wyraz)==0) wyraz=NULL;
   if (dane) if (strlen(dane)==0) dane=NULL;

   // jesli to pierwsze wywolanie (tzn. szukanie od poczatku) to generowanie odmian
/*
   if(start!=NULL)
   printf("dat_find_word(%s,%s,%s,%s,%d)\n",start->podst,podst,wyraz,dane,tryb);
   else
   printf("dat_find_word(NULL,%s,%s,%s,%d)\n",start,podst,wyraz,dane,tryb);
*/
   fleks=voc_find_word(handle,handle->language.voctemp,podst,wyraz,dane,tryb); 

   if (!fleks)
   {
      /*
      proba generowania odmian, i umieszczenie ich w bazie statycznej

      algorytm:
      - szukamy odmiany, ktora pasuje do podanego opisu,
      - sprawdzamy, czy taka odmiana (dokladnie taka) jest w bazie statycznej:
         - jesli jest, to pomijamy ja,
         - jesli nie ma, to dodajemy ja,
      */

      // szukamy odmiany, ktora pasuje do podanego opisu,
      for (wgen=handle->language.wordgenerator;wgen!=NULL;wgen=wgen->next)
      {
         // katalizator: sprawdzamy (nie) zgodnosc wyrazu podstawowego
         if (podst)
         {
            // musi miec zgodna koncowke
            // jesli nie ma, to nastepny
            if (strlen(podst)<strlen(wgen->chars)) continue;
            if (strcmp(podst+strlen(podst)-strlen(wgen->chars),wgen->chars)!=0) continue;
         }

         for (odm=wgen->odmiany;odm!=NULL;odm=odm->next)
         {
            // sprawdzamy zgodnosc opisu fleksyjnego
            if (dane)
            {
               // jesli to zestaw odmian z parametrem pozycyjnym
               if (glo_count_parts(handle,odm->odmiany,',')>1)
               {
                  // to sprawdzamy bez niego
                  // jesli nie sa zgodne, to nastepne odmiany
                  if (strlen(dane)-1!=strlen(odm->opis)) continue;
                  if (dane_strncmp(dane,odm->opis,strlen(dane)-1)!=0) continue;
               }
               // w przeciwnym przypadku
               else
               {
                  // sprawdzamy calosc
                  // jesli nie sa zgodne, to nastepne odmiany
						if (strlen(dane)!=strlen(odm->opis)) continue;
                  if (dane_strncmp(dane,odm->opis,strlen(dane))!=0) continue;
               }
            }

            // katalizator: sprawdzamy (nie) zgodnosc wyrazu odmienionego
            if (wyraz)
            {
               for (nr1=0,ile=glo_count_parts(handle,odm->odmiany,',');nr1<ile;nr1++)
               {
                  if (ile>1) glo_get_part(odm->odmiany,nr1+1,temp1,',');
                  else strcpy(temp1,odm->odmiany);

                  // sprawdzamy zgodnosc przedrostka: (przedrostek)-koncowka
                  if (temp1[0]=='(')
                  {
                     glo_get_part(temp1,1,temp2,'-');
                     if (strncmp(wyraz,temp2+1,strlen(temp2)-2)==0) goto PRZEDR_OK;
                  }
                  else goto PRZEDR_OK;
                  // jesli przedrostek nie jest zgodny, to nastepna pozycja odmiany
                  continue;
                  PRZEDR_OK:;

                  // sprawdzamy zgodnosc koncowki
                  if (temp1[0]=='(') glo_get_part(temp1,2,temp2,'-');
                  else strcpy(temp2,temp1+1);
                  if (strlen(temp2)>0 && strlen(wyraz)>=strlen(temp2))
                  {
                     if (strcmp(wyraz+strlen(wyraz)-strlen(temp2),temp2)==0) goto KATAL_WYRAZ_OK;
                  }
                  // jesli koncowka nie jest zgodna, to nastepna pozycja odmiany
                  continue;
               }
               // jesli nie ma zgodnosci, to nastepny zestaw
               continue;
               KATAL_WYRAZ_OK:;
            }

            // skoro dotarlismy tutaj, to istnieje prawdopodobienstwo dodania odmiany
            // pobieramy kolejne wyrazy podstawowe
            ile=glo_count_parts(handle,odm->odmiany,',');
            for (words=wgen->words;words!=NULL;words=words->next)
            {
               // jesli podano wyraz podstawowy, to musi byc zgodny - jesli nie podano, to nic
               if (podst)
                  if (strcmp(podst,words->word)!=0 || strlen(podst)!=strlen(words->word)) continue;

               // tworzymy temat
               strcpy(temat,words->word);
               temat[strlen(temat)-strlen(wgen->chars)]='\0';

               // pobieramy kolejne pozycje
               for (nr1=0;nr1<ile;nr1++)
               {
                  // jesli podano dane, to pozycja musi byc zgodna - jesli nie podano, to nic
                  if (ile>1)
                  {
                     if (dane)
							{
								if (dane[strlen(dane)-1]!='?')
								{
									if (dane[strlen(dane)-1]-'A'!=nr1) continue;
								}
							}
                     glo_get_part(odm->odmiany,nr1+1,temp1,',');
                  }
                  else strcpy(temp1,odm->odmiany);

                  // tworzymy odmiane
                  temp2[0]='\0';
                  // dodajemy przedrostek
                  if (temp1[0]=='(')
                  {
                     glo_get_part(temp1,1,temp2,'-');
                     glo_get_part(temp1,2,temp3,'-');
                  }
                  else strcpy(temp3,temp1+1);

                  // dodajemy "temat" i koncowke
                  strcat(temp2,temat);
//printf("temat: !%s!\n",temat);
                  strcat(temp2,temp3);

                  // jesli podano wyraz odmieniony, to musi byc zgodny - jesli nie podano, to nic
                  if (wyraz)
                  {
                     if (strcmp(wyraz,temp2)!=0 || strlen(wyraz)!=strlen(temp2)) continue;
//printf("!%s!=!%s!\n",wyraz,temp2);
                  }

                  // dodajemy wyraz do bazy
//printf("add: !%s!%s!%s!%d z %d\n",temp2,words->word,odm->opis,nr1+1,ile);

                  dodaj_wyraz(handle,&handle->language.voctemp,temp2,words->word,odm->opis,nr1+1,ile);
               }
            }
         }
      }
   }

   fleknum=0;
   allfleks=NULL;
   allfleks=(struct Fleksja**) realloc(allfleks,sizeof(struct Fleksja*)*fleknum+1);
   allfleks[fleknum]=NULL;
   for(fleks=voc_find_word(handle,handle->language.voctemp,podst,wyraz,dane,tryb);fleks!=NULL;fleks=voc_find_word(handle,fleks->next,podst,wyraz,dane,tryb)) {
     allfleks[fleknum]=fleks;
     fleknum++;
     allfleks=(struct Fleksja**) realloc(allfleks,sizeof(struct Fleksja*)*(fleknum+1));
     allfleks[fleknum]=NULL;
   }

   for(fleks=voc_find_word(handle,handle->language.vocabulary,podst,wyraz,dane,tryb);fleks!=NULL;fleks=voc_find_word(handle,fleks->next,podst,wyraz,dane,tryb)) {
     allfleks[fleknum]=fleks;
     fleknum++;
     allfleks=(struct Fleksja**) realloc(allfleks,sizeof(struct Fleksja*)*(fleknum+1));
     allfleks[fleknum]=NULL;
   }

   return(allfleks);
}

struct Fleksja *voc_find_word(struct DeniseHandle *handle,struct Fleksja *start,char *podst,char *wyraz,char *dane,int tryb)
{
   /*
   funkcja szuka w bazie fleksji poczwszy od pozycji "start" wyrazu o podanej
   charakterystyce poprzez wyraz podstawowy, wyraz odmieniony i dane.
   w przypadku nie podania parametru (zapis: ""), zwracany jest pierwszy element
   bdcy zgodnym z podanymi,
   funkcja zwraca wskanik na odnalezion struktur lub NULL,

   tryb:
   - 0 - jeli wyraz skada si z wielu czci, to wymagana jest zgodno ich wszystkich
   - 1 - jeli wyraz w bazie skada si z wielu czci, to wymagane jest, aby
         chocia jedna z nich bya taka sama jak wyraz szukany
   */

   struct Fleksja *fleks;
   struct Odmiany *odm;
   char temp[1000],temp_wyraz[100],nr1,ile;

   // akceptowane dwa rodzaje parametrw: NULL lub ""
   if (podst) if (strlen(podst)==0) podst=NULL;
   if (wyraz) if (strlen(wyraz)==0) wyraz=NULL;
   if (dane) if (strlen(dane)==0) dane=NULL;

   sprintf(temp_wyraz,",%s,",wyraz);

   for (fleks=start;fleks!=NULL;fleks=fleks->next)
   {
      // zgodno wyrazu podstawowego
      if (podst)
      {
         if (tryb==0)
         {
            if (strcmp(fleks->podst,podst)!=0 || strlen(fleks->podst)!=strlen(podst)) continue;
            if (glo_what_first(handle,fleks->podst,podst,0)==2)
            {
               return NULL;
            }
         }
         else
         {
            if (glo_get_part_position(1,fleks->podst,podst,' ')==0) continue;
         }
      }

      for (odm=fleks->odm_head;odm!=NULL;odm=odm->next)
      {
         // zgodno danych
         if (dane)
         {
            // jesli to zestaw odmian z parametrem pozycyjnym
            if (glo_count_parts(handle,odm->odmiany,',')>1)
            {
               // to sprawdzamy bez niego
               // jesli nie sa zgodne, to nastepne odmiany
               if (strlen(dane)-1!=strlen(odm->opis)) continue;
               if (dane_strncmp(dane,odm->opis,strlen(dane)-1)!=0) continue;
            }
            // w przeciwnym przypadku
            else
            {
               // sprawdzamy calosc
               // jesli nie sa zgodne, to nastepne odmiany
					if (strlen(dane)!=strlen(odm->opis)) continue;
               if (dane_strncmp(dane,odm->opis,strlen(dane))!=0) continue;
            }
         }

         // zgodno wyrazu odmienionego
         if (wyraz)
         {
            if (tryb==0)
            {
               sprintf(temp,",%s,",odm->odmiany);
               if (strstr(temp,temp_wyraz)==NULL)
               {
                  continue;
               }
            }
            else
            {
               ile=glo_count_parts(handle,odm->odmiany,',');
               for (nr1=ile;nr1>0;nr1--)
               {
                  if (ile>1) glo_get_part(odm->odmiany,nr1,temp,',');
                  else strcpy(temp,odm->odmiany);
                  if (glo_get_part_position(1,temp,wyraz,' ')!=0)
                  {
                     return fleks;
                  }
               }
               continue;
            }
         }
         return fleks;
      }
   }

   return NULL;
}

void dodaj_wyraz(struct DeniseHandle *handle,struct Fleksja **fleks_head,char *wyraz,char *podst,char *dane,int pos,int ile_pos)
{
   /*
   - szukamy, czy jest ju wyraz podstawowy w bazie
      - jeli jest, to zapamitujemy wskanik,
      - jeli nie, to tworzymy tak grup i zapamitujemy wskanik,
   - szukamy, czy jest ju lista takich odmian (dane),
      - jeli jest, to zapamitujemy wskanik,
      - jeli nie, to tworzymy tak list (z odpowiedni liczb pozycji) i zapamitujemy wskanik,
   - dodajemy wyraz na odpowiedniej pozycji,

   po odznaczeniu ponizszego kodu w komentarzach program bedzie pytal o rodzaj
   akcji, w przypadku mozliwosci generowania odmiany,
   */

   struct Fleksja *fleks=NULL,*fleks_prev,*fleks_next;
   struct Odmiany *odm,*odm1;
   char *ptr,*ptr2,nr1,dlug;
   char temp[1000]; // musi pomiescic cig odmian

   fleks=voc_find_word(handle,*fleks_head,podst,"","",0);

   if (fleks==NULL)
   {
  	  fleks=malloc(sizeof(struct Fleksja));
			if (fleks==NULL) {printf("nie ma! database1.c\n");return;}
			fleks->odm_head=NULL;
   	  fleks->next=NULL;
   	  fleks->ile_list=1;

      fleks->podst=malloc(strlen(podst)+1);
      if (fleks->podst==NULL) {printf("nie ma 2! database1.c\n");return;}

      memcpy(fleks->podst,podst,strlen(podst)+1);
      odm=malloc(sizeof(struct Odmiany));
      if (odm==NULL) {printf("nie ma 3! database1.c\n");return;}

      odm->next=NULL;
      odm->odmiany=NULL;
      odm->opis=malloc(strlen(dane)+1);
      if (odm->opis==NULL) {printf("nie ma 4! database1.c\n");return;}
      
      memcpy(odm->opis,dane,strlen(dane)+1);

	temp[0]='\0';
      for (nr1=1;nr1<=ile_pos;nr1++)
      {
         if (nr1==pos) strcat(temp,wyraz);
         if (nr1<ile_pos) strcat(temp,",");
      }
      odm->odmiany=malloc(strlen(temp)+1);
      memcpy(odm->odmiany,temp,strlen(temp)+1);

      fleks->odm_head=odm;
      // szukamy fleks_next, ktra jest pierwsz DALSZ,
      // dajemy fleks_prev->next na fleks, a fleks->next na fleks_next
      // jeli fleks_next==*fleks_head, to fleks->next=*fleks_head, fleksja_head=fleks;

      fleks_prev=NULL;
      for (fleks_next=*fleks_head;fleks_next!=NULL;fleks_prev=fleks_next,fleks_next=fleks_next->next)
      {
         if (glo_what_first(handle,podst,fleks_next->podst,0)==1)
         {
            if (fleks_prev==NULL) *fleks_head=fleks;
            else fleks_prev->next=fleks;

            fleks->next=fleks_next;
            break;
         }
      }

      if (fleks_next==NULL && *fleks_head!=NULL)
      {
         fleks_prev->next=fleks;
         fleks->next=NULL;
      }
      else if (*fleks_head==NULL)
      {
         *fleks_head=fleks;
         fleks->next=NULL;
      }
   }
   else
   {
      for (odm=fleks->odm_head;odm!=NULL;odm=odm->next)
		{
         if (strcmp(odm->opis,dane)==0 && strlen(odm->opis)==strlen(dane)) break;
		}

      if (odm==NULL)
      {
//         printf("adding: %s\n",wyraz);

         fleks->ile_list=fleks->ile_list+1;

         odm=malloc(sizeof(struct Odmiany));
         if (odm==NULL) {printf("nie ma wody na pustyni... database1.c\n");return;}

         odm->opis=malloc(strlen(dane)+1);
         if (odm->opis==NULL) {printf("ala ma kota, a kot ma fiola... database1.c\n");return;}
         memcpy(odm->opis,dane,strlen(dane)+1);
		temp[0]='\0';
         for (nr1=1;nr1<=ile_pos;nr1++)
         {
            if (nr1==pos) strcat(temp,wyraz);
            if (nr1<ile_pos) strcat(temp,",");
         }
         odm->odmiany=malloc(strlen(temp)+1);
         memcpy(odm->odmiany,temp,strlen(temp)+1);

         odm1=fleks->odm_head;
         odm->next=odm1->next;
         odm1->next=odm;
         fleks->odm_head=odm1;
      }
      else
      {
//         printf("filling: %s\n",wyraz);

         ptr=odm->odmiany;
         for (nr1=1;nr1<pos;nr1++)
         {
            ptr=strchr(ptr,',');
            if (ptr==NULL)
            {
               printf("ja sie zastrzele!!! w database1.c\n");
               return;
            }
            ptr=ptr+1;
         }

         ptr2=strchr(ptr,',');
/*
         if (ptr2==NULL)
         {
		 //ptr2=NULL czyli albo w ptr nie bylo przecinkow, albo
		 //jestesmy na ostatnim przecinku
            printf("panikaaaaa! database1.c\n");
            return;
         }
*/
	 if(ptr2==NULL) {
		 dlug=strlen(ptr);
 	 } else {
         	dlug=strlen(ptr)-strlen(ptr2);
	 }

         if (dlug==0)
         {
            sprintf(temp,"%.*s%s%s",(int) strlen(odm->odmiany)-strlen(ptr),odm->odmiany,wyraz,ptr);
//printf("temp: !%s!\n",temp);
            if (odm->odmiany!=NULL) free(odm->odmiany);
            odm->odmiany=malloc(strlen(temp)+1);
            if (odm->odmiany==NULL)
            {
               printf("ide spac... database1.c\n");
               return;
            }
            memcpy(odm->odmiany,temp,strlen(temp)+1);
         }
         else
         {
/*            printf("istnieje ju wpis: %.*s (o-overwrite/c-cancel)\n",dlug,ptr);
            fflush(stdin);
            switch (getchar())
            {
               case 'o':
                  printf("overwrite\n");
                  sprintf(temp,"%.*s%s%s",strlen(odm->odmiany)-strlen(ptr),odm->odmiany,wyraz,ptr+dlug);
                  free(odm->odmiany);
                  odm->odmiany=malloc(strlen(temp)+1);
                  memcpy(odm->odmiany,temp,strlen(temp)+1);
                  break;
               default:
                  printf("operation canceled\n");
                  break;
            }
*/
         }
      }
   }
}

// ***************************************************************************************
// EXTERN                              void ???
// ***************************************************************************************
struct Odmiany *dat_find_odm(struct DeniseHandle *handle,struct Odmiany *start,char *wyraz,char *dane,int tryb)
{
   /*
   funkcja szuka w wyrazu odmienionego o podanych parametrach, poczawszy od pozycji "start"
   w przypadku nie podania parametru (zapis: "" lub NULL), zwracany jest pierwszy element
   bedacy zgodnym z podanymi,
   funkcja zwraca wskaznik na odnaleziona strukture lub NULL,

   tryb:
   - 0 - jesli wyraz sklada sie z wielu czesci, to wymagana jest zgodnosc ich wszystkich
   - 1 - jesli wyraz w bazie sklada sie z wielu czesci, to wymagane jest, aby
         chociaz jedna z nich byla taka sama jak wyraz szukany
   */

   struct Odmiany *odm;
   char temp[1000]; // musi pomiescic ciag odmian
   char temp_wyraz[100]; // musi pomiescic 1 wyraz
   char nr1;

   // akceptowane dwa rodzaje parametrow: NULL lub ""
   if (wyraz) if (strlen(wyraz)==0) wyraz=NULL;
   if (dane) if (strlen(dane)==0) dane=NULL;

   sprintf(temp_wyraz,",%s,",wyraz);

   for (odm=start;odm!=NULL;odm=odm->next)
   {
      // zgodnosc danych
      if (dane)
		{
			if (strlen(odm->opis)!=strlen(dane)) continue;
			if (dane_strncmp(odm->opis,dane,strlen(dane))!=0) continue;
		}

      // zgodnosc wyrazu odmienionego
      if (wyraz)
      {
         if (tryb==0)
         {
            sprintf(temp,",%s,",odm->odmiany);
            if (strstr(temp,temp_wyraz)==NULL) continue;
         }
         else
         {
            for (nr1=glo_count_parts(handle,odm->odmiany,',');nr1>0;nr1--)
            {
               glo_get_part(odm->odmiany,nr1,temp,',');
               if (glo_get_part_position(1,temp,wyraz,' ')!=0) return odm;
            }
            continue;
         }
      }

      return odm;
   }

   return NULL;
}
