// ***************************************************************************************
//                                  		Grammar
// ***************************************************************************************

#include "denise.h"
#include "denise_protos.h"

// deklaracje prywatne
static struct FactSimple *tygrys_pozeracz(struct DeniseHandle* handle,struct FactSimple *fs_head,char *logic);
static BOOL parent_of(struct FSentence *sent,int group,int parent);
static int group_split(struct DeniseHandle* handle,struct FSentence *sent,int nr_grupy,char *con_type);
static struct FactSimple *fsclone(struct FactSimple *fs);
static int gra_zgodnosc(char *opis,char parametr);
static void print_grupa(struct DeniseHandle *handle,struct FactSimple *fs,char *txt,int nr_grupy);
static void rysuj_strukture(struct DeniseHandle* handle,struct FactSimple *fs,int margines,int nr_grupy);
static int get_poziom(struct FSentence *sent,int grupa);
static void get_words_of(struct FSentence *sent,int grupa,char *res);

// ***************************************************************************************
// EXTERN                BOOL gra_conjunctions_analysis(handle,fact)
// ***************************************************************************************
extern BOOL gra_conjunctions_analysis(struct DeniseHandle* handle,struct Fact *fact)
{
	/*
	funkcja ma za zadanie przeksztalcic podany fakt z faktami prostymi na postac,
	z faktami prostymi bez mozliwych do rozdzielenia wyrazen z wyrazami
	fleksyjnymi polaczonymi spojnikami, tzn. chodzi o rozdzielenie polaczen:
	"marek albo darek", "mily i sympatyczny" na odpowiednie, kompletne zdania
	polaczone tymi spojnikami, ale z grupami bez takich polaczen.

	postulaty:
	1) jesli nie ma wyrazenia podrzednego do calej grupy, to grupa moze (i jesli
		moze - to musi) byc rozdzielona,
	2) nie jest wazne, do ktorego wyrazu podrzedna jest grupa; podzial wyrazow
		oparty jest na ulozeniu wyrazow w zdaniu, i nie wykorzystuje tych danych - wazne
		jest tylko to, czy chodzi o podrzednosc do calej grupy, czy TYLKO konkretnego
		wyrazu,

	algorytm:
	- tworzymy jeden ciag opisu logicznego,
	- wysylamy liste faktow prostych i ich opis logiczny do funkcji pozerajacej fakty proste,
	- tworzymy klauzule - dokonujemy uproszczenia zwroconego przez funkcje opisu logicznego,
	- zapamietujemy nowy opis w fakcie,
	*/

	struct FactSimple *fs;
	char logic[1000],temp[1000],symbol;
	int nr1;

	// tworzymy jeden ciag opisu logicznego i rzucamy go funkcji rekurencyjnej na pozarcie
	logic[0]='\0';
	sprintf(temp,"%d",CON_AND);
	for (nr1=0;nr1<fact->ile_klauzul;nr1++)
	{
		if (nr1>0) strcat(logic,temp);
		strcat(logic,"(");
		strcat(logic,fact->logic[nr1]);
		strcat(logic,")");
	}

//DeniseMessage(handle,"LOGIC: %s",logic);
//for (fs=fact->fs;fs!=NULL;fs=fs->next)
//{
//	DeniseReport(handle,"ID: %s",fs->sentence->id_subject);
//	rysuj_strukture(handle,fs,0,-1);
//}
//DeniseReport(handle,"END");
	// wysylamy liste faktow prostych i ich opis logiczny do funkcji rekurencyjnej,
	if ((fs=tygrys_pozeracz(handle,fact->fs,logic))==NULL) return FALSE;
	fact->fs=fs;
//DeniseReport(handle,"LOGIC: END");
//return TRUE;

//sprintf(logic,"A%dB",CON_AND);

//DeniseMessage(handle,"LOGIC RESULT: %s",logic);
	// tworzymy klauzule - dokonujemy uproszczenia zwroconego przez funkcje opisu logicznego,
   log_expr_simplification(logic,temp);
//DeniseMessage(handle,"2: %s",temp);
   log_expr2clause(handle,temp,logic);
//DeniseMessage(handle,"3: %s",logic);
	strcpy(temp,logic);

//DeniseMessage(handle,"A ILE KLAUZUL: %d",handle->cache.facts->ile_klauzul);

//DeniseMessage(handle,"!%s!",logic);

//return FALSE;
//printf("1\n");

	// zapamietujemy nowy opis w fakcie,
	free(fact->logic);
   fact->ile_klauzul=glo_count_parts(handle,logic,',');
   fact->logic=(char **) malloc(fact->ile_klauzul*sizeof(char *));

//DeniseMessage(handle,"B ILE KLAUZUL: %d",handle->cache.facts->ile_klauzul);

   for (nr1=0;nr1<fact->ile_klauzul;nr1++)
   {
      glo_get_part(logic,nr1+1,temp,',');
      fact->logic[nr1]=(char *) malloc(strlen(temp)+1);
      memcpy(fact->logic[nr1],temp,strlen(temp)+1);
   }
//printf("2\n");
//return FALSE;

	if (handle->viewparams.conjunctions_analysis==ON)
   {
		DeniseReport(handle,"LOGIC: %s",logic);
		symbol='A';
		for (fs=fact->fs;fs!=NULL;fs=fs->next,symbol++)
		{
			DeniseReport(handle,"%c",symbol);
		   rysuj_strukture(handle,fs,0,-1);
		}
   }

	return TRUE;
}

static struct FactSimple *tygrys_pozeracz(struct DeniseHandle* handle,struct FactSimple *fs_head,char *logic)
{
	/*
	funkcja w miejsce faktu prostego wstawia ich kilka prostszych, bez wyrazen ze
	spojnikami itd., oraz wprowadza ich opis logiczny w tablicy logic wszedzie
	gdzie znajdowal sie symbol wczesniej wystepujacego pojedynczego faktu
	prostego, ale wczesniej uaktualnia numeracje

	algorytm:
	- dla kolejnych faktow prostych,
	- dla kolejnych grup danego faktu prostego,
	- sprawdzamy, na ile elementow grupe mozna podzielic:
		- jesli na wiecej niz 1 (liczba spojnikow>0):
			- dzielimy na te elementy i tworzymy nowe fakty proste,
			  dopisywane W MIEJSCE tego faktu z odpowiednimi spojnikami (z nawiasami),
			- uaktualniamy symbole logiczne w opisie logicznym calego zbioru,
			- wpisujemy nowa formule logiczna w miejsce starego fs,
			- wpisujemy nowy ciag fs1_head w miejsce starego fs
			- kasujemy stary fs
			- startujemy procedure od pierwszego nowego dopisanego faktu prostego,

	algorytm dzielenia: (nie jest doskonaly! patrz: marek ma zielony, rozwalony od wczoraj rower i starego psa)
	- tworzymy nowy fs,
	- kopiujemy do niego dzielony fs,
	- pobieramy z wyrazenia kolejne numery wyrazow fleksyjnych do usuniecia z danej grupy
		- odszukujemy kolejne wyrazy odseparowane z kasowanego wyrazu fleksyjnego w zdaniu,
			- zaznaczamy ten wyraz do skasowania,
			- zaznaczamy wszystkie wyrazy na lewo i prawo od tego wyrazu
			  odseparowanego w zdaniu, az natrafimy na wyraz z innej grupy lub spojnik
			  (spojnik rowniez zaznaczamy do skasowania),
	- kasujemy kolejne zaznaczone do skasowania wyrazy odseparowane:
		- kasujemy ze zdania uproszczonego zaznaczony wyraz odseparowany,
		- kasujemy z tablicy member dana pozycje,
		- kasujemy z tablicy zrost_poz,
		- kasujemy z tablicy priority,
		- kasujemy z tablicy fleksyjny,
		- zmniejszamy liczbe wyrazow odseparowanych,
	- kasujemy wyrazy fleksyjne, ktore pozostaly bez wyrazow odseparowanych:
		- kasujemy elementy z ciagu voc,
		- kasujemy elementy z ciagu base_words,
		- korygujemy zapisy kolejnych wyrazow fleksyjnych w tablicy fleksyjny,
		- zmniejszamy liczbe wyrazow fleksyjnych,
	- kasujemy grupy, ktore pozostaly bez wyrazow
		- kasujemy z tablicy parent,
      - kasujemy z tablicy parent_word,
      - kasujemy z ciagu danych fleksyjnych grupy,
		- korygujemy zapisy grup w member, parent, parent_word,
      - zmniejszamy liczbe grup,
	- nowe identyfikatory dla faktu fs

uwzgledniac przecinek! to przeciez tez jakis spojnik!
	*/

	struct FactSimple *fs,*fs_last,*fs1,*fs1_head,*fs1_last;
	int nr1,nr2,nr3,nr4,ile_sp,fleks_nr,part;
	int del[100]; // wyrazy odseparowane - 0 - zostaje, 1 - do usuniecia (numery 1 - pozycja 0)
	char con_type[100]; // typy spojnikow - tablica podreczna
	char temp[1000],temp1[1000],temp2[1000];
	char symbol; // znak aktualnego fs (tzn. 'A', 'B' itd.)

//printf("2\n");

	// dla kolejnych faktow prostych,
	fs=fs_head;
	symbol='A';
	START:
	fs_last=NULL;
	for (;fs!=NULL;fs_last=fs,fs=fs->next,symbol++)
	{
//printf("3\n");
		// dla kolejnych grup danego faktu prostego,
		for (nr1=0;nr1<fs->sentence->groups_number;nr1++)
		{
			// sprawdzamy, na ile elementow grupe mozna podzielic:
			ile_sp=group_split(handle,fs->sentence,nr1+1,con_type);

			// jesli na wiecej niz 1 (liczba spojnikow>0):
			if (ile_sp==0) continue;

			// dzielimy na te elementy i tworzymy nowe fakty proste,
			// dopisywane W MIEJSCE tego faktu z odpowiednimi spojnikami (z nawiasami),

			// dla kolejnych wybieranych elementow (part1 AND part2 AND part3...)
			fs1_head=NULL;
			for (part=1;part<=ile_sp+1;part++)
			{
//DeniseReport(handle,"**** part: %d",part);
				// algorytm dzielenia: (nie jest doskonaly! patrz: marek ma zielony, rozwalony od wczoraj rower i starego psa)
				// tworzymy nowy fs,
				// kopiujemy do niego dzielony fs,
				fs1=fsclone(fs);

				// ustalamy grupe na POS
				fs->sentence->posneg[nr1]=TWIERDZENIE;
				fs1->sentence->posneg[nr1]=TWIERDZENIE;

//DeniseReport(handle,"orig (words: %d)",fs1->sentence->words_number);
//rysuj_strukture(handle,fs,0,-1);
//rysuj_strukture(handle,fs1,0,-1);
//DeniseReport(handle,"orig");

				// pobieramy z wyrazenia kolejne numery wyrazow fleksyjnych do usuniecia z danej grupy
				for (nr4=0;nr4<fs1->sentence->words_number;nr4++) del[nr4]=0;
				fleks_nr=0;
				for (nr2=0;nr2<fs1->sentence->ile_fleks;nr2++)
				{
					// czy ten wyraz fleksyjny nalezy do wyrazenia i ma byc usuniety?
					for (nr3=0;nr3<fs1->sentence->words_number;nr3++)
						if (fs1->sentence->fleksyjny[nr3]==nr2+1 && fs1->sentence->member[nr3]==nr1+1) goto FOUNDED;
					continue;

					// nalezy do grupy, ale czy ma zostac usuniety?
					FOUNDED:

					// nie liczymy spojnikow w grupie jako elementow
					glo_get_part(fs1->sentence->voc,nr2+1,temp,',');
					if (gra_conjunction_type(handle,NULL,temp)==NO_CON)
					{
						// nie liczymy tez zaprzeczen
						glo_get_part(fs1->sentence->base_words,nr3+1,temp,',');
						if (temp[0]!='!') fleks_nr++;
					}
					else continue;
					if (fleks_nr==part || (fleks_nr+1==part && temp[0]=='!'))
					{
						if (temp[0]=='!')
						{
							fs1->sentence->posneg[nr1]=PRZECZENIE;
						}

  	 					// kasujemy spojnik z lewej lub prawej z tej grupy

						// LEWO
						for (nr4=nr3-1;nr4>=0;nr4--)
						{
							// czy ten wyraz nalezy do innej grupy?
//glo_get_part(fs1->sentence->words,nr4+1,temp,' ');
//DeniseReport(handle,"TEST LEFT: %s (%d) (member: %d, oczekuje: %d) (nr4: %d)",temp,nr4+1,fs1->sentence->member[nr4],nr1+1,nr4+1);

							if (fs1->sentence->member[nr4]!=nr1+1) break;
//							if (parent_of(fs1->sentence,fs1->sentence->member[nr4],nr1+1)==FALSE) break;

							// czy ten wyraz to spojnik z dzielonej grupy?
							glo_get_part(fs1->sentence->voc,fs1->sentence->fleksyjny[nr4],temp,',');
							if (gra_conjunction_type(handle,NULL,temp)!=NO_CON)
							{
				  				// zaznaczamy do kasowania
								del[nr4]=1;
//glo_get_part(fs1->sentence->words,nr4+1,temp,' ');
//DeniseReport(handle,"LEFT: %s (%d)",temp,nr4+1);
								break;
							}
						}

						// PRAWO
						for (nr4=nr3+1;nr4<fs1->sentence->words_number;nr4++)
						{
							// czy ten wyraz nalezy do innej grupy?
//glo_get_part(fs1->sentence->words,nr4+1,temp,' ');
//DeniseReport(handle,"TEST RIGHT: %s (%d) (member: %d, oczekuje: %d) (nr4: %d)",temp,nr4+1,fs1->sentence->member[nr4],nr1+1,nr4+1);
							if (fs1->sentence->member[nr4]!=nr1+1) break;
//							if (parent_of(fs1->sentence,fs1->sentence->member[nr4],nr1+1)==FALSE) break;

							// czy ten wyraz to spojnik z dzielonej grupy?
							glo_get_part(fs1->sentence->voc,fs1->sentence->fleksyjny[nr4],temp,',');
							if (gra_conjunction_type(handle,NULL,temp)!=NO_CON)
							{
				  				// zaznaczamy do kasowania
								del[nr4]=1;
//glo_get_part(fs1->sentence->words,nr4+1,temp,' ');
//DeniseReport(handle,"RIGHT: %s (%d)",temp,nr4+1);
								break;
							}
						}

               			continue;
					}

					// w przeciwnym razie ma zostac usuniety
					// odszukujemy kolejne wyrazy odseparowane z kasowanego wyrazu fleksyjnego w zdaniu,
					for (nr3=0;nr3<fs1->sentence->words_number;nr3++)
					{
						if (fs1->sentence->fleksyjny[nr3]!=nr2+1) continue;

						// zaznaczamy ten wyraz do skasowania,
						del[nr3]=1;
//glo_get_part(fs1->sentence->words,nr3+1,temp,' ');
//DeniseReport(handle,"DEL WORD: %s (%d)",temp,nr3+1);


						// zaznaczamy wszystkie wyrazy na lewo i prawo od tego wyrazu
						// odseparowanego w zdaniu, az natrafimy na wyraz z innej grupy lub spojnik
						// (spojnik rowniez zaznaczamy do skasowania),

						// LEWO
//DeniseReport(handle,"del: left");
						for (nr4=nr3-1;nr4>=0;nr4--)
						{
							// czy ten wyraz nalezy do innej grupy?
//glo_get_part(fs1->sentence->words,nr4+1,temp,' ');
//DeniseReport(handle,"TEST LEFT: %s (%d) (member: %d, oczekuje: %d) (nr4: %d)",temp,nr4+1,fs1->sentence->member[nr4],nr1+1,nr4+1);

							if (fs1->sentence->member[nr4]!=nr1+1) break;
//							if (parent_of(fs1->sentence,fs1->sentence->member[nr4],nr1+1)==FALSE) break;

//glo_get_part(fs1->sentence->words,nr4+1,temp,' ');
//DeniseReport(handle,"LEFT: %s (%d)",temp,nr4+1);

							// zaznaczamy do kasowania
							del[nr4]=1;

							// czy ten wyraz to spojnik z dzielonej grupy?
							glo_get_part(fs1->sentence->voc,fs1->sentence->fleksyjny[nr4],temp,',');
							if (gra_conjunction_type(handle,NULL,temp)!=NO_CON) break;
						}

						// PRAWO
//DeniseReport(handle,"del: right (words: %d)",fs1->sentence->words_number);
						for (nr4=nr3+1;nr4<fs1->sentence->words_number;nr4++)
						{
							// czy ten wyraz nalezy do innej grupy?
//glo_get_part(fs1->sentence->words,nr4+1,temp,' ');
//DeniseReport(handle,"TEST RIGHT: %s (%d) (member: %d, oczekuje: %d) (nr4: %d)",temp,nr4+1,fs1->sentence->member[nr4],nr1+1,nr4+1);
							if (fs1->sentence->member[nr4]!=nr1+1) break;
//							if (parent_of(fs1->sentence,fs1->sentence->member[nr4],nr1+1)==FALSE) break;

//glo_get_part(fs1->sentence->words,nr4+1,temp,' ');
//DeniseReport(handle,"RIGHT: %s (%d)",temp,nr4+1);

							// zaznaczamy do kasowania
							del[nr4]=1;

							// czy ten wyraz to spojnik z dzielonej grupy?
							glo_get_part(fs1->sentence->voc,fs1->sentence->fleksyjny[nr4],temp,',');
							if (gra_conjunction_type(handle,NULL,temp)!=NO_CON) break;
						}
					}
				}
				// kasujemy kolejne zaznaczone do skasowania wyrazy odseparowane: (od ostatniego!)
				for (nr2=fs1->sentence->words_number-1;nr2>=0;nr2--)
				{
					// czy wyraz ma zostac usuniety?
					if (del[nr2]==0) continue;

					// kasujemy ze zdania uproszczonego zaznaczony wyraz odseparowany,
					glo_delete_part_sep(handle,fs1->sentence->words,nr2+1,' ');

					for (nr3=nr2;nr3<fs1->sentence->words_number;nr3++)
					{
						// kasujemy z tablicy member dana pozycje,
						fs1->sentence->member[nr3]=fs1->sentence->member[nr3+1];
						// kasujemy z tablicy zrost_poz,
						fs1->sentence->zrost_poz[nr3]=fs1->sentence->zrost_poz[nr3+1];
						// kasujemy z tablicy priority,
						fs1->sentence->priority[nr3]=fs1->sentence->priority[nr3+1];
						// kasujemy z tablicy fleksyjny,
						fs1->sentence->fleksyjny[nr3]=fs1->sentence->fleksyjny[nr3+1];
					}

					// zmniejszamy liczbe wyrazow odseparowanych,
					fs1->sentence->words_number=fs1->sentence->words_number-1;
				}

				// kasujemy wyrazy fleksyjne, ktore pozostaly bez wyrazow odseparowanych:
				for (nr2=fs1->sentence->ile_fleks-1;nr2>=0;nr2--)
				{
					// czy wyraz ma zostac usuniety?
					for (nr3=0;nr3<fs1->sentence->words_number;nr3++)
						if (fs1->sentence->fleksyjny[nr3]==nr2+1) goto NEXT_FLEKS;

					// kasujemy elementy z ciagu voc,
					glo_delete_part_sep(handle,fs1->sentence->voc,nr2+1,',');

					// kasujemy elementy z ciagu base_words,
					glo_delete_part_sep(handle,fs1->sentence->base_words,nr2+1,',');

					// korygujemy zapisy kolejnych wyrazow fleksyjnych w tablicy fleksyjny,
					for (nr3=0;nr3<fs1->sentence->words_number;nr3++)
						if (fs1->sentence->fleksyjny[nr3]>nr2+1) fs1->sentence->fleksyjny[nr3]=fs1->sentence->fleksyjny[nr3]-1;

					// zmniejszamy liczbe wyrazow fleksyjnych,
					fs1->sentence->ile_fleks=fs1->sentence->ile_fleks-1;

					NEXT_FLEKS:;
				}

				// kasujemy grupy, ktore pozostaly bez wyrazow
            // oraz grupy im podrzedne
				for (nr4=0;nr4<fs1->sentence->groups_number;nr4++) del[nr4]=0;
				for (nr2=0;nr2<fs1->sentence->groups_number;nr2++)
				{
            		// czy sa jakies wyrazy w tej grupie?
					for (nr3=0;nr3<fs1->sentence->words_number;nr3++)
               			if (fs1->sentence->member[nr3]==nr2+1) goto GROUP_EXIST;

					// zaznaczamy do skasowania
					del[nr2]=1;

					// i wszystkie grupy jej podrzedne
					for (nr3=0;nr3<fs1->sentence->groups_number;nr3++)
               			if (parent_of(fs1->sentence,nr3+1,nr2+1)==TRUE) del[nr3]=1;

					GROUP_EXIST:;
				}

				for (nr2=fs1->sentence->groups_number-1;nr2>=0;nr2--)
				{
            		if (del[nr2]==0) continue;

//DeniseReport(handle,"DEL: %d",nr2+1);
					for (nr3=nr2;nr3<fs1->sentence->groups_number;nr3++)
					{
						// kasujemy z tablicy parent,
               			fs1->sentence->parent[nr3]=fs1->sentence->parent[nr3+1];

						// kasujemy z tablicy parent_word,
               			fs1->sentence->parent_word[nr3]=fs1->sentence->parent_word[nr3+1];
					}
					// kasujemy z ciagu danych fleksyjnych grupy,
					glo_delete_part_sep(handle,fs1->sentence->group_voc,nr2+1,',');

					// korygujemy zapisy grup w member, parent, parent_word,
					for (nr3=0;nr3<fs1->sentence->groups_number;nr3++)
					{
               			if (fs1->sentence->member[nr3]>nr2+1) fs1->sentence->member[nr3]--;
               			if (fs1->sentence->parent[nr3]>nr2+1) fs1->sentence->parent[nr3]--;
               			if (fs1->sentence->parent_word[nr3]>nr2+1) fs1->sentence->parent_word[nr3]--;
					}
					// zmniejszamy liczbe grup,
					fs1->sentence->groups_number--;
				}

//DeniseReport(handle,"result groups: %d",fs1->sentence->groups_number);
//rysuj_strukture(handle,fs,0,-1);
//rysuj_strukture(handle,fs1,0,-1);
//DeniseReport(handle,"result END");

				// nowe identyfikatory dla faktu fs
				ide_get_fsent_id(handle,fs1->sentence,temp1,ID_FROM_SUBJECT);
				if (strlen(temp1)>0)
				{
     				fs1->sentence->id_subject=malloc(strlen(temp1)+1);
    				strcpy(fs1->sentence->id_subject,temp1);
				}

				ide_get_fsent_id(handle,fs1->sentence,temp1,ID_FROM_OTHER);
  				if (strlen(temp1)>0)
        		{
					fs1->sentence->id_global=malloc(strlen(temp1)+1);
					strcpy(fs1->sentence->id_global,temp1);
     			}

//DeniseReport(handle,"AAAAAAAA");
//rysuj_strukture(handle,fs,0,-1);
//rysuj_strukture(handle,fs1,0,-1);
//DeniseReport(handle,"BBBBBBBB");
//rysuj_strukture(handle,fs1,0,-1);
				if (fs1_head==NULL) fs1_head=fs1;
				else fs1_last->next=fs1;
				fs1_last=fs1;
				fs1->next=NULL;

//DeniseReport(handle,"SUPER");
//rysuj_strukture(handle,fs1,0,-1);
//DeniseReport(handle,"SUPER END");

			}

//return FALSE;

			// okreslamy polaczenia logiczne miedzy nowymi faktami,
			// pamietajac o odpowiedniej numeracji
			// pakujemy nawias otwierajacy
			sprintf(temp,"(");
			for (part=1;part<=ile_sp+1;part++)
			{
				// jesli to nie jest pierwszy element to spojnik
				if (part>1)
				{
					sprintf(temp1,"%d",con_type[part-2]);
					strcat(temp,temp1);
				}
				// numer fs
				sprintf(temp1,"%c",symbol+part-1);
				strcat(temp,temp1);
			}
			// pakujemy nawias zamykajacy
			sprintf(temp1,")");
			strcat(temp,temp1);

//printf("TEMP: !%s!\n",temp);

			// uaktualniamy symbole logiczne w opisie logicznym calego zbioru,
			// trzeba przesunac wszystkie symbole > od aktualnego o tyle
			// ile jest elementow 2*ile_sp-1 (-1 bo wykorzystujemy takze
			// symbol aktualnego fs)
			for (nr2=0;nr2<(int) strlen(logic);nr2++)
				if (logic[nr2]>symbol) logic[nr2]=logic[nr2]+2*ile_sp-1;

			// wpisujemy nowa formule logiczna w miejsce starego fs,
			// na wszystkich miejscach, w ktorych wystepowal
			temp2[0]='\0';
			for (nr2=0;nr2<(int) strlen(logic);nr2++)
			{
				if (logic[nr2]==symbol) sprintf(temp1,"%s",temp);
				else sprintf(temp1,"%c",logic[nr2]);
				strcat(temp2,temp1);
			}
			strcpy(logic,temp2);

			// wpisujemy nowy ciag fs1_head w miejsce starego fs
			if (fs==fs_head)
			{
         		fs1->next=fs_head->next;
         		fs_head=fs1_head;
			}
			else fs_last->next=fs1_head;
			fs_last=fs1;

			// kasujemy stary fs
			dat_fact_simple_free(fs);
			free(fs);

			// startujemy procedure od pierwszego nowego dopisanego faktu prostego,
			// stad - symbol nie zmienia sie - jest aktualny, ten sam
			fs=fs_head;
			goto START;
		}
	}

	return fs_head;
}

static BOOL parent_of(struct FSentence *sent,int group,int parent)
{
	// funkcja sprawdza, czy grupa group jest parent lub jest podrzedna do parent
  	for (;;)
   {
		if (group==parent) return TRUE;
		if (sent->parent[group-1]==0) return FALSE;
   	group=sent->parent[group-1];
   }
}

static int group_split(struct DeniseHandle* handle,struct FSentence *sent,int nr_grupy,char *con_type)
{
	/*
	funkcja sprawdza na ile wyrazen mozna podzielic dana grupe,
	zwraca liczbe spojnikow

	algorytm:
	<etap 1>
	- szukamy grup podrzednych do tej grupy,
	- jesli istnieje grupa podrzedna podrzedna do CALEJ grupy, to RETURN 0
	<etap 2>
	- liczymy spojniki w grupie
	- jesli nie ma spojnika to RETURN 0
	- jesli sa spojniki to zwracamy wartosc: liczba_spojnikow
	*/

	char temp[100];
	int nr1,ile;

	// <etap 1>
	// szukamy grup podrzednych do tej grupy,
	for (nr1=0;nr1<sent->groups_number;nr1++)
	{
		// jesli istnieje grupa podrzedna podrzedna do CALEJ grupy, to RETURN 0
		if (sent->parent[nr1]==nr_grupy && sent->parent_word[nr1]==0) return 0;
	}

	// <etap 2>
	// liczymy spojniki w grupie (zakladamy, ze spojniki to pojedyncze wyrazy odseparowane!)
	ile=0;
	for (nr1=0;nr1<sent->words_number;nr1++)
	{
		if (sent->member[nr1]!=nr_grupy) continue;

		glo_get_part(sent->voc,sent->fleksyjny[nr1],temp,',');

		// czy to spojnik?
		con_type[ile]=gra_conjunction_type(handle,NULL,temp);
		if (con_type[ile]!=NO_CON) ile++;
	}

	// jesli nie ma spojnika to RETURN 1
	if (ile==0) return 0;

	// jesli sa spojniki to zwracamy wartosc: liczba_spojnikow
	return ile;
}

static struct FactSimple *fsclone(struct FactSimple *fs)
{
	struct FactSimple *fs1;
	int nr1;

	fs1=malloc(sizeof(struct FactSimple));
	fs1->next=NULL;
	fs1->quality=fs->quality;
	strcpy(fs1->databaseID,fs->databaseID);
	fs1->theme_number=fs->theme_number;
	fs1->sentence=malloc(sizeof(struct FSentence));

	fs1->sentence->language=malloc(strlen(fs->sentence->language)+1);
	strcpy(fs1->sentence->language,fs->sentence->language);

	fs1->sentence->id_subject=NULL;
	fs1->sentence->id_global=NULL;

	fs1->sentence->orig=malloc(strlen(fs->sentence->orig)+1);
	strcpy(fs1->sentence->orig,fs->sentence->orig);

	fs1->sentence->words=malloc(strlen(fs->sentence->words)+1);
	strcpy(fs1->sentence->words,fs->sentence->words);

	fs1->sentence->base_words=malloc(strlen(fs->sentence->base_words)+1);
	strcpy(fs1->sentence->base_words,fs->sentence->base_words);

	fs1->sentence->voc=malloc(strlen(fs->sentence->voc)+1);
	strcpy(fs1->sentence->voc,fs->sentence->voc);

	if (fs->sentence->group_voc==NULL) fs1->sentence->group_voc=NULL;
	else
	{
		fs1->sentence->group_voc=malloc(strlen(fs->sentence->group_voc)+1);
		strcpy(fs1->sentence->group_voc,fs->sentence->group_voc);
	}

	fs1->sentence->language=malloc(strlen(fs->sentence->language)+1);
	strcpy(fs1->sentence->language,fs->sentence->language);

	fs1->sentence->zrost_poz=malloc(fs->sentence->words_number+1);
	fs1->sentence->priority=malloc(fs->sentence->words_number+1);
	fs1->sentence->fleksyjny=malloc(fs->sentence->words_number+1);
	fs1->sentence->member=malloc(fs->sentence->words_number+1);
	for (nr1=0;nr1<fs->sentence->words_number;nr1++)
	{
		fs1->sentence->member[nr1]=fs->sentence->member[nr1];
		fs1->sentence->zrost_poz[nr1]=fs->sentence->zrost_poz[nr1];
		fs1->sentence->priority[nr1]=fs->sentence->priority[nr1];
		fs1->sentence->fleksyjny[nr1]=fs->sentence->fleksyjny[nr1];
	}

	fs1->sentence->parent=malloc(fs->sentence->groups_number+1);
	fs1->sentence->parent_word=malloc(fs->sentence->groups_number+1);
	fs1->sentence->function=malloc(fs->sentence->groups_number+1);
	fs1->sentence->posneg=malloc(fs->sentence->groups_number+1);
	for (nr1=0;nr1<fs->sentence->groups_number;nr1++)
	{
		fs1->sentence->parent[nr1]=fs->sentence->parent[nr1];
		fs1->sentence->parent_word[nr1]=fs->sentence->parent_word[nr1];
		fs1->sentence->function[nr1]=fs->sentence->function[nr1];
		fs1->sentence->posneg[nr1]=fs->sentence->posneg[nr1];
	}

	fs1->sentence->words_number=fs->sentence->words_number;
	fs1->sentence->ile_fleks=fs->sentence->ile_fleks;
	fs1->sentence->groups_number=fs->sentence->groups_number;

	return fs1;
}

// ***************************************************************************************
// EXTERN                int gra_wyrazenie_check(handle,char *params,char *opis,...)
// ***************************************************************************************
extern int gra_wyrazenie_check(struct DeniseHandle *handle,char *params,char *opis,...)
{
   /*
   funkcja sprawdza, czy podany opis wyrazenia dotyczy podanych dalej slow,
   tj. czy ich fleksja charaketryzuje sie parametrami jak okresla opis,

   params: opis fleksji+opis fleksji+...+opis fleksji
   opis: dane zakodowane: ABAAAC,ABCCA...

//   tryb:
//   0 - kolejnosc wyrazow moze byc dowolna i zostanie automatycznie dopasowana,
   1 - kolejnosc podanych wyrazow musi byczgodna z podana w opisie,

   dopuszczalny opis z parametrami: $nr, '/', ENV-nr itp.

   params - jesli rozne od NULL to:
            zawiera informacje o tym, czy wyrazy sa w grupach twierdzacych czy zanegowanych

   zwraca: 0 - rozne, 1 - zgodne,
   */

   char **arg,*opis_wyrazu,*param;
   int nr1,nr2,ile_wyrazow,ile_param;
   char temp[1000],korel[1000];
   int env;
   va_list ap;

   // rezerwujemy pamiec dla opisu pojedynczego wyrazu
   if ((opis_wyrazu=malloc(strlen(opis)+1))==NULL) return 0;

   // rezerwujemy pamiec dla pojedynczego parametru
   if ((param=malloc(strlen(opis)+1))==NULL) return 0;

   sprintf(korel,",");

   ile_wyrazow=glo_count_parts(handle,opis,'+');
   arg=(char **) malloc(ile_wyrazow*sizeof(char *)); // tablica z wyrazami
	if (arg==NULL || ile_wyrazow==0)
	{
DeniseReport(handle,"apokazlipsa! grammar2.c\n");
		return 0;
	}

   va_start(ap,opis);
   for (nr1=0;nr1<ile_wyrazow;nr1++) arg[nr1]=va_arg(ap,char *);

	if (!isdigit(opis[0]))
	{
		DeniseReport(handle,"error: zla kolejnosc parametrow w gra_wyrazenie_check");
		return 0;
	}

   // sprawdzenie zgodnosci opisu z podanymi odmianami
   for (nr1=0;nr1<ile_wyrazow;nr1++)
   {
      // pobieramy opis jednego wyrazu z opisu wszystkich
      glo_get_part(opis,nr1+1,opis_wyrazu,'+');

      // ilosc danych musi byc nie mniejsza niz liczba elementow opisu wyrazu
      ile_param=glo_count_parts(handle,opis_wyrazu,',');
      if ((int) strlen(arg[nr1])<ile_param)
      {
         if (arg) free(arg);
         free(opis_wyrazu);
         free(param);
         return 0;
      }

      for (nr2=0;nr2<ile_param;nr2++)
      {
         // pobieramy kolejne parametry z opisu
         glo_get_part(opis_wyrazu,nr2+1,param,',');

         // (ENV-nr) - oznacza, ze jesli nie jest ustawiona wewnetrzna zmienna nr,
         // to parametr nie jest brany pod uwage

         if (strncmp(param,"(ENV-",5)==0)
         {
            env=atoi(param+5);
            memmove(param,strchr(param,' ')+1,strlen(strchr(param,' ')));

            // podjecie decyzji: grupa twierdzaca/przeczaca
            if (params!=NULL)
				{
					if ((int) strlen(params)<nr1+1)
					{
printf("blad! params za krotkie! grammar1.c (123)");
						return 0;
					}
               if ((env==1 && params[nr1]!=TWIERDZENIE) || (env==2 && params[nr1]!=PRZECZENIE)) continue;
				}
         }

         // sprawdzenie parametrow podanych
         if (param[0]!='$')
         {
            if (gra_zgodnosc(param,arg[nr1][nr2])==0)
            {
               free(arg);
               free(opis_wyrazu);
               free(param);
               return 0;
            }
         }
         // sprawdzenie parametrow ustalonych (skorelowanych)
         else
         {
            // ,nr=cos,nr=cos,nr=cos,
            sprintf(temp,",%s=",param+1);
            // jesli parametr ten nie byl wczesniej identyfikowany, to dopisujemy go
            if (strstr(korel,temp)==NULL)
            {
               sprintf(temp,"%s=%c,",param+1,arg[nr1][nr2]);
               strcat(korel,temp);
            }
            // w przeciwnym razie sprawdzamy, czy parametry sa poprawnie skorelowane
            else
            {
               sprintf(temp,",%s=%c,",param+1,arg[nr1][nr2]);
               if (strstr(korel,temp)==NULL)
               {
                  free(arg);
                  free(opis_wyrazu);
                  free(param);
                  return 0;
               }
            }
         }
      }
   }

   free(arg);
   free(opis_wyrazu);
   free(param);
   return 1;
}

// ***************************************************************************************
// EXTERN                int gra_conjunction_type(handle,wyraz,dane)
// ***************************************************************************************
extern int gra_conjunction_type(struct DeniseHandle* handle,char *wyraz,char *dane)
{
   /*
   funkcja sprawdza, czy podany wyraz jest lacznikiem, i jesli jest, to zwraca jest typ
   w przeciwnym razie zwraca NO_CON (0)

   algorytm:
   - szukamy wyrazu w bazie fleksji,
   - sprawdzamy, czy dane sa zgodne z opisem spojnikow,

   dodac buforowanie!!!
   */

   struct GrammarData *gra;
	struct Fleksja *fleks,**allfleks;
   char posneg=TWIERDZENIE,opis[100];
   int nr1;

   if (dane==NULL)
	{
		allfleks=dat_find_word(handle,handle->language.vocabulary,"",wyraz,"",0);
		fleks=*allfleks;
		free(allfleks);
		if (fleks==NULL) return NO_CON;
		strcpy(opis,fleks->odm_head->opis);
	}
	else strcpy(opis,dane);

   // sprawdzamy dane w bazie gramatycznej
   for (nr1=0;nr1<4;nr1++)
      for (gra=handle->language.lo2[nr1];gra!=NULL;gra=gra->next)
      {
         // sprawdzamy zgodnosc
         if (gra_wyrazenie_check(handle,&posneg,gra->data,opis)==0) continue;

         switch (nr1)
         {
            case 0: return CON_AND;
            case 1: return CON_XOR;
            case 2: return CON_OR;
            case 3: return CON_THEN;
         }
      }

   return NO_CON;
}

static int gra_zgodnosc(char *opis,char parametr)
{
   /*
   funkcja sprawdza, czy podany parametr znajduje sie w opisie,
   kontrola zachodzi co do wartosci, nie tresci,
   funkcja zwraca:
   0 - jesli parametr nie jest zgodny z opisem,
   1 - jesli jest zgodny,
   */

   char temp[5];

   // jesli opis okresla dowolny parametr to zgodnosc,
   if (opis[0]=='?') return 1;

   // sprawdzamy, czy w opisie wymieniono podany parametr
   if (atoi(opis)==parametr-'A'+1) return 1;
   sprintf(temp,"/%d-",parametr-'A'+1);
   if (strstr(opis,temp)!=NULL) return 1;

   // jesli nie wymieniono, to parametr nie jest zgodny z opisem
   return 0;
}

static void print_grupa(struct DeniseHandle *handle,struct FactSimple *fs,char *txt,int nr_grupy)
{
   int nr1,check=0;
   char temp[100];

   sprintf(txt,"");

   for (nr1=0;nr1<fs->sentence->words_number;nr1++)
   {
      if (fs->sentence->member[nr1]==nr_grupy)
      {
         if (check==0) check=1;
         else
         {
            sprintf(temp," ");
            strcat(txt,temp);
         }
			glo_get_part(fs->sentence->words,nr1+1,temp,' ');
         strcat(txt,temp);
      }
   }
}

static void rysuj_strukture(struct DeniseHandle* handle,struct FactSimple *fs,int margines,int nr_grupy)
{
   /*
   algorytm:
   - szukamy grupy, dla ktorej nr_grupy jest nadrzedna, i wypisujemy ja na "marginesie"
      - rekurencyjnie
   */

   char *nazwy[]={"podmiot","orzeczenie","przydawka","dopelnienie","okolicznik"};
   char spaces[100];
   char txt[100];
   int nr1;

   memset(spaces,' ',100);
   txt[0]='\0';

   if (nr_grupy==-1)
   {
      for (nr1=0;nr1<fs->sentence->groups_number;nr1++)
         if (fs->sentence->function[nr1]==1) break; // podmiot ma funkcje 1

      print_grupa(handle,fs,txt,nr1+1);
      DeniseReport(handle,"%s (%s)",txt,nazwy[0]);
      nr_grupy=nr1+1;
      margines=3;
   }

   for (nr1=0;nr1<fs->sentence->groups_number;nr1++)
   {
      if (fs->sentence->parent[nr1]==nr_grupy)
      {
         print_grupa(handle,fs,txt,nr1+1);
         DeniseReport(handle,"%.*s%s (%s)",margines,spaces,txt,nazwy[fs->sentence->function[nr1]-1]);
         rysuj_strukture(handle,fs,margines+3,nr1+1);
      }
   }
}

// ***************************************************************************************
// EXTERN               BOOL gra_is_explanation(handle)
// ***************************************************************************************
BOOL gra_is_explanation(struct DeniseHandle* handle)
{
	/*
   funkcja sprawdza, czy wprowadzona wypowiedz jest wyjasnieniem
   pozwalajacym zinterpretowac wczesniejsza wypowiedz, ktora posiadala
   kilka mozliwych poprawnych gramatycznie rozbiorow,

   kasujemy rozbior, ktory zostal uznany za bledny kontekstowo
   */

	struct Fact *fact;
   int ile_wyr,nr1,nr11,nr2;
   char temp[200];

   ile_wyr=glo_count_parts(handle,handle->syntax.busy,' ');
	for (nr1=0,nr11=1;nr1<2;nr1++,nr11--)
   {
	   if (ile_wyr==glo_count_parts(handle,handle->cache.exp_wyrazenie[nr1],' '))
      {
      	// musza wystepowac te same wyrazy
			for (nr2=0;nr2<ile_wyr;nr2++)
         {
				glo_get_part(handle->cache.exp_wyrazenie[nr1],nr2+1,temp,' ');
            if (glo_get_part_position(1,handle->syntax.busy,temp,' ')==0) break;
         }
         // jesli sie zgadza!
         if (nr2==ile_wyr)
         {
				// kasujemy dany fakt z listy
           	if (handle->cache.facts==handle->cache.exp_fact[nr11])
            {
    	         handle->cache.facts=handle->cache.facts->next;
					dat_fact_free(handle->cache.exp_fact[nr11]);
               free(handle->cache.exp_fact[nr11]);
               return TRUE;
				}
				for (fact=handle->cache.facts;fact->next!=NULL;fact=fact->next)
            {
	           	if (fact->next==handle->cache.exp_fact[nr11])
   	         {
    		         fact->next=handle->cache.exp_fact[nr11]->next;
						dat_fact_free(handle->cache.exp_fact[nr11]);
	               free(handle->cache.exp_fact[nr11]);
	               return TRUE;
               }
            }
         }
      }
   }

	return FALSE;
}

// ***************************************************************************************
// EXTERN                void gra_help_explain(handle)
// ***************************************************************************************
void gra_help_explain(struct DeniseHandle* handle)
{
	/*
   funkcja ma pomoc w identyfikacji wypowiedzi - ma pomoc
   wyeliminowac bledne interpretacje,

   zasada dzialania:
   szukamy roznic w odpowiednich sobie faktach prostych kolejnych rozbiorow,
   generujac je w postaci mozliwosci, sugerujac wybor jednej z nich,

   algorytm:
   - pobieramy kolejne fakty proste z rozbioru 1 (w ktorym jest najwiecej fs),
		- pobieramy kolejne fakt proste z rozbioru 2 <- probujemy znalezc
		  fs z taka sama liczba grup, a jak nie ma, to kolejne
		- kontrolujemy grupy - w liczbie rownej mniejszej liczbie grup
        w tych faktach prostych
   	 - jesli sa identyczne, to powtarzamy operacje
	     (jesli zabranie fs, to wystapil blad!)
		- skoro sa rozne, to wychwytujemy roznice:
      	- pobieramy grupe nadrzedna i podrzedna z rozbioru 1 i odpowiednia
           grupe nadrzedna i podrzedna z rozbioru 2, w ktorych zachodzi
           roznica,
         - wyswietlamy mozliwosci rozmowcy polaczone spojnikiem ALBO,
   */

	struct Fact *fact1,*fact2;
   struct FactSimple *fs1,*fs2;
   int ilefs1,ilefs2,poziom,gr1,gr2,find_same;
   char wyrazenie1[100],wyrazenie2[100],par1[100],par2[100];

	// rozbior 1 w fact 1 (musi miec najwiecej fs)
	ilefs1=0;
	for (fact2=handle->cache.facts;fact2!=NULL;fact2=fact2->next)
   {
   	ilefs2=fac_count_fs(fact2);
      if (ilefs2>ilefs1)
      {
      	ilefs1=ilefs2;
         fact1=fact2;
      }
   }
	// rozbior nr 2
   for (fact2=handle->cache.facts;fact2!=NULL;fact2=fact2->next)
   	if (fact2!=fact1) break;

   // pobieramy kolejne fakty proste z rozbioru 1,
	for (fs1=fact1->fs;fs1!=NULL;fs1=fs1->next)
	{
		find_same=1; // zeby probowac znalezc ten sam fakt prosty, a jak nie uda sie, to trudno
		FIND:;
		// pobieramy kolejne fakty proste z rozbioru 2,
		for (fs2=fact2->fs;fs2!=NULL;fs2=fs2->next)
		{
			if (find_same>0)
			{
				if (fs1->sentence->groups_number!=fs2->sentence->groups_number) continue;
				else find_same=2;
			}

			// kontrolujemy grupy - w liczbie rownej mniejszej liczbie grup
        	// w tych faktach prostych
         for (poziom=0;poziom<min(fs1->sentence->groups_number,fs2->sentence->groups_number);poziom++)
         {
         	// szukamy grupy z fs1 z tego poziomu
				for (gr1=0;gr1<fs1->sentence->groups_number;gr1++)
	         	if (get_poziom(fs1->sentence,gr1)==poziom) break;

         	// szukamy grupy z fs2 z tego poziomu
				for (gr2=0;gr2<fs2->sentence->groups_number;gr2++)
	         	if (get_poziom(fs2->sentence,gr2)==poziom) break;

				// pobieramy grupe z fs1
            get_words_of(fs1->sentence,gr1+1,wyrazenie1);

				// pobieramy grupe z fs2
            get_words_of(fs2->sentence,gr2+1,wyrazenie2);

            if (strcmp(wyrazenie1,wyrazenie2)!=0)
            {
					// generujemy zdanie
					// pobieramy grupe nadrzedna z fs1
	            get_words_of(fs1->sentence,fs1->sentence->parent[gr1],par1);
					// pobieramy grupe nadrzedna z fs2
	            get_words_of(fs2->sentence,fs2->sentence->parent[gr2],par2);
					DeniseMessage(handle,UNN_GetLocaleString(handle,MSG_HELP_EXPLAIN,"mam rozumiec, ze %s %s czy %s %s?"),par1,wyrazenie1,par2,wyrazenie2);

               sprintf(handle->cache.exp_wyrazenie[0],"%s %s",par1,wyrazenie1);
               sprintf(handle->cache.exp_wyrazenie[1],"%s %s",par2,wyrazenie2);

					handle->cache.exp_fact[0]=fact1;
					handle->cache.exp_fact[1]=fact2;
					return;
				}
         }
      }
		if (find_same==1) {find_same=0;goto FIND;}
	}
}

static int get_poziom(struct FSentence *sent,int grupa)
{
	int poziom;

	for (poziom=0;;poziom++,grupa=sent->parent[grupa]-1)
   	if (sent->parent[grupa]==0) return poziom;
}

static void get_words_of(struct FSentence *sent,int grupa,char *res)
{
	char space[2],temp[200];
	int nr1;

   res[0]='\0';
   sprintf(space," ");

	for (nr1=0;nr1<sent->words_number;nr1++)
   {
   	if (sent->member[nr1]!=grupa) continue;
      if (res[0]!='\0') strcat(res,space);
		glo_get_part(sent->words,nr1+1,temp,' ');
      strcat(res,temp);
   }
}

