// ***************************************************************************************
//                                          IDENTIFY
// ***************************************************************************************

#include "denise.h"
#include "denise_protos.h"

// deklaracje prywatne
static int cmp_identifiers(struct DeniseHandle* handle,char *ident1,char *ident2,int mode);
static int get_fleksyjny_group(struct DeniseHandle *handle,struct FSentence *fsent,int fleks_nr);

// ***************************************************************************************
// EXTERN                  void ide_get_fact_id(handle,fact,result,mode)
// ***************************************************************************************
void ide_get_fact_id(struct DeniseHandle* handle,struct Fact *fact,char *result,int mode)
{
   /*
   funkcja tworzy pelny opis identyfikacyjny dla calego faktu, obejmujac
   identyfikatory wszystkich faktow prostych, oraz uwzgledniajac ich powtarzalnosc,
   tak, zeby sie nie powtarzaly w pelnym opisie; wykorzystywana jest do tego
   funkcja cmp_identifiers()

   mode:
   ID_FROM_SUBJECT - identyfikuj podany fakt tylko na podstawie podmiotu,
   ID_FROM_OTHER - identyfikacja faktu w oparciu o grupy inne niz podmiot,
   ID_FROM_ALL - identyfikacja faktu w oparciu o wszystkie grupy,

   algorytm:
   - tworzymy tablice identyfikatorow poszczegolnych faktow prostych,
     przy czym identyfikatory wielorakie rozbijamy na jednorodne,
   - sprawdzamy, ktore sie pokrywaja i eliminujemy nadmiarowe elementy tablicy,
   - syntezujemy pozostale identyfikatory,
   */

   struct FactSimple *fs;
   char ident[10][1000],temp1[1000];
   int id_number,nr1,nr2;

   result[0]='\0';

   // tworzymy tablice identyfikatorow poszczegolnych faktow prostych,
   for (id_number=0,fs=fact->fs;fs!=NULL;fs=fs->next)
   {
      // identyfikatory wielorakie rozbijamy na jednorodne,
      for (nr1=0,nr2=glo_count_parts(handle,fs->sentence->id_subject,'|');nr1<nr2;nr1++)
      {
         ident[id_number][0]='\0';

         if (mode!=ID_FROM_OTHER)
         {
            glo_get_part(fs->sentence->id_subject,nr1+1,temp1,'|');
            strcat(ident[id_number],temp1);
         }

         if (mode==ID_FROM_ALL && fs->sentence->id_global!=NULL && strlen(fs->sentence->id_global)>0)
         {
            sprintf(temp1,",%s",fs->sentence->id_global);
            strcat(ident[id_number],temp1);
         }
         else if (mode==ID_FROM_OTHER && fs->sentence->id_global!=NULL && strlen(fs->sentence->id_global)>0)
         {
            strcat(ident[id_number],fs->sentence->id_global);
         }

         id_number++;
         if (mode==ID_FROM_OTHER) break;
      }
   }

   // sprawdzamy, ktore sie pokrywaja i eliminujemy nadmiarowe elementy tablicy,
   for (nr1=0;nr1<id_number;nr1++)
   {
      if (ident[nr1][0]=='\0') continue;

      for (nr2=nr1+1;nr2<id_number;nr2++)
      {
         if (ident[nr2][0]=='\0') continue;

         // sprawdzamy jaka relacja zachodzi miedzy ident[nr1] a ident[nr2]
         switch (cmp_identifiers(handle,ident[nr1],ident[nr2],MODE_COMPATIBLE))
         {
            // jesli rowne, to eliminujemy jeden
            case FCMP_A_EQU_B: ident[nr2][0]='\0'; break;
            // jesli rozne, to musimy pamietac oba
            case FCMP_A_DIF_B: break;
            // jesli ident[nr1] jest uproszczeniem ident[nr2] to kasujemy nr1
            case FCMP_A_IN_B:
            case FCMP_B_PR_A:
               ident[nr1][0]='\0'; goto NEXT_NR1; break;
            // jesli ident[nr2] jest uproszczeniem ident[nr1] to kasujemy nr2
            case FCMP_B_IN_A:
               case FCMP_A_PR_B:
               ident[nr2][0]='\0'; break;
         }
      }
      NEXT_NR1:;
   }

   // syntezujemy pozostale identyfikatory,
   for (nr1=0;nr1<id_number;nr1++)
   {
      if (ident[nr1][0]=='\0') continue;
      if (result[0]=='\0') sprintf(temp1,"%s",ident[nr1]);
      else sprintf(temp1,"|%s",ident[nr1]);
      strcat(result,temp1);
   }
}

static int cmp_identifiers(struct DeniseHandle* handle,char *ident1,char *ident2,int mode)
{
   /*
   ident1 - identyfikatory tematu A,
   ident2 - identyfikatory tematu B,

   mode:
   MODE_EXACT - bez uwzgledniania kwantyfikatorow,
   MODE_COMPATIBLE - z uwzglednianiem kwantyfikatorow,

   funkcja porownuje identyfikatory obu tematow, przy czym musza to byc
   identyfikatory tematow jednorodnych, a nie wielorakich, oraz musza byc
   w formie zoptymalizowanej

   funkcja zwraca wartosci:
   FCMP_A_DIF_B 2
   FCMP_A_EQU_B 3
   FCMP_A_IN_B 4
   FCMP_B_IN_A 5
   FCMP_A_PR_B 6
   FCMP_B_PR_A 7
   FCMP_A_EQU_B_NEG 8
   FCMP_A_IN_B_NEG 9
   FCMP_B_IN_A_NEG 10
   FCMP_A_PR_B_NEG 11
   FCMP_B_PR_A_NEG 12

   algorytm:
   - wstepnie ustalamy wynik na FCMP_A_DIF_B,
   - ustalamy identA na liste dluzsza, a identB na krotsza, nie liczac zanegowanych identyfikatorow,
   - obliczamy i zapamietujemy max. liczbe wyrazow, ktorych moze brakowac: roznica w ich liczbie po prostu,
   - pobieramy kolejne identyfikatory z identA,
      - jesli to identyfikator zanegowany (np. !programista):
         - pobieramy kolejne identyfikatory z identB,
            - jesli identyfikator jest zanegowany to pobieramy kolejny identyfikator z identB,
            - w przeciwnym razie sprawdzamy oba identyfikatory:
               - jesli sa rowne (wyrazy) to RETURN FCMP_A_DIF_B,
               - jesli sa rozne (wyrazy) to sprawdzamy kwantyfikatory, jesli opcja wlaczona:
                  - jesli wynikiem jest FCMP_A_DIF_B to pobieramy nastepny z identB,
                  - jesli wynikiem jest FCMP_A_EQU_B to RETURN FCMP_A_DIF_B,
                  - jesli wynikiem jest FCMP_A_IN_B (!programista/ktos+wszyscy) to RETURN FCMP_A_DIF_B,
                  - jesli wynikiem jest FCMP_B_IN_A (!wszyscy+programista/ktos) to pobieramy nastepny z identB,
                  - jesli wynikiem jest FCMP_A_PR_B (!programista+ktos) to pobieramy nastepny z identB,
                  - jesli wynikiem jest FCMP_B_PR_A (!ktos+programista) to RETURN FCMP_A_DIF_B,
      - jesli identyfikator nie jest zanegowany
         - ustalamy zmiennacmp=FCMP_NULL,
         - pobieramy kolejne identyfikatory z identB,
            - jesli identyfikator z identB jest zanegowany, to sprawdzamy oba identyfikatory:
               - jesli sa rowne (wyrazy) to RETURN FCMP_A_DIF_B,
               - jesli sa rozne (wyrazy) to sprawdzamy kwantyfikatory, jesli opcja wlaczona:
                  - jesli wynikiem jest FCMP_A_DIF_B to pobieramy nastepny z identB,
                  - jesli wynikiem jest FCMP_A_EQU_B to RETURN FCMP_A_DIF_B,
                  - jesli wynikiem jest FCMP_A_IN_B (programista/ktos+!wszyscy) to pobieramy nastepny z identB,
                  - jesli wynikiem jest FCMP_B_IN_A (wszyscy+!programista/ktos) to RETURN FCMP_A_DIF_B,
                  - jesli wynikiem jest FCMP_A_PR_B (programista+!ktos) to RETURN FCMP_A_DIF_B,
                  - jesli wynikiem jest FCMP_B_PR_A (ktos+!programista) to pobieramy nastepny z identB,
            - jesli identyfikator z identB nie jest zanegowany, to sprawdzamy:
               - jesli sa rowne (wyrazy) to cmp=FCMP_A_EQU_B i wychodzimy z pobierania kolejnych z identB,
               - jesli sa rozne (wyrazy) to kwantyfikatory, jesli opcja wlaczona:
                  - jesli wynikiem jest FCMP_A_DIF_B to pobieramy nastepny z identB,
                  - jesli wynikiem jest FCMP_A_EQU_B to cmp=FCMP_A_EQU_B i wychodzimy z pobierania kolejnych z identB,
                  - jesli wynikiem jest FCMP_A_IN_B (programista/ktos+wszyscy) to cmp=FCMP_A_IN_B i wychodzimy z pobierania kolejnych z identB,
                  - jesli wynikiem jest FCMP_B_IN_A (wszyscy+programista/ktos) to cmp=FCMP_B_IN_A i wychodzimy z pobierania kolejnych z identB,
                  - jesli wynikiem jest FCMP_A_PR_B (programista+ktos) to cmp=FCMP_A_PR_B i wychodzimy z pobierania kolejnych z identB,
                  - jesli wynikiem jest FCMP_B_PR_A (ktos+programista) to cmp=FCMP_B_PR_A i wychodzimy z pobierania kolejnych z identB,
         - badamy zmienna cmp:
            - cmp==FCMP_NULL to:
               - zmniejszamy max_dif o jeden
                  - jesli max_dif mniejsze od 0, to wychodzimy return FCMP_A_DIF_B;
                  - ustawiamy not_equ - nie moga byc juz rowne
               - jesli result==FCMP_A_DIF_B to result=FCMP_A_DIF_B,
               - jesli result==FCMP_A_EQU_B to result=FCMP_B_IN_A,
               - jesli result==FCMP_A_IN_B to RETURN FCMP_A_DIF_B,
               - jesli result==FCMP_B_IN_A to result=FCMP_B_IN_A,
               - jesli result==FCMP_A_PR_B to RETURN FCMP_A_DIF_B,
               - jesli result==FCMP_B_PR_A to RETURN FCMP_A_DIF_B,
            - cmp==FCMP_A_EQU_B to result bez zmian, a jesli byl A_DIF_B to uwzgledniamy ile_dif
            - cmp==FCMP_A_IN_B to:
               - jesli result==FCMP_A_DIF_B to result=FCMP_A_IN_B,
               - jesli result==FCMP_A_EQU_B to result=FCMP_A_IN_B,
               - jesli result==FCMP_A_IN_B to result=FCMP_A_IN_B,
               - jesli result==FCMP_B_IN_A to RETURN FCMP_A_DIF_B,
               - jesli result==FCMP_A_PR_B to result=FCMP_A_IN_B,
               - jesli result==FCMP_B_PR_A to result=FCMP_A_IN_B,
            - cmp==FCMP_B_IN_A to:
               - jesli result==FCMP_A_DIF_B to result=FCMP_B_IN_A,
               - jesli result==FCMP_A_EQU_B to result=FCMP_B_IN_A,
               - jesli result==FCMP_A_IN_B to RETURN FCMP_A_DIF_B,
               - jesli result==FCMP_B_IN_A to result=FCMP_B_IN_A,
               - jesli result==FCMP_A_PR_B to result=FCMP_B_IN_A,
               - jesli result==FCMP_B_PR_A to result=FCMP_B_IN_A,
            - cmp==FCMP_A_PR_B to:
               - jesli result==FCMP_A_DIF_B to result=FCMP_A_PR_B,
               - jesli result==FCMP_A_EQU_B to result=FCMP_A_PR_B,
               - jesli result==FCMP_A_IN_B to result=FCMP_A_IN_B,
               - jesli result==FCMP_B_IN_A to result=FCMP_B_IN_A,
               - jesli result==FCMP_A_PR_B to result=FCMP_A_PR_B,
               - jesli result==FCMP_B_PR_A to RETURN FCMP_A_DIF_B,
            - cmp==FCMP_B_PR_A to:
               - jesli result==FCMP_A_DIF_B to result=FCMP_B_PR_A,
               - jesli result==FCMP_A_EQU_B to result=FCMP_B_PR_A,
               - jesli result==FCMP_A_IN_B to result=FCMP_A_IN_B,
               - jesli result==FCMP_B_IN_A to result=FCMP_B_IN_A,
               - jesli result==FCMP_A_PR_B to RETURN FCMP_A_DIF_B,
               - jesli result==FCMP_B_PR_A to result=FCMP_B_PR_A,
   */

   char temp[100];
   char wyraz1[100],wyraz2[100];
   char fleks1[100],fleks2[100];
   char *identA,*identB;
   int result,max_dif,cmp,nr1,idA_nr,idB_nr,idA_ile,idB_ile,posnegA,posnegB;
   BOOL not_equ=FALSE;

   if (ident1==NULL || ident2==NULL)
   {
      printf("a to niezapowiedzianka nr 1 w identify!\n");
      return FCMP_A_EQU_B;
   }
   if (strlen(ident1)==0 || strlen(ident2)==0)
   {
      printf("a to niezapowiedzianka nr 2 w identify!\n");
      return FCMP_A_EQU_B;
   }
   if (strchr(ident1,'|')!=NULL || strchr(ident2,'|')!=NULL)
   {
      printf("ide_cmp_identifiers wymaga tematow jednorodnych a nie wielorakich!\n");
      return FCMP_A_DIF_B;
   }

   idA_ile=glo_count_parts(handle,ident1,',');
   cmp=idA_ile;
   for (nr1=0;nr1<idA_ile;nr1++)
   {
      glo_get_part(ident1,nr1+1,temp,',');
      if (temp[0]=='!') cmp--;
   }
   idA_ile=cmp;

   idB_ile=glo_count_parts(handle,ident2,',');
   cmp=idB_ile;
   for (nr1=0;nr1<idB_ile;nr1++)
   {
      glo_get_part(ident2,nr1+1,temp,',');
      if (temp[0]=='!') cmp--;
   }
   idB_ile=cmp;

   // zapamietujemy max. liczbe wyrazow, ktorych moze brakowac: roznica w ich liczbie po prostu,
   max_dif=abs(idA_ile-idB_ile);

   // przypisujemy identA i identB
   if (idA_ile>idB_ile)
   {
      identA=ident1;
      identB=ident2;
      idA_ile=glo_count_parts(handle,ident1,',');
      idB_ile=glo_count_parts(handle,ident2,',');
   }
   else
   {
      identA=ident2;
      identB=ident1;
      idA_ile=glo_count_parts(handle,ident2,',');
      idB_ile=glo_count_parts(handle,ident1,',');
   }

   // wstepnie ustalamy wynik na FCMP_A_DIF_B,
   result=FCMP_A_DIF_B;

   // pobieramy kolejne identyfikatory z identA,
   for (idA_nr=0;idA_nr<idA_ile;idA_nr++)
   {
      glo_get_part(identA,idA_nr+1,temp,',');
      if (temp[0]=='!') {posnegA=1;memmove(temp,temp+1,strlen(temp));}
      else posnegA=0;
      glo_get_part(temp,1,wyraz1,'+');
      glo_get_part(temp,2,fleks1,'+');

      // jesli to identyfikator zanegowany (np. !programista):
      if (posnegA==1)
      {
         // pobieramy kolejne identyfikatory z identB,
         for (idB_nr=0;idB_nr<idB_ile;idB_nr++)
         {
            glo_get_part(identB,idB_nr+1,temp,',');
            if (temp[0]=='!') {posnegB=1;memmove(temp,temp+1,strlen(temp));}
            else posnegB=0;
            glo_get_part(temp,1,wyraz2,'+');
            glo_get_part(temp,2,fleks2,'+');

            // jesli identyfikator jest zanegowany to pobieramy kolejny identyfikator z identB,
            if (posnegB==1) continue;

            // w przeciwnym razie sprawdzamy oba identyfikatory:
            // jesli sa rowne (wyrazy) to RETURN FCMP_A_DIF_B,
            if (strcmp(wyraz1,wyraz2)==0) return FCMP_A_DIF_B;

            // jesli sa rozne (wyrazy) to sprawdzamy kwantyfikatory, jesli opcja wlaczona:
            if (mode==MODE_EXACT) continue;

            switch (log_cmp_kwantyfikatory(handle,fleks1,fleks2))
            {
               // jesli wynikiem jest FCMP_A_DIF_B to pobieramy nastepny z identB,
               case FCMP_A_DIF_B: break;
               // jesli wynikiem jest FCMP_A_EQU_B to RETURN FCMP_A_DIF_B,
               case FCMP_A_EQU_B: return FCMP_A_DIF_B;
               // jesli wynikiem jest FCMP_A_IN_B (!programista/ktos+wszyscy) to RETURN FCMP_A_DIF_B,
               case FCMP_A_IN_B: return FCMP_A_DIF_B;
               // jesli wynikiem jest FCMP_B_IN_A (!wszyscy+programista/ktos) to pobieramy nastepny z identB,
               case FCMP_B_IN_A: break;
               // jesli wynikiem jest FCMP_A_PR_B (!programista+ktos) to pobieramy nastepny z identB,
               case FCMP_A_PR_B: break;
               // jesli wynikiem jest FCMP_B_PR_A (!ktos+programista) to RETURN FCMP_A_DIF_B,
               case FCMP_B_PR_A: return FCMP_A_DIF_B;
            }
         }
      }
      // jesli identyfikator nie jest zanegowany
      else if (posnegA==0)
      {
         // ustalamy zmiennacmp=FCMP_NULL,
         cmp=FCMP_NULL;
         // pobieramy kolejne identyfikatory z identB,
         for (idB_nr=0;idB_nr<idB_ile;idB_nr++)
         {
            glo_get_part(identB,idB_nr+1,temp,',');
            if (temp[0]=='!') {posnegB=1;memmove(temp,temp+1,strlen(temp));}
            else posnegB=0;
            glo_get_part(temp,1,wyraz2,'+');
            glo_get_part(temp,2,fleks2,'+');

            // jesli identyfikator z identB jest zanegowany, to sprawdzamy oba identyfikatory:
            if (posnegB==1)
            {
               // jesli sa rowne (wyrazy) to RETURN FCMP_A_DIF_B,
               if (strcmp(wyraz1,wyraz2)==0) return FCMP_A_DIF_B;

               // jesli sa rozne (wyrazy) to sprawdzamy kwantyfikatory, jesli opcja wlaczona:
               if (mode==MODE_EXACT) continue;

               switch (log_cmp_kwantyfikatory(handle,fleks1,fleks2))
               {
                  // jesli wynikiem jest FCMP_A_DIF_B to pobieramy nastepny z identB,
                  case FCMP_A_DIF_B: break;
                  // jesli wynikiem jest FCMP_A_EQU_B to RETURN FCMP_A_DIF_B,
                  case FCMP_A_EQU_B: return FCMP_A_DIF_B;
                  // jesli wynikiem jest FCMP_A_IN_B (programista/ktos+!wszyscy) to pobieramy nastepny z identB,
                  case FCMP_A_IN_B: break;
                  // jesli wynikiem jest FCMP_B_IN_A (wszyscy+!programista/ktos) to RETURN FCMP_A_DIF_B,
                  case FCMP_B_IN_A: return FCMP_A_DIF_B;
                  // jesli wynikiem jest FCMP_A_PR_B (programista+!ktos) to RETURN FCMP_A_DIF_B,
                  case FCMP_A_PR_B: return FCMP_A_DIF_B;
                  // jesli wynikiem jest FCMP_B_PR_A (ktos+!programista) to pobieramy nastepny z identB,
                  case FCMP_B_PR_A: break;
               }
            }
            // jesli identyfikator z identB nie jest zanegowany, to sprawdzamy:
            else if (posnegB==0)
            {
               // jesli sa rowne (wyrazy) to cmp=FCMP_A_EQU_B i wychodzimy z pobierania kolejnych z identB,
               if (strcmp(wyraz1,wyraz2)==0) {cmp=FCMP_A_EQU_B;goto CMP_END;}

               // jesli sa rozne (wyrazy) to sprawdzamy kwantyfikatory, jesli opcja wlaczona:
               if (mode==MODE_EXACT) continue;

               switch (log_cmp_kwantyfikatory(handle,fleks1,fleks2))
               {
                  // jesli wynikiem jest FCMP_A_DIF_B to pobieramy nastepny z identB,
                  case FCMP_A_DIF_B: break;
                  // jesli wynikiem jest FCMP_A_EQU_B to cmp=FCMP_A_EQU_B i wychodzimy z pobierania kolejnych z identB,
                  case FCMP_A_EQU_B: cmp=FCMP_A_EQU_B; goto CMP_END;
                  // jesli wynikiem jest FCMP_A_IN_B (programista/ktos+wszyscy) to cmp=FCMP_A_IN_B i wychodzimy z pobierania kolejnych z identB,
                  case FCMP_A_IN_B: cmp=FCMP_A_IN_B; goto CMP_END;
                  // jesli wynikiem jest FCMP_B_IN_A (wszyscy+programista/ktos) to cmp=FCMP_B_IN_A i wychodzimy z pobierania kolejnych z identB,
                  case FCMP_B_IN_A: cmp=FCMP_B_IN_A; goto CMP_END;
                  // jesli wynikiem jest FCMP_A_PR_B (programista+ktos) to cmp=FCMP_A_PR_B i wychodzimy z pobierania kolejnych z identB,
                  case FCMP_A_PR_B: cmp=FCMP_A_PR_B; goto CMP_END;
                  // jesli wynikiem jest FCMP_B_PR_A (ktos+programista) to cmp=FCMP_B_PR_A i wychodzimy z pobierania kolejnych z identB,
                  case FCMP_B_PR_A: cmp=FCMP_B_PR_A; goto CMP_END;
               }
            }
         }
         CMP_END:
         // badamy zmienna cmp:
         switch (cmp)
         {
            // cmp==FCMP_NULL - nie odnaleziono identyfikatora:
            case FCMP_NULL:
               max_dif--;
               if (max_dif<0) return FCMP_A_DIF_B;
               not_equ=TRUE;
               switch (result)
               {
                  // jesli result==FCMP_A_DIF_B to result=FCMP_A_DIF_B,
                  case FCMP_A_DIF_B: result=FCMP_A_DIF_B; break;
                  // jesli result==FCMP_A_EQU_B to result=FCMP_B_IN_A,
                  case FCMP_A_EQU_B: result=FCMP_B_IN_A; break;
                  // jesli result==FCMP_A_IN_B to RETURN FCMP_A_DIF_B,
                  case FCMP_A_IN_B: return FCMP_A_DIF_B;
                  // jesli result==FCMP_B_IN_A to result=FCMP_B_IN_A,
                  case FCMP_B_IN_A: result=FCMP_B_IN_A; break;
                  // jesli result==FCMP_A_PR_B to RETURN FCMP_A_DIF_B,
                  case FCMP_A_PR_B: return FCMP_A_DIF_B;
                  // jesli result==FCMP_B_PR_A to RETURN FCMP_A_DIF_B,
                  case FCMP_B_PR_A: return FCMP_A_DIF_B;
               }
               break;
            // cmp==FCMP_A_EQU_B to result bez zmian (a jesli A_DIF_B to uwzgledniamy ile_dif),
            case FCMP_A_EQU_B:
               if (result==FCMP_A_DIF_B)
               {
                  if (not_equ==TRUE) result=FCMP_B_IN_A;
                  else result=FCMP_A_EQU_B;
               }
               break;
            // cmp==FCMP_A_IN_B to:
            case FCMP_A_IN_B:
               switch (result)
               {
                  // jesli result==FCMP_A_DIF_B to result=FCMP_A_IN_B,
                  case FCMP_A_DIF_B: result=FCMP_A_IN_B; break;
                  // jesli result==FCMP_A_EQU_B to result=FCMP_A_IN_B,
                  case FCMP_A_EQU_B: result=FCMP_A_IN_B; break;
                  // jesli result==FCMP_A_IN_B to result=FCMP_A_IN_B,
                  case FCMP_A_IN_B: result=FCMP_A_IN_B; break;
                  // jesli result==FCMP_B_IN_A to RETURN FCMP_A_DIF_B,
                  case FCMP_B_IN_A: return FCMP_A_DIF_B;
                  // jesli result==FCMP_A_PR_B to result=FCMP_A_IN_B,
                  case FCMP_A_PR_B: result=FCMP_A_IN_B; break;
                  // jesli result==FCMP_B_PR_A to result=FCMP_A_IN_B,
                  case FCMP_B_PR_A: result=FCMP_A_IN_B; break;
               }
               break;
            // cmp==FCMP_B_IN_A to:
            case FCMP_B_IN_A:
               switch (result)
               {
                  // jesli result==FCMP_A_DIF_B to result=FCMP_B_IN_A,
                  case FCMP_A_DIF_B: result=FCMP_B_IN_A; break;
                  // jesli result==FCMP_A_EQU_B to result=FCMP_B_IN_A,
                  case FCMP_A_EQU_B: result=FCMP_B_IN_A; break;
                  // jesli result==FCMP_A_IN_B to RETURN FCMP_A_DIF_B,
                  case FCMP_A_IN_B: return FCMP_A_DIF_B; break;
                  // jesli result==FCMP_B_IN_A to result=FCMP_B_IN_A,
                  case FCMP_B_IN_A: result=FCMP_B_IN_A; break;
                  // jesli result==FCMP_A_PR_B to result=FCMP_B_IN_A,
                  case FCMP_A_PR_B: result=FCMP_B_IN_A; break;
                  // jesli result==FCMP_B_PR_A to result=FCMP_B_IN_A,
                  case FCMP_B_PR_A: result=FCMP_B_IN_A; break;
               }
               break;
            // cmp==FCMP_A_PR_B to:
            case FCMP_A_PR_B:
               switch (result)
               {
                  // jesli result==FCMP_A_DIF_B to result=FCMP_A_PR_B,
                  case FCMP_A_DIF_B: result=FCMP_A_PR_B; break;
                  // jesli result==FCMP_A_EQU_B to result=FCMP_A_PR_B,
                  case FCMP_A_EQU_B: result=FCMP_A_PR_B; break;
                  // jesli result==FCMP_A_IN_B to result=FCMP_A_IN_B,
                  case FCMP_A_IN_B: result=FCMP_A_IN_B; break;
                  // jesli result==FCMP_B_IN_A to result=FCMP_B_IN_A,
                  case FCMP_B_IN_A: result=FCMP_B_IN_A; break;
                  // jesli result==FCMP_A_PR_B to result=FCMP_A_PR_B,
                  case FCMP_A_PR_B: result=FCMP_A_PR_B; break;
                  // jesli result==FCMP_B_PR_A to RETURN FCMP_A_DIF_B,
                  case FCMP_B_PR_A: return FCMP_A_DIF_B; break;
               }
               break;
            // cmp==FCMP_B_PR_A to:
            case FCMP_B_PR_A:
               switch (result)
               {
                  // jesli result==FCMP_A_DIF_B to result=FCMP_B_PR_A,
                  case FCMP_A_DIF_B: result=FCMP_B_PR_A; break;
                  // jesli result==FCMP_A_EQU_B to result=FCMP_B_PR_A,
                  case FCMP_A_EQU_B: result=FCMP_B_PR_A; break;
                  // jesli result==FCMP_A_IN_B to result=FCMP_A_IN_B,
                  case FCMP_A_IN_B: result=FCMP_B_IN_A; break;
                  // jesli result==FCMP_B_IN_A to result=FCMP_B_IN_A,
                  case FCMP_B_IN_A: result=FCMP_B_IN_A; break;
                  // jesli result==FCMP_A_PR_B to RETURN FCMP_A_DIF_B,
                  case FCMP_A_PR_B: return FCMP_A_DIF_B; break;
                  // jesli result==FCMP_B_PR_A to result=FCMP_B_PR_A,
                  case FCMP_B_PR_A: result=FCMP_B_PR_A; break;
               }
               break;
         }
      }
   }

   if (ident1==identB)
   {
      if (result==FCMP_A_IN_B) result=FCMP_B_IN_A;
      else if (result==FCMP_B_IN_A) result=FCMP_A_IN_B;
      else if (result==FCMP_A_PR_B) result=FCMP_B_PR_A;
      else if (result==FCMP_B_PR_A) result=FCMP_A_PR_B;
   }

   return result;
}

// ***************************************************************************************
// EXTERN               void ide_get_fsent_id(handle,fsent,result,mode)
// ***************************************************************************************
void ide_get_fsent_id(struct DeniseHandle* handle,struct FSentence *fsent,char *result,int mode)
{
   /*
   mode:
   ID_FROM_SUBJECT - identyfikuj podane zdanie tylko na podstawie podmiotu,
   ID_FROM_OTHER - identyfikacja zdania w oparciu o grupy inne niz podmiot,
   ID_FROM_ALL - identyfikacja zdania w oparciu o wszystkie grupy,

   funkcja zwraca ciag wyrazow bedacych identyfikatorami w podanym fakcie,
   uwzglednia ona wiele tematow, identyfikatory lokalne oraz globalne,
   np.:

   ludzie: marek kowalski i darek sa mezczyznami

   marek,kowalski|darek <- odrebne tematy (przecinek oddziela wyrazy fleksyjne)
   czlowiek,mezczyzna <- dotycza kazdego z tematow (identyfikatory globalne)

   "marek kowalski" <- to temat glowny i identyfikator lokalny, przy czym nie mozna
   wyodrebnic tematu i identyfikatora, stanowia one calosc konstrukcyjna,

   UWAGA! funkcja ignoruje pola: id_subject id_globals, kierujac sie wylacznie
   pelna trescia zdania - dlatego nie nadaje sie do pobieranie identyfikatorow
   z pojedynczego faktu prostego w przypadku wypowiedzi wielotematycznych
   (np. "marek, szymon jest czlowiekiem")

   algorytm:
   - zerujemy wyrazenie tymczasowe,
   - pobieramy kolejne wyrazy fleksyjne,
      - sprawdzamy, czy charakterystyka wyrazu jest zgodna z jakas pozycja w
        bazie logicznej opisujacej wyrazy identyfikujace:
        - jesli jest zgodna, to:
            - dopisujemy do wyrazenia tymczasowego,
            - jesli to koniec grupy lub za wyrazem stoi znak ',' lub spojnik typu AND:
               - jesli to grupa podmiotu, to dopisujemy wyrazenie tymczasowe jako nowy temat,
                 i zerujemy wyrazenie tymczasowe,
               - jesli to nie grupa podmiotu, to dopisujemy wyrazenie tymczasowe jako nowy
                 identyfikator globalny i zerujemy wyrazenie tymczasowe,
            - jesli za wyrazem fleksyjnym w zdaniu stoi znak ':':
               - bez wzgledu jaka to grupa, dopisujemy wyrazenie tymczasowe jako nowy
                 identyfikator globalny i zerujemy wyrazenie tymczasowe,
   - syntezujemy tablice tematow i identyfikatorow globalnych:
     pobieramy kolejne tematy i dopisujemy wszystkie identyfikatory globalne do kazdego tematu,
   */

   struct GrammarData *log;
   struct GrammarLo5Data *lo5_data;
   char posneg=TWIERDZENIE;
   char temp1[1000],temp2[1000],temp3[1000],fleks[100],posneg_tabl[2],and_fleks[3],*ptr;
   char temporary[1000];
   char themes[10][1000];
   char globals[10][1000];
   int nr1,nr2,nr3,ile,nr_grupy,themes_number,globals_number;

   result[0]='\0';
   themes_number=0;
   globals_number=0;

   sprintf(and_fleks,"+");

   // zerujemy wyrazenie tymczasowe,
   temporary[0]='\0';

   // sprawdzamy kolejne grupy
   for (nr_grupy=0;nr_grupy<fsent->groups_number;nr_grupy++)
   {
      // jesli to tryb ID_FROM_SUBJECT, to w gre wchodzi tylko grupa identyfikacyjna (podmiot)
      if (mode==ID_FROM_SUBJECT && fsent->function[nr_grupy]!=1) continue;
      if (fsent->posneg[nr_grupy]==PRZECZENIE) sprintf(posneg_tabl,"!");
      else posneg_tabl[0]='\0';

      // pobieramy kolejne wyrazy fleksyjne,
      for (nr1=0;nr1<fsent->ile_fleks;nr1++)
      {
         if (get_fleksyjny_group(handle,fsent,nr1+1)!=nr_grupy+1) continue;
         glo_get_part(fsent->voc,nr1+1,temp1,',');

         // jesli to spojnik typu AND:
         glo_get_part(fsent->voc,fsent->fleksyjny[nr1],fleks,',');

         // sprawdzamy dane w bazie gramatycznej
         for (log=handle->language.lo2[0];log!=NULL;log=log->next)
         {
            // sprawdzamy zgodnosc
            if (gra_wyrazenie_check(handle,&posneg,log->data,fleks)==1)
            {
               // tak, to spojnik, wiec:
               // jesli to grupa podmiotu, to dopisujemy wyrazenie tymczasowe jako nowy temat,
               // i zerujemy wyrazenie tymczasowe,
               if (fsent->function[nr_grupy]==1)
               {
                  strcpy(themes[themes_number],posneg_tabl);
                  strcat(themes[themes_number],temporary);
                  themes_number++;
               }
               // jesli to nie grupa podmiotu, to dopisujemy wyrazenie tymczasowe jako nowy
               // identyfikator globalny i zerujemy wyrazenie tymczasowe,
               else
               {
                  strcpy(globals[globals_number],posneg_tabl);
                  strcat(globals[globals_number],temporary);
                  globals_number++;
               }
               temporary[0]='\0';
               goto NEXT;
            }
         }

         // sprawdzamy, czy charakterystyka wyrazu jest zgodna z jakas pozycja w
         // bazie logicznej opisujacej wyrazy identyfikujace:
         for (lo5_data=handle->language.lo5;lo5_data!=NULL;lo5_data=lo5_data->next)
         {
            if (fsent->function[nr_grupy]!=lo5_data->func) continue;
            if (gra_wyrazenie_check(handle,&posneg,lo5_data->czesc_mowy,temp1)==0) continue;

            if (lo5_data->banned!=NULL)
            {
               for (nr2=0;nr2<fsent->words_number;nr2++)
               {
                  if (fsent->member[nr2]!=nr_grupy+1) continue;
                  glo_get_part(fsent->voc,fsent->fleksyjny[nr2],temp2,',');

                  for (ile=glo_count_parts(handle,lo5_data->banned,';'),nr3=0;nr3<ile;nr3++)
                  {
                     glo_get_part(lo5_data->banned,nr3+1,temp3,';');
                     if (gra_wyrazenie_check(handle,&posneg,temp3,temp2)==1) goto NEXT;
                  }
               }
            }

            if (lo5_data->requied!=NULL)
            {
               for (ile=glo_count_parts(handle,lo5_data->requied,';'),nr2=0;nr2<ile;nr2++)
               {
                  glo_get_part(lo5_data->requied,nr2+1,temp2,';');
                  for (nr3=0;nr3<fsent->words_number;nr3++)
                  {
                     if (fsent->member[nr3]!=nr_grupy+1) continue;
                     glo_get_part(fsent->voc,fsent->fleksyjny[nr3],temp3,',');
                     if (gra_wyrazenie_check(handle,&posneg,temp2,temp3)==0) continue;
                     goto FOUNDED;
                  }
                  goto NEXT;
                  FOUNDED:;
               }
            }

            // jesli jest zgodna, to:
            // dopisujemy do wyrazenia tymczasowego,
            if (temporary[0]!='\0')
            {
               sprintf(temp1,",");
               strcat(temporary,temp1);
            }

            glo_get_part(fsent->base_words,nr1+1,temp1,',');
//            printf("wyraz fleksyjny: '%s'\n",temp1);
            strcat(temporary,temp1);
            strcat(temporary,and_fleks);
            strcat(temporary,fleks);

            // sprawdzamy, czy za wyrazem stoi znak ',' lub ':'
            ptr=strstr(fsent->orig,temp1);
            if (ptr)
				{
				ptr+=strlen(temp1);
				for (;;)
	            {
               // jesli za wyrazem stoi znak ',':
	               if (*ptr==',')
	               {
                  // jesli to grupa podmiotu, to dopisujemy wyrazenie tymczasowe jako nowy temat,
                  // i zerujemy wyrazenie tymczasowe,
	                  if (fsent->function[nr_grupy]==1)
	                  {
	                     strcpy(themes[themes_number],posneg_tabl);
	                     strcat(themes[themes_number],temporary);
	                     themes_number++;
	                  }
                  // jesli to nie grupa podmiotu, to dopisujemy wyrazenie tymczasowe jako nowy
                  // identyfikator globalny i zerujemy wyrazenie tymczasowe,
	                  else
	                  {
	                     strcpy(globals[globals_number],posneg_tabl);
	                     strcat(globals[globals_number],temporary);
	                     globals_number++;
	                  }
	                  temporary[0]='\0';
	                  break;
	               }
	               // jesli za wyrazem fleksyjnym w zdaniu stoi znak ':':
	               else if (*ptr==':')
	               {
                  // bez wzgledu jaka to grupa, dopisujemy wyrazenie tymczasowe jako nowy
                  // identyfikator globalny i zerujemy wyrazenie tymczasowe,
	                  strcpy(globals[globals_number],posneg_tabl);
	                  strcat(globals[globals_number],temporary);
	                  globals_number++;
	                  temporary[0]='\0';
	                  break;
	               }
	               else if (*ptr!=' ') break;
	               ptr++;
	            }
				}
         }
         NEXT:;
      }

      // nie dopisujemy "pustych" wyrazen
      if (temporary[0]=='\0') continue;

      // jesli to koniec grupy:
      // jesli to grupa podmiotu, to dopisujemy wyrazenie tymczasowe jako nowy temat,
      // i zerujemy wyrazenie tymczasowe,
      if (fsent->function[nr_grupy]==1)
      {
         strcpy(themes[themes_number],posneg_tabl);
         strcat(themes[themes_number],temporary);
         themes_number++;
      }
      // jesli to nie grupa podmiotu, to dopisujemy wyrazenie tymczasowe jako nowy
      // identyfikator globalny i zerujemy wyrazenie tymczasowe,
      else
      {
         strcpy(globals[globals_number],posneg_tabl);
         strcat(globals[globals_number],temporary);
         globals_number++;
      }
      temporary[0]='\0';
   }

   // przygotowujemy wynik
   if (mode==ID_FROM_ALL)
   {
      // syntezujemy tablice tematow i identyfikatorow globalnych:
      // pobieramy kolejne tematy i dopisujemy wszystkie identyfikatory globalne do kazdego tematu,
      for (nr1=0;nr1<themes_number;nr1++)
      {
         if (globals_number>0) sprintf(temp1,"%s,",themes[nr1]);
         else
         {
            if (nr1<themes_number-1) sprintf(temp1,"%s|",themes[nr1]);
            else sprintf(temp1,"%s",themes[nr1]);
         }
         strcat(result,temp1);

         for (nr2=0;nr2<globals_number;nr2++)
         {
            if (nr2<globals_number-1) sprintf(temp1,"%s,",globals[nr2]);
            else
            {
               if (nr1<themes_number-1) sprintf(temp1,"%s|",globals[nr2]);
               else sprintf(temp1,"%s",globals[nr2]);
            }
            strcat(result,temp1);
         }
      }
   }
   else if (mode==ID_FROM_SUBJECT)
   {
      // syntezujemy tablice tematow
      for (nr1=0;nr1<themes_number;nr1++)
      {
         if (nr1<themes_number-1) sprintf(temp1,"%s|",themes[nr1]);
         else sprintf(temp1,"%s",themes[nr1]);

         strcat(result,temp1);
      }
   }
   else if (mode==ID_FROM_OTHER)
   {
      // syntezujemy tablice identyfikatorow globalnych
      for (nr1=0;nr1<globals_number;nr1++)
      {
         if (nr1<globals_number-1) sprintf(temp1,"%s,",globals[nr1]);
         else sprintf(temp1,"%s",globals[nr1]);

         strcat(result,temp1);
      }
   }

//printf("!%s!\n",result);
}

static int get_fleksyjny_group(struct DeniseHandle *handle,struct FSentence *fsent,int fleks_nr)
{
   // fleks_nr - numer (od 1) wyrazu fleksyjnego

   int nr1;

   for (nr1=0;nr1<fsent->words_number;nr1++)
      if (fsent->fleksyjny[nr1]==fleks_nr)
         return fsent->member[nr1]; // zwracamy numer grupy (liczony od 1)

   DeniseReport(handle,"blad!");
   return -1; // blad!
}

// ***************************************************************************************
// EXTERN              void ide_join_identifiers(handle,theme,fact)
// ***************************************************************************************
void ide_add_identifiers(struct DeniseHandle* handle,struct Theme *theme,char *ident)
{
   /*
   funkcja dolacza identyfikatory z ident, opcjonalnie poszerzajac zakres
   identyfikatorow w temacie,

   algorytm:
   - pobieramy identyfikatory z tematu,
   - laczymy pobrane identyfikatory z podanymi,
   - dokonujemy optymalizacji nowych identyfikatorow,
   - zapisujemy identyfikatory do tematu,
   */

   char result[2000];

   if (ident==NULL || strlen(ident)==0) return;

   // pobieramy identyfikatory z tematu,
   // laczymy pobrane identyfikatory z podanymi,
   sprintf(result,"%s,%s",theme->identify,ident);

   // dokonujemy optymalizacji nowych identyfikatorow,
   ide_optimize_identifiers(handle,result);

   // zapisujemy identyfikatory do tematu,
   free(theme->identify);
   theme->identify=malloc(strlen(result)+1);
   strcpy(theme->identify,result);

//printf("!%s!\n",result);
}

// ***************************************************************************************
// EXTERN               void ide_optimize_identifiers(handle,ident)
// ***************************************************************************************
void ide_optimize_identifiers(struct DeniseHandle* handle,char *ident)
{
   /*
   funkcja dokonuje optymalizacji identyfikatorow: ident1,ident2,...,ident3
   pozostawiajac najogolniejsze, np.:
   marek,wszyscy,czlowiek -> wszyscy
   marek,drzewo,wszystko -> marek,wszystko

   algorytm:
   - pobieramy kolejne identyfikatory z ident,
      - pobrany identyfikator porownujemy z nastepnymi:
         - jesli ktorys sie zawiera w drugim, lub sa identyczne, to eliminujemy jeden,
           i sprawdzamy od nowa,
         - jesli wynika sprzecznosc w identyfikacji, to informujemy,
   */

   char temp[100];
   char wyraz1[100],wyraz2[100];
   char fleks1[100],fleks2[100];
   int nr1,nr2,ile,posneg1,posneg2;

   // pobieramy kolejne identyfikatory z ident,
   START:
   for (nr1=0,ile=glo_count_parts(handle,ident,',');nr1<ile-1;nr1++)
   {
      glo_get_part(ident,nr1+1,temp,',');
      if (temp[0]=='!') {posneg1=1;memmove(temp,temp+1,strlen(temp));}
      else posneg1=0;
      glo_get_part(temp,1,wyraz1,'+');
      glo_get_part(temp,2,fleks1,'+');

      // pobrany identyfikator porownujemy z nastepnymi:
      for (nr2=nr1+1;nr2<ile;nr2++)
      {
         glo_get_part(ident,nr2+1,temp,',');
         if (temp[0]=='!') {posneg2=1;memmove(temp,temp+1,strlen(temp));}
         else posneg2=0;
         glo_get_part(temp,1,wyraz2,'+');
         glo_get_part(temp,2,fleks2,'+');

         if (strcmp(wyraz1,wyraz2)==0)
         {
            if (posneg1==posneg2)
            {
               glo_delete_part_sep(handle,ident,nr1+1,',');
               goto START;
            }
            else
            {
               DeniseReport(handle,"sprzeczne identyfikatory: %s %s i %s %s",posneg1==0 ? "to" : "to nie",wyraz1,posneg2==0 ? "to" : "to nie",wyraz2);
               ident[0]='\0';
               return;
            }
         }

         // jesli ktorys sie zawiera w drugim, lub sa identyczne, to eliminujemy jeden,
         // i sprawdzamy od nowa,
         switch (log_cmp_kwantyfikatory(handle,fleks1,fleks2))
         {
            case FCMP_A_EQU_B:
               if (posneg1==posneg2)
               {
                  glo_delete_part_sep(handle,ident,nr1+1,',');
                  goto START;
               }
               else
               {
                  DeniseReport(handle,"sprzeczne identyfikatory: %s %s i %s %s",posneg1==0 ? "to" : "to nie",wyraz1,posneg2==0 ? "to" : "to nie",wyraz2);
                  ident[0]='\0';
                  return;
               }
               break;
            case FCMP_A_IN_B:
               // "ktos/marek"+"wszyscy"
               if (posneg1==0 && posneg2==0)
               {
                  glo_delete_part_sep(handle,ident,nr1+1,',');
                  goto START;
               }
               // "ktos/marek"+"NIE wszyscy"
               else if (posneg1==0 && posneg2==1)
               {
               }
               // "NIE ktos/marek"+"wszyscy"
               else if (posneg1==1 && posneg2==0)
               {
                  DeniseReport(handle,"sprzeczne identyfikatory: %s %s i %s %s",posneg1==0 ? "to" : "to nie",wyraz1,posneg2==0 ? "to" : "to nie",wyraz2);
                  ident[0]='\0';
                  return;
               }
               // "NIE ktos/marek"+"NIE wszyscy"
               else if (posneg1==1 && posneg2==1)
               {
                  glo_delete_part_sep(handle,ident,nr2+1,',');
                  goto START;
               }
               break;
            case FCMP_B_IN_A:
               // "wszyscy"+"ktos/marek"
               if (posneg1==0 && posneg2==0)
               {
                  glo_delete_part_sep(handle,ident,nr2+1,',');
                  goto START;
               }
               // "NIE wszyscy"+"ktos/marek"
               else if (posneg1==1 && posneg2==0)
               {
               }
               // "wszyscy"+"NIE ktos/marek"
               else if (posneg1==0 && posneg2==1)
               {
                  DeniseReport(handle,"sprzeczne identyfikatory: %s %s i %s %s",posneg1==0 ? "to" : "to nie",wyraz1,posneg2==0 ? "to" : "to nie",wyraz2);
                  ident[0]='\0';
                  return;
               }
               // "NIE wszyscy"+"NIE ktos/marek"
               else if (posneg1==1 && posneg2==1)
               {
                  glo_delete_part_sep(handle,ident,nr1+1,',');
                  goto START;
               }
               break;
            case FCMP_A_PR_B:
               // "marek"+"ktos"
               if (posneg1==0 && posneg2==0)
               {
                  glo_delete_part_sep(handle,ident,nr2+1,',');
                  goto START;
               }
               // "marek"+"NIE ktos"
               else if (posneg1==0 && posneg2==1)
               {
                  DeniseReport(handle,"sprzeczne identyfikatory: %s %s i %s %s",posneg1==0 ? "to" : "to nie",wyraz1,posneg2==0 ? "to" : "to nie",wyraz2);
                  ident[0]='\0';
                  return;
               }
               // "NIE marek"+"ktos"
               else if (posneg1==1 && posneg2==0)
               {
               }
               // "NIE marek"+"NIE ktos"
               else if (posneg1==1 && posneg2==1)
               {
                  glo_delete_part_sep(handle,ident,nr1+1,',');
                  goto START;
               }
               break;
            case FCMP_B_PR_A:
               // "ktos"+"marek"
               if (posneg1==0 && posneg2==0)
               {
                  glo_delete_part_sep(handle,ident,nr1+1,',');
                  goto START;
               }
               // "NIE ktos"+"marek"
               else if (posneg1==0 && posneg2==1)
               {
                  DeniseReport(handle,"sprzeczne identyfikatory: %s %s i %s %s",posneg1==0 ? "to" : "to nie",wyraz1,posneg2==0 ? "to" : "to nie",wyraz2);
                  ident[0]='\0';
                  return;
               }
               // "ktos"+"NIE marek"
               else if (posneg1==1 && posneg2==0)
               {
               }
               // "NIE ktos"+"NIE marek"
               else if (posneg1==1 && posneg2==1)
               {
                  glo_delete_part_sep(handle,ident,nr2+1,',');
                  goto START;
               }
               break;
         }
      }
   }
}

// ***************************************************************************************
// EXTERN            void ide_filter_identifiers(handle,ident,result,mode)
// ***************************************************************************************
void ide_filter_identifiers(struct DeniseHandle* handle,char *ident,char *result,int mode)
{
   /*
   mode:
   0 - nie zamieniaj znakow negacji '!' na wyraz "NIE"
   1 - zamieniaj, oraz stawiaj odstep zamiast przecinka
   2 - zamieniaj, oraz zostawiaj przecinka
   */

   char temp1[1000],temp2[1000],temp3[100];
   int nr1,ile;

   result[0]='\0';
   temp2[0]='\0';

   ile=glo_count_parts(handle,ident,',');
   for (nr1=0;nr1<ile;nr1++)
   {
      if (nr1>0)
      {
         sprintf(temp1,",");
         strcat(temp2,temp1);
      }
      glo_get_part(ident,nr1+1,temp1,',');
      glo_delete_part_sep(handle,temp1,2,'+');

      strcat(temp2,temp1);
   }

   if (mode==1 || mode==2)
   {
      // zamieniamy '!' na wyraz negujacy i dodajemy odstep po przecinku
      temp1[0]='\0';
      for (nr1=0;nr1<(int) strlen(temp2);nr1++)
      {
         if (temp2[nr1]=='!') sprintf(temp3,"nie ");
         else if (temp2[nr1]==',' && mode==1) sprintf(temp3," ");
         else if (temp2[nr1]==',' && mode==2) sprintf(temp3,", ");
         else sprintf(temp3,"%c",temp2[nr1]);
         strcat(temp1,temp3);
      }
      strcpy(result,temp1);
   }
   else strcpy(result,temp2);
}
