// ***************************************************************************************
//                                     LOGIC - EXPLANATION
// ***************************************************************************************

#include "denise.h"
#include "denise_protos.h"

// deklaracje prywatne
static void prove_expl_rek(struct DeniseHandle* handle,struct LogicClause *clause,int margines);
static void opis_klauzuli(struct DeniseHandle *handle,struct LogicClause *clause,char *dst);

// ***************************************************************************************
// EXTERN           void log_prove_explanation(DeniseHandle*,LogicClause *)
// ***************************************************************************************
void log_prove_explanation(struct DeniseHandle* handle,struct LogicClause *deduce)
{
   /*
   funkcja przedstawia przebieg wnioskowania,

   algorytm:
   - szukamy klauzuli pustej,
   - jesli dokladnie jedna z klauzul parent tej klauzuli jest hipoteza, to:
      - wypisujemy te klauzule, ktora nie jest hipoteza,
      - wypisujemy drzewko drugiej klauzuli parent jako wyjasnienie,
   - jesli zadna z klauzul nie jest hipoteza, to:
      - wypisujemy drzewko calej klauzuli pustej,
   */

   struct LogicClause *clause;
   char temp1[100];

   if (deduce==NULL) return;
   for (clause=deduce;clause->opis[0]!='\0';clause=clause->next);
   if (clause==NULL) return;

//DeniseMessage(handle,"CLAUSE: %s",clause->fs[0]->sentence->base_words);

   if ((clause->parent[0]->type==IS_FACT && clause->parent[1]->type==IS_HYPOTHESIS) ||
       (clause->parent[1]->type==IS_FACT && clause->parent[0]->type==IS_HYPOTHESIS))
   {
      opis_klauzuli(handle,clause->parent[clause->parent[0]->type==IS_FACT ? 0 : 1],temp1);
      DeniseMessage(handle,"%s %s ",UNN_GetLocaleString(handle,MSG_BECAUSE,"poniewaz"),temp1);
      prove_expl_rek(handle,clause->parent[clause->parent[0]->type==IS_FACT ? 0 : 1],0);
   }
   else
   {
	   opis_klauzuli(handle,clause,temp1);
	   DeniseMessage(handle,"%s %s",UNN_GetLocaleString(handle,MSG_BECAUSE,"poniewaz"),temp1);
       prove_expl_rek(handle,clause,0);
   }
}

static void prove_expl_rek(struct DeniseHandle* handle,struct LogicClause *clause,int margines)
{
   /*
   funkcja wyswietla drzewo wnioskowania
   */

   char spaces[50],temp1[1000];

   memset(spaces,' ',margines);
   spaces[margines]='\0';

   if (clause==NULL) return;
   if (clause->parent[0]==NULL && clause->parent[1]==NULL) return;

   DeniseMessage(handle,"%s%s",spaces,UNN_GetLocaleString(handle,MSG_IS_CONCLUSION,"co wynika z tego, ze"));

//   DeniseMessage(handle,"PARENT 0: %s",clause->parent[0]->opis);
   opis_klauzuli(handle,clause->parent[0],temp1);
   DeniseMessage(handle,"%s%s",spaces,temp1);
   prove_expl_rek(handle,clause->parent[0],margines+3);

   opis_klauzuli(handle,clause->parent[1],temp1);
   DeniseMessage(handle,"%s%s %s",spaces,UNN_GetLocaleString(handle,MSG_AND,"oraz tego, ze"),temp1);
   prove_expl_rek(handle,clause->parent[1],margines+3);
}

// ***************************************************************************************
// EXTERN           void log_result_explanation(DeniseHandle*,LogicClause *)
// ***************************************************************************************
void log_result_explanation(struct DeniseHandle* handle,struct LogicClause *deduce)
{
   /*
   funkcja przedstawia wynik wnioskowania,

   algorytm:
   - szukamy klauzuli pustej,
   - wypisujemy jedna z klauzul PARENT, ktora nie jest hipoteza,
     i najlepiej nie powstala z wnioskowania,
   */

   struct LogicClause *clause;
   char temp1[1000],score1,score2;

   if (deduce==NULL) return;
   for (clause=deduce;clause->opis[0]!='\0';clause=clause->next);
   if (clause==NULL) return;

   score1=0; score2=0;
   if (clause->parent[0]!=NULL)
   {
      score1+=1;
      if (clause->parent[0]->type==IS_FACT) score1+=2;
      if (clause->parent[0]->parent[0]==NULL) score1+=1;
   }

   if (clause->parent[1]!=NULL)
   {
      score2+=1;
      if (clause->parent[1]->type==IS_FACT) score2+=2;
      if (clause->parent[1]->parent[0]==NULL) score2+=1;
   }

   if (score1>=score2) opis_klauzuli(handle,clause->parent[0],temp1);
   else opis_klauzuli(handle,clause->parent[1],temp1);

   DeniseMessage(handle,"%s",temp1);
}

static void opis_klauzuli(struct DeniseHandle *handle,struct LogicClause *clause,char *dst)
{
	static struct Declension decl;
   struct Theme *fs_theme;
   char temp1[1000],temp2[1000];
   int nr1,subject,write_subject;

   sprintf(dst,"%s",UNN_GetLocaleString(handle,MSG_EMPTY_CLAUSE,"jesli nie, to powstanie klauzula pusta"));
   if (clause==NULL) return;

   temp1[0]='\0';

   if (clause==NULL) return;
   if (clause->parent[0]==NULL && clause->parent[1]==NULL)
   {
      if (clause->type==IS_FACT) sprintf(temp2,"%s ",UNN_GetLocaleString(handle,MSG_IS_FACT,"faktem jest, ze"));
      else if (clause->type==IS_HYPOTHESIS) sprintf(temp2,"%s ",UNN_GetLocaleString(handle,MSG_IS_HYPOTHESIS,"przyjeto hipoteze, ze"));
      strcat(temp1,temp2);
   }

   for (nr1=0;nr1<(int) strlen(clause->opis);nr1++)
   {
//DeniseMessage(handle,"REPORT: %c",clause->opis[nr1]);
	 if (clause->opis[nr1]>='A' && clause->opis[nr1]<='Z')
     {
		 // okreslamy podmiot
         fs_theme=ide_find_theme(handle,clause->fs[clause->opis[nr1]-'A']->databaseID,clause->fs[clause->opis[nr1]-'A']->theme_number);
         if (fs_theme==NULL)
		 {
//			 DeniseReport(handle,"blad! prosze to zglosic!!! LOGIC3.C FS_THEME==NULL");
			 sprintf(temp2,"<%s>",clause->fs[clause->opis[nr1]-'A']->sentence->base_words);
		 }
		 else
		 {
			if (fs_theme==handle->cache.user && handle->cache.user!=NULL)
			{
	         	subject=SUBJECT_USER;
				write_subject=(rand() % 10)<6 ? WRITE_WITHOUT_SUBJECT : (rand() % 3)==0 ? WRITE_WITH_SUBJECT : WRITE_WITH_PRONOUN;
			}
			else if (fs_theme==handle->cache.denise && handle->cache.denise!=NULL)
			{
	         	subject=SUBJECT_DENISE;
				write_subject=(rand() % 5)==0 ? WRITE_WITH_PRONOUN : WRITE_WITHOUT_SUBJECT;
			}
			else
			{
	         	subject=SUBJECT_OTHER;
				write_subject=WRITE_WITH_SUBJECT;
			}

     		// przepisujemy zdanie w postaci odmienionej
			if (gra_sent_declension(handle,clause->fs[clause->opis[nr1]-'A']->sentence,&decl,subject,write_subject)==TRUE)
					sprintf(temp2,"%s",decl.result[0]);
			else
			{
//					sprintf(temp2,"<%s>",clause->fs[clause->opis[nr1]-'A']->sentence->words);
				   sprintf(temp2,"<%s>",clause->fs[clause->opis[nr1]-'A']->sentence->base_words);
//				   DeniseMessage(handle,"!%s!????",clause->fs[clause->opis[nr1]-'A']->sentence->base_words);
	         }
		 }
     }
//	 continue;
	 else if (clause->opis[nr1]-'1'+1==CON_NOT) sprintf(temp2,"%s ",UNN_GetLocaleString(handle,MSG_NOT,"nie jest prawda, ze"));
     else if (clause->opis[nr1]-'1'+1==CON_OR) sprintf(temp2," %s ",UNN_GetLocaleString(handle,MSG_OR,"lub"));
     else if (clause->opis[nr1]-'1'+1==CON_AND) sprintf(temp2," %s ",UNN_GetLocaleString(handle,MSG_AND,"oraz "));
     else sprintf(temp2," %c ",clause->opis[nr1]);

//DeniseMessage(handle,"!!!!%s!!!!!",temp2);
     strcat(temp1,temp2);
   }

   if (strlen(temp1)>0)
   {
      strcpy(dst,temp1);
   }
}
