// ***************************************************************************************
//                                           STRUCT
// ***************************************************************************************

#ifndef D4_STR
#define D4_STR

struct DeniseAppHandle
{
	UNN_Window Window;
};

// parametry analizy skladniowej
struct SyntaxParams
{
   // katalizatory algorytmu
   int min_words; // min. liczba wyrazow w zdaniu: 1-99/OFF
   int min_groups; // min. liczba grup w zdaniu: 1-99/OFF
   int max_groups; // max. liczba grup w zdaniu: 1-99/OFF
   int max_fleks; // max. liczba wyrazow odseparowanych w wyrazie fleksyjnym: 1-99/OFF
   int max_words; // max. liczba wyrazow odseparowanych w grupie: 1-99/OFF
   int max_clauses; // max. liczba zdan pojedynczych w wypowiedzi: 1-99/OFF
   int first_clause; // zalozenie: zdanie pierwsze nigdy podrzedne,
   int subject_first; // zalozenie: podmiot stoi zawsze przed orzeczeniem,

   // katalizatory gramatyczne
   int min_prefix_len; // min. dlugosc zrostu,
   int prefix_check; // kontrola zrostow: ON/OFF
   int prefix_check_mode; // tryb kontroli zrostow: 0-PROSTY/1-PELNY
   int remember_first; // rozbior dokonywany do momentu znalezienia pierwszego zaakceptowanego: ON/OFF

   // parametry oceny gramatycznej
   int phrase_quality; // minimalna jakosc budowy pojedynczego wyrazenia (0-100%)
   int structure_quality; // minimalna jakosc oceny wszystkich zwiazkow miedzy wyrazeniami (0-100%)
   int function_quality; // minimalna jakosc oceny funkcji wszystkich wyrazen w zdaniu (0-100%)
};

// parametry wyswietlania informacji
struct ViewParams
{
	int words; // wyswietlanie informacji o przyjetym podzieleniu wyrazow
	int clause; // wyswietlanie kolejnych dokonywanych rozbiorow zdan
	int syntax; // wyswietlanie ostatecznego rozbioru calej wypowiedzi
	int syntax_time; // wyswietlanie czasu analizy gramatycznej

	int preliminary_analysis; // analiza wstepna w myth
	int theme_analysis; // analiza tematu w myth
	int interlocutory_analysis; // analiza konwersacyjna w myth
	int logic_analysis; // analiza logiczna w myth
	int fact_analysis; // opcjonalne zapamietywanie
	int conjunctions_analysis; // analiza spojnikow

	int unknown_words; // wyswietla slowa, ktorych nie odnaleziono w slowniku
};

// parametry zapisywania informacji w "dzienniku"
struct LogbookParams
{
	int words; // zapisywanie nieznanych slow
	int clauses; // zapisywanie zdan, ktorych nie udalo sie poprawnie zidentyfikowac
	int dialog; // zapisywanie tekstu z okna dialogu
   int reports; // zapisywanie tekstu z okna raportu
};

struct WordEditor
{
   struct WordEditor *next;
   char *data;
};

struct Fleksja
{
   struct Fleksja *next;
   char *podst;
   int ile_list;
   struct Odmiany *odm_head;
};

struct Odmiany
{
   struct Odmiany *next;
   char *opis;
   char *odmiany;
};

struct WordGenerator
{
   struct WordGenerator *next;
   char *chars; // koncowca wyrazu podstawowego (np. "izm")
   struct Odmiany *odmiany; // lista odmian
   struct WordsList *words; // lista wyrazow podstawowych
};

struct WordsList
{
   struct WordsList *next;
   char *word;
};

struct WordData
{
   char orig[100]; // wyraz odseparowany (tekst)
   int ile_mozliwosci; // ile mozliwych podzialow wyrazu na wyrazy jednostkowe (1 - 1 wyraz, bez zrostow)
   int zrost_poz[100]; // tablica podzialow - pozycja zrostu, (dlugosc wyrazu - jednolity wyraz)
   int zrost_nr; // aktualnie obowiazujacy numer podzialu
   int priority; // ktora czesc (zrost) wyrazu odseparowanego jest glowna, (0 - pierwsza czesc, 1 - druga) (domyslnie: pozostala czesc jest modyfikatorem),

   // aktualna fleksja wyrazu modyfikujacego w tym wyrazie modyfikujacym,
   int fleks_buff_nr;
   int fleks_nr;
};

struct Clause
{
   // jakosc analizy zdania: 0-100%
   int quality;
   int ctrl_numbers; // liczba punktow oceny jakosci
   // aktualna fleksja wyrazu fleksyjnego o danym numerze wyrazu fleksyjnego,
   int fleks_buff_nr[100]; // (numer_wyr_fleks-1 - liczenie od 0)
   int fleks_nr[100]; // (numer_wyr_fleks-1 - liczenie od 0)
	// aktualna fleksja wyrazenia (grupy) o danym numerze
	char group_fleks[20][20];
   // ...
   struct WordData *wyrazy[100]; // wskaznik do wyrazow odseparowanych w danym zdaniu
   int words_number; // ile wyrazow odseparowanych w zdaniu
   int ile_fleks; // ile wyrazow fleksyjnych
   int groups_number; // ile grup w zdaniu
   unsigned char fleksyjny[100]; // do ktorego wyrazu fleksyjnego nalezy dany wyraz odseparowany (numer od 1),
	unsigned char member[100]; // do ktorej grupy nalezy dany wyraz odseparowany (numer od 1),
	unsigned char parent[100]; // jaka grupa jest nadrzedna dla danej grupy (struktura grup) (numer od 1, 0 - brak nadrzednej)
	char parent_word[100]; // numer wybranego wyrazu fleksyjnego do ktorego grupa jest nadrzedna (numer od 1; 0 - do calej grupy)
   char function[100]; // funkcja danej grupy w zdaniu pojedynczym,
   // ...
   char posneg[100]; // czy grupa jest twierdzaca, czy przeczaca
};

struct AcceptedClause
{
   // jakosc analizy zdania: 0-100%
   int quality;
   int ctrl_numbers; // liczba punktow oceny jakosci
	// aktualna fleksja wyrazenia (grupy) o danym numerze
	char group_fleks[20][20];
   // aktualna fleksja wyrazu fleksyjnego o danym numerze wyrazu fleksyjnego,
   struct
   {
      char wyraz[100];

      // dane fleksyjne
      struct Fleksja *fleks;
      struct Odmiany *odm;
      int poz; // pozycja odmiany na liscie odmian
   } voc_data[100]; // tablica zaakceptowanych danych fleksyjnych (numer_wyr_fleks-1 - liczenie od 0)
   // ...
   struct WordData *wyrazy[100]; // wskaznik do wyrazow odseparowanych w danym zdaniu
   int words_number; // ile wyrazow odseparowanych w zdaniu
   int ile_fleks; // ile wyrazow fleksyjnych
   int groups_number; // ile grup w zdaniu
   char fleksyjny[100]; // do ktorego wyrazu fleksyjnego nalezy dany wyraz odseparowany (numer od 1),
   char member[100]; // do ktorej grupy nalezy dany wyraz odseparowany (numer od 1),
   char parent[100]; // jaka grupa jest nadrzedna dla danej grupy (struktura grup) (numer od 1, 0 - brak nadrzednej)
	char parent_word[100]; // numer wybranego wyrazu fleksyjnego do ktorego grupa jest nadrzedna (numer od 1; 0 - do calej grupy)
   char function[100]; // funkcja danej grupy w zdaniu pojedynczym,
   // ...
   char posneg[100]; // czy grupa jest twierdzaca, czy przeczaca
};

struct Syntax
{
   char orig[200]; // wprowadzona wypowiedz, oryginal
   char busy[200]; // wypowiedz "pusta"

   int input_type; // typ wypowiedzi: oznajmujaca/pytajaca - dotyczy CALEJ WYPOWIEDZI
   int prefered_output; // preferowany typ odpowiedzi (dowod, wniosek, dowolny)

   int words_number; // liczba wyrazow odseparowanych,
   struct WordData words[100]; // tablica z danymi o wyrazach odseparowanych

   int clause_number; // aktualna liczba zdan pojedynczych
   int clause_tree[100]; // drzewo zdan (gramatyczna struktura zdan)
   int logic_tree[100]; // drzewo logiczne (logiczna struktura zdan)

   int buffer_len;
   struct
   {
      char wyraz[100];
      int fleks_ile;

      // dane fleksyjne
      struct Fleksja *fleks[100];
      struct Odmiany *odm[100];
      int poz[100]; // pozycja odmiany na liscie odmian
   } voc_buffer[100]; // tablica danych fleksyjnych (tablice wewnatrz - zbior odmian)

   // struktura zawiera ORYGINALNE dane n. t. wyrazow w kazdym zdaniu
   // dane te sa kopiowane do 'clause', lecz tam dochodza modyfikacje, i informacja jest tracona,
   // dlatego wymagane jest potem jej odtworzenie w postaci oryginalnej,
   struct
   {
      struct WordData *wyrazy[100]; // wskaznik do wyrazow odseparowanych w danym zdaniu
      int words_number; // ile wyrazow odseparowanych w zdaniu
   } podzial[10]; // tablica zdan

   struct Clause clause[10]; // tablica zdan
   struct AcceptedClause accepted[10][10]; // zaakceptowane WSZYSTKIE rozbiory poszczegolnych zdan [zdanie][rozbior]
	int ile_accepted[10]; // liczba zaakceptowanych rozbiorow danego zdania
	int active[10]; // aktualny numer rozbioru (od 0) kazdego zdania 
};

// struktura odmiany zdania
struct DeclensionWord
{
	char podst[100];

	// tablice podreczne modulu odmiany
	int fleks_number;
	struct Fleksja *fleks[DECLENSION_MAX_FLEKS];
	struct Odmiany *odm[DECLENSION_MAX_FLEKS];
	int poz[DECLENSION_MAX_FLEKS];

	int utility[DECLENSION_MAX_FLEKS]; // uzytecznosc odmiany - tablica podreczna modulu odmiany
	int selected; // wybrany zestaw - zmienna podreczna modulu odmiany
};

struct Declension
{
	int subject; // jaki podmiot - denise, user, temat, inny
   int write_subject; // czy w zdaniu ma byc podmiot podany, czy domyslny, czy zaimek

	int groups_number;
	struct
	{
		int function;
		int parent;

		int types_number; // liczba zwiazkow - wypelnia modul odmiany
		struct GrammarFuncData *type[500]; // zwiazki - wypelnia modul odmiany
		int utility[500]; // uzytecznosc zwiazku - tablica podreczna modulu odmiany
		int selected; // wybrany opis- zmienna podreczna modulu odmiany

		int words_number;
		struct DeclensionWord word[10];
	} group[20];

	// wypelnia modul odmiany
	int person;
   int results_number;
	char result[50][200]; // umieszczone sa tutaj wszystkie mozliwosci odmiany uznane za poprawne
};

struct GrammarData
{
   struct GrammarData *next;
   char *data;
};

struct GrammarFuncData
{
   struct GrammarFuncData *next;
   char *data;
   char *func;
   char *memb_nadrz;
   char *memb_podrz;
   char *zakaz_nadrz;
   char *zakaz_podrz;
   int kolejnosc; // 0 - brak informacji
   char *zakaz_miedzy_fun; // funkcja
   char *zakaz_miedzy_fle; // fleksja
};

struct GrammarExtData
{
   struct GrammarExtData *next;
   char *data;
   int parts_number;
   char *zakaz_miedzy;
	char *fleksja_wynikowa;
};

struct GrammarLo5Data
{
   struct GrammarLo5Data *next;
   char *czesc_mowy;
   int func;
   char *requied;
   char *banned;
};

struct GrammarLo6Data
{
   struct GrammarLo6Data *next;
   char *czesc_mowy;
   int func;
   int person;
};

struct GrammarLo8Data
{
   struct GrammarLo8Data *next;
   char *cmp_data;
   int person;
	int type; // 1 - osobowy, 2 - dzierzawczy
};

struct GrammarLo9Data
{
//   struct GrammarLo9Data *next;
   char *mean1; // znaki poczatku wyjasnienia: 'tekst','tekst'...
   char *mean2; // znaki drugiej czesci wyjasnienia: 'tekst','tekst'...
   char *concl1; // znaki poczatku warunkow: 'tekst','tekst'...
   char *concl2; // znaki poczatku wniosku: 'tekst','tekst'...
};

struct FilterData
{
   struct FilterData *next;
   char *find;
   char *replace;
   int operation;
   int range;
};

struct Language
{
   char *name; // nazwa jezyka

   // FLEKSJA
   struct WordEditor *wordeditor;
   struct Fleksja *vocabulary;
   struct Fleksja *voctemp;
   struct WordGenerator *wordgenerator;

   // SKLADNIA - GRAMATYKA
   struct GrammarData *gr1; // zrosty
   char gr2[100]; // wyrazy samodzielne
   struct GrammarExtData *gr3[100]; // wyrazy niesamodzielne z samodzielnymi
   struct GrammarExtData *gr4[100]; // szeregi wyrazow samodzielnych
   struct GrammarExtData *gr5[100]; // wyrazy samodzielne roznych czesci mowy
   struct GrammarData *gr6[5]; // funkcje wyrazen
   struct GrammarFuncData *gr7[100]; // zwiazki miedzy wyrazeniami (tablica nadrzednych czesci mowy:
                                     // indeksy zgodnie z WED pomniejszone o 1)
   // GRAMATYKA -> LOGIKA
   struct GrammarData *lo1; // wyrazy negujace grupe
   struct GrammarData *lo2[4]; // laczniki w zdaniach wspolrzednych (0-AND,1-XOR,2-OR)
   struct GrammarData *lo3[4]; // kwantyfikatory logiczne (0-ALL,1-EXIST,2-NO_EXIST,3-NO_EXIST_NO)
   BOOL lo3_voc_filter[100]; // tablica z informacja, czy dana czesc mowy moze byc kwantyfikatorem (znacznie przyspiesza prace programu)
   struct GrammarData *lo4[4]; // wyrazy wymagajace szczegolnego rodzaju odpowiedzi (0-PROVE_WITH_CON,1-PROVE_WITHOUT_CON,2-RESULT_WITH_CON,3-RESULT_WITHOUT_CON)
   struct GrammarLo5Data *lo5; // wyrazy bedace identyfikatorami tematu
   struct GrammarLo6Data *lo6; // opis metod ekstrakcji osoby gramatycznej ze zdania
   struct GrammarData *lo7; // wyrazy wymagajace scislej identyfikacji - nazwy wlasne
   struct GrammarLo8Data *lo8; // zasady identyfikacji zaimkw rzeczownych (osobowych i dzierawczych)
   struct GrammarLo9Data *lo9; // definicje

   // FILTRY
   struct FilterData *fil;
};

// struktura faktu prostego zdania pojedynczego
struct FSentence
{
   char *language; // nazwa jezyka, w ktorym jest zdanie

   char *id_subject; // identyfikatory faktu (ident1|ident2...) pobrane z tematu
   char *id_global; // identyfikatory globalne z innych grup
   char *orig; // wpisane zdanie w formie oryginalnej
   char *words; // wpisane zdanie w formie przefiltrowanej
   char *base_words; // wyrazy podstawowe ulozone wyrazami fleksyjnymi (!aa a,b b+b,c c c)
   char *voc; // dane fleksyjne kolejnych wyrazow (glownych, nie modyfikatorow) fleksyjnych (AABA,ABAD,BAA)
	char *group_voc; // dane fleksyjne kolejnych grup (wyrazen), jesli takie dane istnieja

   char *zrost_poz; // tablica podzialow - pozycja zrostu, (0 - brak zrostu, jednolity wyraz)
   char *priority; // ktora czesc (zrost) wyrazu odseparowanego jest glowna, (0 - pierwsza czesc, 1 - druga) (domyslnie: pozostala czesc jest modyfikatorem),

   char *fleksyjny; // do ktorego wyrazu fleksyjnego nalezy dany wyraz odseparowany (numer od 1),
   char *member; // do ktorej grupy nalezy dany wyraz odseparowany (numer od 1),
   char *parent; // jaka grupa jest nadrzedna dla danej grupy (struktura grup) (numer od 1, 0 - brak nadrzednej)
	char *parent_word; // numer wybranego wyrazu fleksyjnego do ktorego grupa jest nadrzedna (numer od 1; 0 - do calej grupy)
   char *function; // funkcja danej grupy w zdaniu,

   char *posneg; // informacja o typie grupy: TWIERDZENIE/PRZECZENIE

   // zmienne podreczne
   int words_number;
   int ile_fleks;
   int groups_number;
};

// struktura faktu prostego
struct FactSimple
{
   struct FactSimple *next;
   int quality; // jakosc (analizy) danych zawartych w fakcie prostym
   char databaseID[30]; // unikalny numer identyfikacyjny bazy faktow, w ktorym jest temat, do ktorego nalezy FS,
   int theme_number; // numer tematu w bazie (databaseID), do ktorego nalezy fakt prosty
   struct FSentence *sentence; // informacja w postaci zdania
};

// struktura faktu (uwiklanego)
struct Fact
{
   struct Fact *next;
   char time[30]; // czas zapamietania faktu
   int ile_klauzul; // liczba klauzul
   char **logic; // zbior klauzul opisujacych fakt
   struct FactSimple *fs; // fakty proste danego faktu
};

// struktura tematu w bazie faktow
struct Theme
{
   struct Theme *next;
   char *identify; // slowa identyfikujace temat
   struct FactDatabase *database; // baza danych, w ktorej wystepuje temat
   int number; // numer identyfikacyjny tematu
   char fwd_databaseID[30]; // przekierowanie na inny temat w bazie o takim ID
   int fwd_number; // numer tematu w bazie, na ktora dokonano przekierowania
};

// struktura bazy faktow
struct FactDatabase
{
   struct FactDatabase *next; // inne bazy faktow
   char *mode_name1; // nazwy trybow
   char *mode_name2; // w postaci takiej,
   char *mode_name3; // jaka wpisano w bazie,
   char databaseID[30]; // unikalny numer identyfikacyjny bazy faktow
   int new_themes; // 1-ACCEPT_NEW_THEMES, 2-NO_NEW_THEMES
   int new_facts; // 1-ACCEPT_NEW_FACTS, 2-NO_NEW_FACTS
   int fwd_themes; // 1-ACCEPT_FORWARD_THEMES, 2-NO_FORWARD_THEMES
   struct Theme *themes; // tematy w bazie faktow
   struct Fact *facts; // lista faktow w bazie
   char *name; // nazwa bazy danych
};

// klauzula logiczna
struct LogicClause
{
   struct LogicClause *next; // kolejna klauzula

   char opis[20]; // opis klauzuli w oparciu o fs: A21B... A=0, B=1 w tablicy fs
   struct FactSimple *fs[10]; // wskazniki na fs wykorzystane w opisie
   struct LogicClause *parent[2]; // wskazniki klauzul, z ktorych wynika klauzula (NULL jesli nie powstala na drodze wnioskowania)
   int type; // IS_FACT - oznacza fakt, IS_HYPOTHESIS - hipoteza
};

// struktura zwracana przez funkcje analizy logicznej
struct LogicAnalysis
{
   int logic_state; // LOGIC_TRUE, LOGIC_FALSE
   BOOL neg; // jesli fakt jest SPRZECZNY to TRUE, inaczej FALSE
   struct LogicClause *deduce; // wskaznik do klauzul
};

struct FScmp
{
   int fscmp;
   BOOL neg;
};

// struktura baz logicznych
struct LogicDatabase
{
   struct FactsCMPData *factscmp; // tablica przejsc w porownywaniu faktow
	struct ConvData *conversation; // tablica przejsc dla konwersacji
};

struct FactsCMPData
{
   struct FactsCMPData *next;
   int state;
   int posnegA;
   int posnegB;
   int cmp_res;
   int new_state;
   int operation;
   int groups; // jakie typy grup obejmuje
};

// mozna pozniej zamienic na wskazniki i alokowac pamiec
struct ConvData
{
	struct ConvData *next;
	char conditions[200]; // warunki
	char action[200]; // czynnosci do wykonania
};

struct ThemesList
{
	struct ThemesList *next;
	struct Theme *theme; // wskaznik na temat
};

// struktura bazy znaczen tozsamosciowych A=B
struct Meaning
{
	struct Meaning *next;

	char **basisA; // wyrazy podstawowe sekcji A
	int basisA_number; // liczba wyrazow w tablicy

	char **basisB; // j. w. dla sekcji B
	int basisB_number;
};

struct CacheMemory
{
	struct LogicClause *last_list; // ostatnia lista klauzul

	BOOL identify_mode; // tryb identyfikacji
	BOOL explain_mode; // tryb wyjasniania

	struct Fact *temp_fact; // fakt, ktory jest pamietany w celu lepszej identyfikacji
	struct Fact *facts; // lista faktow pamietana po rozbiorze zdania 
	char exp_wyrazenie[2][100]; // wersje odpowiedzi (mozliwosci)
	struct Fact *exp_fact[2]; // wskazniki na fakty odpowiednie dla mozliwosci

	struct Theme *user; // zidentyfikowany obiekt rozmowcy
	struct Theme *denise; // obiekt Denise
	struct Theme *theme; // obiekt aktualnego tematu
};

#endif
